@@ -14,13 +14,13 @@ public:
 	COpData(COpData const&) = delete;
 	COpData& operator=(COpData const&) = delete;
 
-	int opState;
+	int opState{};
 	Command const opId;
 
-	bool waitForAsyncRequest;
-	bool holdsLock;
+	bool waitForAsyncRequest{};
+	bool holdsLock{};
 
-	COpData *pNextOpData;
+	COpData *pNextOpData{};
 };
 
 class CConnectOpData : public COpData
@@ -28,21 +28,23 @@ class CConnectOpData : public COpData
 public:
 	CConnectOpData()
 		: COpData(Command::connect)
-		, port(0)
 	{
 	}
 
-	wxString host;
-	unsigned int port;
+	std::wstring host;
+	unsigned int port{};
 };
 
+class wxMBConv;
+class wxCSConv;
+
 class CFileTransferOpData : public COpData
 {
 public:
-	CFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path);
-	virtual ~CFileTransferOpData();
+	CFileTransferOpData(bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path);
+	virtual ~CFileTransferOpData() = default;
 	// Transfer data
-	wxString localFile, remoteFile;
+	std::wstring localFile, remoteFile;
 	CServerPath remotePath;
 	const bool download;
 
@@ -71,7 +73,7 @@ public:
 	CServerPath path;
 	CServerPath currentPath;
 	CServerPath commonParent;
-	std::vector<wxString> segments;
+	std::vector<std::wstring> segments;
 };
 
 class CChangeDirOpData : public COpData
@@ -80,8 +82,6 @@ public:
 	CChangeDirOpData()
 		: COpData(Command::cwd)
 	{
-		tryMkdOnFail = false;
-		link_discovery = false;
 	}
 
 	virtual ~CChangeDirOpData()
@@ -89,11 +89,11 @@ public:
 	}
 
 	CServerPath path;
-	wxString subDir;
-	bool tryMkdOnFail;
+	std::wstring subDir;
+	bool tryMkdOnFail{};
 	CServerPath target;
 
-	bool link_discovery;
+	bool link_discovery{};
 };
 
 enum class TransferEndReason
@@ -123,13 +123,13 @@ public:
 	virtual int Connect(const CServer &server) = 0;
 	virtual int Disconnect();
 	virtual void Cancel();
-	virtual int List(CServerPath path = CServerPath(), wxString subDir = wxString(), int flags = 0);
-	virtual int FileTransfer(const wxString localFile, const CServerPath &remotePath,
-							 const wxString &remoteFile, bool download,
-							 const CFileTransferCommand::t_transferSettings& transferSettings);
-	virtual int RawCommand(const wxString& command = wxString());
-	virtual int Delete(const CServerPath& path, std::deque<wxString>&& files);
-	virtual int RemoveDir(const CServerPath& path = CServerPath(), const wxString& subDir = wxString());
+	virtual int List(CServerPath path = CServerPath(), std::wstring const& subDir = std::wstring(), int flags = 0);
+	virtual int FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
+							 std::wstring const& remoteFile, bool download,
+							 CFileTransferCommand::t_transferSettings const& transferSettings);
+	virtual int RawCommand(std::wstring const& command = std::wstring());
+	virtual int Delete(const CServerPath& path, std::deque<std::wstring>&& files);
+	virtual int RemoveDir(CServerPath const& path = CServerPath(), std::wstring const& subDir = std::wstring());
 	virtual int Mkdir(const CServerPath& path);
 	virtual int Rename(const CRenameCommand& command);
 	virtual int Chmod(const CChmodCommand& command);
@@ -148,10 +148,10 @@ public:
 	const CServer* GetCurrentServer() const;
 
 	// Conversion function which convert between local and server charset.
-	wxString ConvToLocal(const char* buffer, size_t len);
-	wxChar* ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen);
-	wxChar* ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen);
-	wxCharBuffer ConvToServer(const wxString& str, bool force_utf8 = false);
+	std::wstring ConvToLocal(const char* buffer, size_t len);
+	wchar_t* ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen);
+	wchar_t* ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen);
+	std::string ConvToServer(std::wstring const&, bool force_utf8 = false);
 
 	void SetActive(CFileZillaEngine::_direction direction);
 
@@ -174,7 +174,7 @@ protected:
 	fz::duration GetTimezoneOffset() const;
 
 	virtual int DoClose(int nErrorCode = FZ_REPLY_DISCONNECTED);
-	bool m_closed;
+	bool m_closed{};
 
 	virtual int ResetOperation(int nErrorCode);
 
@@ -185,23 +185,22 @@ protected:
 	// Called by ResetOperation if there's a queued operation
 	virtual int ParseSubcommandResult(int prevResult);
 
-	wxString ConvertDomainName(wxString const& domain);
+	std::wstring ConvertDomainName(std::wstring const& domain);
 
 	int CheckOverwriteFile();
 
-	void CreateLocalDir(const wxString &local_file);
+	void CreateLocalDir(std::wstring const& local_file);
 
-	bool ParsePwdReply(wxString reply, bool unquoted = false, const CServerPath& defaultPath = CServerPath());
+	bool ParsePwdReply(std::wstring reply, bool unquoted = false, const CServerPath& defaultPath = CServerPath());
 
-	COpData *m_pCurOpData;
-	int m_nOpState;
+	COpData *m_pCurOpData{};
 	CFileZillaEnginePrivate & engine_;
-	CServer *m_pCurrentServer;
+	CServer *m_pCurrentServer{};
 
 	CServerPath m_CurrentPath;
 
-	wxCSConv *m_pCSConv;
-	bool m_useUTF8;
+	wxCSConv *m_pCSConv{};
+	bool m_useUTF8{};
 
 	// Timeout data
 	fz::timer_id m_timer{};
@@ -223,8 +222,8 @@ protected:
 	// SendNextCommand will be called once the lock gets available
 	// and engine could obtain it.
 	// Lock is recursive. Lock counter increases on suboperations.
-	bool TryLockCache(enum locking_reason reason, const CServerPath& directory);
-	bool IsLocked(enum locking_reason reason, const CServerPath& directory);
+	bool TryLockCache(locking_reason reason, const CServerPath& directory);
+	bool IsLocked(locking_reason reason, const CServerPath& directory);
 
 	// Unlocks the cache. Can be called if not holding the lock
 	// Doesn't need reason as one engine can at most hold one lock
@@ -234,7 +233,7 @@ protected:
 	// Returns reason != unknown iff engine is the first waiting engine
 	// and obtains the lock.
 	// On failure, the engine was not waiting for a lock.
-	enum locking_reason ObtainLockFromEvent();
+	locking_reason ObtainLockFromEvent();
 
 	bool IsWaitingForLock();
 
@@ -242,7 +241,7 @@ protected:
 	{
 		CControlSocket* pControlSocket;
 		CServerPath directory;
-		enum locking_reason reason;
+		locking_reason reason;
 		bool waiting;
 		int lockcount;
 	};
@@ -254,7 +253,7 @@ protected:
 	// End cache locking stuff
 	// -----------------------
 
-	bool m_invalidateCurrentPath;
+	bool m_invalidateCurrentPath{};
 
 	virtual void operator()(fz::event_base const& ev);
 
@@ -276,11 +275,11 @@ public:
 
 protected:
 	virtual int DoClose(int nErrorCode = FZ_REPLY_DISCONNECTED);
-	void ResetSocket();
+	virtual void ResetSocket();
 
 	virtual void operator()(fz::event_base const& ev);
 	void OnSocketEvent(CSocketEventSource* source, SocketEventType t, int error);
-	void OnHostAddress(CSocketEventSource* source, wxString const& address);
+	void OnHostAddress(CSocketEventSource* source, std::string const& address);
 
 	virtual void OnConnect();
 	virtual void OnReceive();
