@@ -1,14 +1,10 @@
-// FileZillaEngine.cpp: Implementierung der Klasse CFileZillaEngine.
-//
-//////////////////////////////////////////////////////////////////////
-
 #include <filezilla.h>
 #include "ControlSocket.h"
 #include "directorycache.h"
 #include "engineprivate.h"
 
-CFileZillaEngine::CFileZillaEngine(CFileZillaEngineContext& engine_context)
-	: impl_(new CFileZillaEnginePrivate(engine_context, *this))
+CFileZillaEngine::CFileZillaEngine(CFileZillaEngineContext& engine_context, EngineNotificationHandler& notificationHandler)
+	: impl_(new CFileZillaEnginePrivate(engine_context, *this, notificationHandler))
 {
 }
 
@@ -17,11 +13,6 @@ CFileZillaEngine::~CFileZillaEngine()
 	delete impl_;
 }
 
-int CFileZillaEngine::Init(wxEvtHandler *pEventHandler)
-{
-	return impl_->Init(pEventHandler);
-}
-
 int CFileZillaEngine::Execute(const CCommand &command)
 {
 	return impl_->Execute(command);
@@ -42,7 +33,7 @@ bool CFileZillaEngine::IsPendingAsyncRequestReply(std::unique_ptr<CAsyncRequestN
 	return impl_->IsPendingAsyncRequestReply(pNotification);
 }
 
-bool CFileZillaEngine::IsActive(enum CFileZillaEngine::_direction direction)
+bool CFileZillaEngine::IsActive(CFileZillaEngine::_direction direction)
 {
 	return CFileZillaEnginePrivate::IsActive(direction);
 }
