@@ -1,5 +1,7 @@
 #include <filezilla.h>
 
+#include <libfilezilla/format.hpp>
+
 std::wstring CDirentry::dump() const
 {
 	std::wstring str = fz::sprintf(L"name=%s\nsize=%d\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nunsure=%d\n",
@@ -17,20 +19,25 @@ std::wstring CDirentry::dump() const
 
 bool CDirentry::operator==(const CDirentry &op) const
 {
-	if (name != op.name)
+	if (name != op.name) {
 		return false;
+	}
 
-	if (size != op.size)
+	if (size != op.size) {
 		return false;
+	}
 
-	if (permissions != op.permissions)
+	if (permissions != op.permissions) {
 		return false;
+	}
 
-	if (ownerGroup != op.ownerGroup)
+	if (ownerGroup != op.ownerGroup) {
 		return false;
+	}
 
-	if (flags != op.flags)
+	if (flags != op.flags) {
 		return false;
+	}
 
 	if (has_date()) {
 		if (time != op.time) {
@@ -43,15 +50,13 @@ bool CDirentry::operator==(const CDirentry &op) const
 
 const CDirentry& CDirectoryListing::operator[](unsigned int index) const
 {
-	// Commented out, too heavy speed penalty
-	// wxASSERT(index < m_entryCount);
 	return *(*m_entries)[index];
 }
 
-CDirentry& CDirectoryListing::operator[](unsigned int index)
+CDirentry& CDirectoryListing::get(unsigned int index)
 {
 	// Commented out, too heavy speed penalty
-	// wxASSERT(index < m_entryCount);
+	// assert(index < m_entryCount);
 	return m_entries.get()[index].get();
 }
 
@@ -64,12 +69,15 @@ void CDirectoryListing::Assign(std::deque<fz::shared_value<CDirentry>> &entries)
 	m_flags &= ~(listing_has_dirs | listing_has_perms | listing_has_usergroup);
 
 	for (auto & entry : entries) {
-		if (entry->is_dir())
+		if (entry->is_dir()) {
 			m_flags |= listing_has_dirs;
-		if (!entry->permissions->empty())
+		}
+		if (!entry->permissions->empty()) {
 			m_flags |= listing_has_perms;
-		if (!entry->ownerGroup->empty())
+		}
+		if (!entry->ownerGroup->empty()) {
 			m_flags |= listing_has_usergroup;
+		}
 		own_entries.emplace_back(std::move(entry));
 	}
 
@@ -79,18 +87,21 @@ void CDirectoryListing::Assign(std::deque<fz::shared_value<CDirentry>> &entries)
 
 bool CDirectoryListing::RemoveEntry(unsigned int index)
 {
-	if (index >= GetCount())
+	if (index >= GetCount()) {
 		return false;
+	}
 
 	m_searchmap_case.clear();
 	m_searchmap_nocase.clear();
 
 	std::vector<fz::shared_value<CDirentry> >& entries = m_entries.get();
 	std::vector<fz::shared_value<CDirentry> >::iterator iter = entries.begin() + index;
-	if ((*iter)->is_dir())
+	if ((*iter)->is_dir()) {
 		m_flags |= CDirectoryListing::unsure_dir_removed;
-	else
+	}
+	else {
 		m_flags |= CDirectoryListing::unsure_file_removed;
+	}
 	entries.erase(iter);
 
 	return true;
@@ -99,11 +110,12 @@ bool CDirectoryListing::RemoveEntry(unsigned int index)
 void CDirectoryListing::GetFilenames(std::vector<std::wstring> &names) const
 {
 	names.reserve(GetCount());
-	for (unsigned int i = 0; i < GetCount(); ++i)
+	for (unsigned int i = 0; i < GetCount(); ++i) {
 		names.push_back((*m_entries)[i]->name);
+	}
 }
 
-int CDirectoryListing::FindFile_CmpCase(const wxString& name) const
+int CDirectoryListing::FindFile_CmpCase(std::wstring const& name) const
 {
 	if (!m_entries || m_entries->empty()) {
 		return -1;
@@ -114,9 +126,10 @@ int CDirectoryListing::FindFile_CmpCase(const wxString& name) const
 	}
 
 	// Search map
-	auto iter = m_searchmap_case->find(to_wstring(name));
-	if (iter != m_searchmap_case->end())
+	auto iter = m_searchmap_case->find(name);
+	if (iter != m_searchmap_case->end()) {
 		return iter->second;
+	}
 
 	unsigned int i = m_searchmap_case->size();
 	if (i == m_entries->size()) {
@@ -131,28 +144,32 @@ int CDirectoryListing::FindFile_CmpCase(const wxString& name) const
 		std::wstring const& entry_name = (*entry_iter)->name;
 		searchmap_case.insert(std::pair<std::wstring const, unsigned int>(entry_name, i));
 
-		if (entry_name == name)
+		if (entry_name == name) {
 			return i;
+		}
 	}
 
 	// Map is complete, item not in it
 	return -1;
 }
 
-int CDirectoryListing::FindFile_CmpNoCase(wxString name) const
+int CDirectoryListing::FindFile_CmpNoCase(std::wstring const& name) const
 {
-	if (!m_entries || m_entries->empty())
+	if (!m_entries || m_entries->empty()) {
 		return -1;
+	}
 
-	if (!m_searchmap_nocase)
+	if (!m_searchmap_nocase) {
 		m_searchmap_nocase.get();
+	}
 
-	name.MakeLower();
+	std::wstring lwr = fz::str_tolower(name);
 
 	// Search map
-	auto iter = m_searchmap_nocase->find(to_wstring(name));
-	if (iter != m_searchmap_nocase->end())
+	auto iter = m_searchmap_nocase->find(lwr);
+	if (iter != m_searchmap_nocase->end()) {
 		return iter->second;
+	}
 
 	unsigned int i = m_searchmap_nocase->size();
 	if (i == m_entries->size()) {
@@ -164,12 +181,12 @@ int CDirectoryListing::FindFile_CmpNoCase(wxString name) const
 	// Build map if not yet complete
 	std::vector<fz::shared_value<CDirentry> >::const_iterator entry_iter = m_entries->begin() + i;
 	for (; entry_iter != m_entries->end(); ++entry_iter, ++i) {
-		wxString entry_name = (*entry_iter)->name;
-		entry_name.MakeLower();
-		searchmap_nocase.insert(std::pair<std::wstring const, unsigned int>(to_wstring(entry_name), i));
+		std::wstring entry_lrw = fz::str_tolower((*entry_iter)->name);
+		searchmap_nocase.insert(std::pair<std::wstring const, unsigned int>(entry_lrw, i));
 
-		if (entry_name == name)
+		if (entry_lrw == lwr) {
 			return i;
+		}
 	}
 
 	// Map is complete, item not in it
@@ -178,8 +195,9 @@ int CDirectoryListing::FindFile_CmpNoCase(wxString name) const
 
 void CDirectoryListing::ClearFindMap()
 {
-	if (!m_searchmap_case)
+	if (!m_searchmap_case) {
 		return;
+	}
 
 	m_searchmap_case.clear();
 	m_searchmap_nocase.clear();
@@ -188,4 +206,4 @@ void CDirectoryListing::ClearFindMap()
 void CDirectoryListing::Append(CDirentry&& entry)
 {
 	m_entries.get().emplace_back(entry);
-}
\ No newline at end of file
+}
