@@ -50,7 +50,7 @@ namespace listingEncoding
 class CDirectoryListingParser final
 {
 public:
-	CDirectoryListingParser(CControlSocket* pControlSocket, const CServer& server, listingEncoding::type encoding = listingEncoding::unknown, bool sftp_mode = false);
+	CDirectoryListingParser(CControlSocket* pControlSocket, const CServer& server, listingEncoding::type encoding = listingEncoding::unknown);
 	~CDirectoryListingParser();
 
 	CDirectoryListingParser(CDirectoryListingParser const&) = delete;
@@ -59,7 +59,7 @@ public:
 	CDirectoryListing Parse(const CServerPath &path);
 
 	bool AddData(char *pData, int len);
-	bool AddLine(const wchar_t* pLine);
+	bool AddLine(std::wstring && line, std::wstring && name, fz::datetime const& time);
 
 	void Reset();
 
@@ -72,7 +72,7 @@ protected:
 
 	bool ParseData(bool partial);
 
-	bool ParseLine(CLine &line, const enum ServerType serverType, bool concatenated);
+	bool ParseLine(CLine &line, ServerType const serverType, bool concatenated, CDirentry const* override = 0);
 
 	bool ParseAsUnix(CLine &line, CDirentry &entry, bool expect_date);
 	bool ParseAsDos(CLine &line, CDirentry &entry);
@@ -128,7 +128,7 @@ protected:
 	int m_currentOffset;
 
 	std::deque<t_list> m_DataList;
-	std::deque<CRefcountObject<CDirentry>> m_entryList;
+	std::deque<fz::shared_value<CDirentry>> m_entryList;
 	int64_t m_totalData;
 
 	CLine *m_prevLine;
@@ -136,15 +136,13 @@ protected:
 	CServer m_server;
 
 	bool m_fileListOnly;
-	std::vector<wxString> m_fileList;
+	std::vector<std::wstring> m_fileList;
 
 	bool m_maybeMultilineVms;
 
 	fz::duration m_timezoneOffset;
 
 	listingEncoding::type m_listingEncoding;
-
-	bool sftp_mode_{};
 };
 
 #endif
