@@ -8,6 +8,7 @@
 #include "socket.h"
 
 #include <libfilezilla/event_loop.hpp>
+#include <libfilezilla/thread_pool.hpp>
 
 namespace {
 struct logging_options_changed_event_type;
@@ -50,6 +51,8 @@ public:
 		, optionChangeHandler_(options, loop_)
 	{
 		CLogging::UpdateLogLevel(options);
+
+		directory_cache_.SetTtl(fz::duration::from_seconds(options.GetOptionVal(OPTION_CACHE_TTL)));
 	}
 
 	~Impl()
@@ -57,6 +60,7 @@ public:
 		optionChangeHandler_.remove_handler();
 	}
 
+	fz::thread_pool pool_;
 	fz::event_loop loop_;
 	CRateLimiter limiter_;
 	CDirectoryCache directory_cache_;
@@ -79,6 +83,11 @@ COptionsBase& CFileZillaEngineContext::GetOptions()
 	return options_;
 }
 
+fz::thread_pool& CFileZillaEngineContext::GetThreadPool()
+{
+	return impl_->pool_;
+}
+
 fz::event_loop& CFileZillaEngineContext::GetEventLoop()
 {
 	return impl_->loop_;
