@@ -18,24 +18,29 @@ std::vector<CFileZillaEnginePrivate*> CFileZillaEnginePrivate::m_engineList;
 std::atomic_int CFileZillaEnginePrivate::m_activeStatus[2] = {{0}, {0}};
 std::list<CFileZillaEnginePrivate::t_failedLogins> CFileZillaEnginePrivate::m_failedLogins;
 
+namespace {
+unsigned int get_next_engine_id(fz::mutex& mutex)
+{
+	fz::scoped_lock lock(mutex);
+	static unsigned int id = 0;
+	return ++id;
+}
+}
+
 CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& context, CFileZillaEngine& parent, EngineNotificationHandler& notificationHandler)
 	: event_handler(context.GetEventLoop())
 	, transfer_status_(*this)
 	, notification_handler_(notificationHandler)
+	, m_engine_id(get_next_engine_id(mutex_))
 	, m_options(context.GetOptions())
 	, m_rateLimiter(context.GetRateLimiter())
 	, directory_cache_(context.GetDirectoryCache())
 	, path_cache_(context.GetPathCache())
 	, parent_(parent)
+	, thread_pool_(context.GetThreadPool())
 {
 	m_engineList.push_back(this);
 
-	{
-		fz::scoped_lock lock(mutex_);
-		static int id = 0;
-		m_engine_id = ++id;
-	}
-
 	m_pLogging = new CLogging(*this);
 
 	{
@@ -81,8 +86,9 @@ CFileZillaEnginePrivate::~CFileZillaEnginePrivate()
 
 	delete m_pLogging;
 
-	if (m_engineList.empty())
+	if (m_engineList.empty()) {
 		CSocket::Cleanup(true);
+	}
 }
 
 void CFileZillaEnginePrivate::OnEngineEvent(EngineNotificationType type)
@@ -190,7 +196,7 @@ void CFileZillaEnginePrivate::SendQueuedLogs(bool reset_flag)
 	notification_handler_.OnEngineEvent(&parent_);
 }
 
-void CFileZillaEnginePrivate::ClearQueuedLogs(fz::scoped_lock& lock, bool reset_flag)
+void CFileZillaEnginePrivate::ClearQueuedLogs(fz::scoped_lock&, bool reset_flag)
 {
 	for (auto msg : queued_logs_) {
 		delete msg;
@@ -211,7 +217,7 @@ void CFileZillaEnginePrivate::ClearQueuedLogs(bool reset_flag)
 int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 {
 	fz::scoped_lock lock(mutex_);
-	m_pLogging->LogMessage(MessageType::Debug_Debug, _T("CFileZillaEnginePrivate::ResetOperation(%d)"), nErrorCode);
+	m_pLogging->LogMessage(MessageType::Debug_Debug, L"CFileZillaEnginePrivate::ResetOperation(%d)", nErrorCode);
 
 	if (nErrorCode & FZ_REPLY_DISCONNECTED)
 		m_lastListDir.clear();
@@ -281,7 +287,7 @@ unsigned int CFileZillaEnginePrivate::GetNextAsyncRequestNumber()
 }
 
 // Command handlers
-int CFileZillaEnginePrivate::Connect(const CConnectCommand &command)
+int CFileZillaEnginePrivate::Connect(CConnectCommand const& command)
 {
 	if (IsConnected())
 		return FZ_REPLY_ALREADYCONNECTED;
@@ -314,7 +320,7 @@ int CFileZillaEnginePrivate::Disconnect(CDisconnectCommand const&)
 	return res;
 }
 
-int CFileZillaEnginePrivate::List(const CListCommand &command)
+int CFileZillaEnginePrivate::List(CListCommand const& command)
 {
 	int flags = command.GetFlags();
 	bool const refresh = (command.GetFlags() & LIST_FLAG_REFRESH) != 0;
@@ -354,12 +360,12 @@ int CFileZillaEnginePrivate::List(const CListCommand &command)
 	return m_pControlSocket->List(command.GetPath(), command.GetSubDir(), flags);
 }
 
-int CFileZillaEnginePrivate::FileTransfer(const CFileTransferCommand &command)
+int CFileZillaEnginePrivate::FileTransfer(CFileTransferCommand const& command)
 {
 	return m_pControlSocket->FileTransfer(command.GetLocalFile(), command.GetRemotePath(), command.GetRemoteFile(), command.Download(), command.GetTransferSettings());
 }
 
-int CFileZillaEnginePrivate::RawCommand(const CRawCommand& command)
+int CFileZillaEnginePrivate::RawCommand(CRawCommand const& command)
 {
 	{
 		fz::scoped_lock lock(notification_mutex_);
@@ -379,22 +385,22 @@ int CFileZillaEnginePrivate::Delete(CDeleteCommand& command)
 	return m_pControlSocket->Delete(command.GetPath(), command.ExtractFiles());
 }
 
-int CFileZillaEnginePrivate::RemoveDir(const CRemoveDirCommand& command)
+int CFileZillaEnginePrivate::RemoveDir(CRemoveDirCommand const& command)
 {
 	return m_pControlSocket->RemoveDir(command.GetPath(), command.GetSubDir());
 }
 
-int CFileZillaEnginePrivate::Mkdir(const CMkdirCommand& command)
+int CFileZillaEnginePrivate::Mkdir(CMkdirCommand const& command)
 {
 	return m_pControlSocket->Mkdir(command.GetPath());
 }
 
-int CFileZillaEnginePrivate::Rename(const CRenameCommand& command)
+int CFileZillaEnginePrivate::Rename(CRenameCommand const& command)
 {
 	return m_pControlSocket->Rename(command);
 }
 
-int CFileZillaEnginePrivate::Chmod(const CChmodCommand& command)
+int CFileZillaEnginePrivate::Chmod(CChmodCommand const& command)
 {
 	return m_pControlSocket->Chmod(command);
 }
@@ -403,10 +409,10 @@ void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath
 {
 	fz::scoped_lock lock(mutex_);
 
-	wxASSERT(m_pControlSocket);
+	assert(m_pControlSocket);
 
 	const CServer* const pOwnServer = m_pControlSocket->GetCurrentServer();
-	wxASSERT(pOwnServer);
+	assert(pOwnServer);
 
 	m_lastListDir = path;
 
@@ -486,18 +492,21 @@ unsigned int CFileZillaEnginePrivate::GetRemainingReconnectDelay(const CServer&
 			++iter;
 			m_failedLogins.erase(prev);
 		}
-		else if (!iter->critical && iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())
+		else if (!iter->critical && iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort()) {
 			return delay * 1000 - span.get_milliseconds();
-		else if (iter->server == server)
+		}
+		else if (iter->server == server) {
 			return delay * 1000 - span.get_milliseconds();
-		else
+		}
+		else {
 			++iter;
+		}
 	}
 
 	return 0;
 }
 
-void CFileZillaEnginePrivate::OnTimer(int)
+void CFileZillaEnginePrivate::OnTimer(fz::timer_id)
 {
 	if (!m_retryTimer) {
 		return;
@@ -505,10 +514,11 @@ void CFileZillaEnginePrivate::OnTimer(int)
 	m_retryTimer = 0;
 
 	if (!m_pCurrentCommand || m_pCurrentCommand->GetId() != Command::connect) {
-		wxFAIL_MSG(_T("CFileZillaEnginePrivate::OnTimer called without pending Command::connect"));
+		m_pLogging->LogMessage(MessageType::Debug_Warning, L"CFileZillaEnginePrivate::OnTimer called without pending Command::connect");
 		return;
 	}
-	wxASSERT(!IsConnected());
+
+	assert(!IsConnected());
 
 	m_pControlSocket.reset();
 
@@ -520,7 +530,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 	fz::scoped_lock lock(mutex_);
 
 	if (!m_pCurrentCommand || m_pCurrentCommand->GetId() != Command::connect) {
-		m_pLogging->LogMessage(MessageType::Debug_Warning, _T("CFileZillaEnginePrivate::ContinueConnect called without pending Command::connect"));
+		m_pLogging->LogMessage(MessageType::Debug_Warning, L"CFileZillaEnginePrivate::ContinueConnect called without pending Command::connect");
 		return ResetOperation(FZ_REPLY_INTERNALERROR);
 	}
 
@@ -528,7 +538,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 	const CServer& server = pConnectCommand->GetServer();
 	unsigned int delay = GetRemainingReconnectDelay(server);
 	if (delay) {
-		m_pLogging->LogMessage(MessageType::Status, wxPLURAL("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay + 999) / 1000), (delay + 999) / 1000);
+		m_pLogging->LogMessage(MessageType::Status, fztranslate("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay + 999) / 1000), (delay + 999) / 1000);
 		stop_timer(m_retryTimer);
 		m_retryTimer = add_timer(fz::duration::from_milliseconds(delay), true);
 		return FZ_REPLY_WOULDBLOCK;
@@ -550,7 +560,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 		m_pControlSocket = std::make_unique<CHttpControlSocket>(*this);
 		break;
 	default:
-		m_pLogging->LogMessage(MessageType::Debug_Warning, _T("Not a valid protocol: %d"), server.GetProtocol());
+		m_pLogging->LogMessage(MessageType::Debug_Warning, L"Not a valid protocol: %d", server.GetProtocol());
 		return FZ_REPLY_SYNTAXERROR|FZ_REPLY_DISCONNECTED;
 	}
 
@@ -565,9 +575,9 @@ void CFileZillaEnginePrivate::InvalidateCurrentWorkingDirs(const CServerPath& pa
 {
 	fz::scoped_lock lock(mutex_);
 
-	wxASSERT(m_pControlSocket);
+	assert(m_pControlSocket);
 	const CServer* const pOwnServer = m_pControlSocket->GetCurrentServer();
-	wxASSERT(pOwnServer);
+	assert(pOwnServer);
 
 	for (auto & engine : m_engineList) {
 		if (!engine || engine == this)
@@ -678,7 +688,7 @@ void CFileZillaEnginePrivate::DoCancel()
 		return;
 
 	if (m_retryTimer) {
-		wxASSERT(m_pCurrentCommand && m_pCurrentCommand->GetId() == Command::connect);
+		assert(m_pCurrentCommand && m_pCurrentCommand->GetId() == Command::connect);
 
 		m_pControlSocket.reset();
 
@@ -816,7 +826,7 @@ int CFileZillaEnginePrivate::CacheLookup(const CServerPath& path, CDirectoryList
 	if (!IsConnected())
 		return FZ_REPLY_ERROR;
 
-	wxASSERT(m_pControlSocket->GetCurrentServer());
+	assert(m_pControlSocket->GetCurrentServer());
 
 	bool is_outdated = false;
 	if (!directory_cache_.Lookup(listing, *m_pControlSocket->GetCurrentServer(), path, true, is_outdated))
