@@ -1,5 +1,5 @@
-#ifndef __FILEZILLAENGINEPRIVATE_H__
-#define __FILEZILLAENGINEPRIVATE_H__
+#ifndef FILEZILLA_ENGINEPRIVATE_HEADER
+#define FILEZILLA_ENGINEPRIVATE_HEADER
 
 #include <libfilezilla/event.hpp>
 #include <libfilezilla/event_handler.hpp>
@@ -35,11 +35,11 @@ public:
 
 	bool empty();
 
-	void Init(wxFileOffset totalSize, wxFileOffset startOffset, bool list);
+	void Init(int64_t totalSize, int64_t startOffset, bool list);
 	void Reset();
 	void SetStartTime();
 	void SetMadeProgress();
-	void Update(wxFileOffset transferredBytes);
+	void Update(int64_t transferredBytes);
 
 	CTransferStatus Get(bool &changed);
 
@@ -56,11 +56,9 @@ protected:
 class CFileZillaEnginePrivate final : public fz::event_handler, COptionChangeEventHandler
 {
 public:
-	CFileZillaEnginePrivate(CFileZillaEngineContext& engine_context, CFileZillaEngine& parent);
+	CFileZillaEnginePrivate(CFileZillaEngineContext& engine_context, CFileZillaEngine& parent, EngineNotificationHandler& notificationHandler);
 	virtual ~CFileZillaEnginePrivate();
 
-	int Init(wxEvtHandler *pEventHandler);
-
 	int Execute(CCommand const& command);
 	int Cancel();
 	int ResetOperation(int nErrorCode);
@@ -83,6 +81,7 @@ public:
 	void SetActive(int direction);
 
 	// Add new pending notification
+	void AddNotification(fz::scoped_lock& lock, CNotification *pNotification); // note: Unlocks the mutex!
 	void AddNotification(CNotification *pNotification);
 	void AddLogNotification(CLogmsgNotification *pNotification);
 	std::unique_ptr<CNotification> GetNextNotification();
@@ -91,6 +90,7 @@ public:
 	CRateLimiter& GetRateLimiter() { return m_rateLimiter; }
 	CDirectoryCache& GetDirectoryCache() { return directory_cache_; }
 	CPathCache& GetPathCache() { return path_cache_; }
+	fz::thread_pool& GetThreadPool() { return thread_pool_; }
 
 	void SendDirectoryListingNotification(const CServerPath& path, bool onList, bool modified, bool failed);
 
@@ -103,7 +103,7 @@ public:
 	// directory.
 	void InvalidateCurrentWorkingDirs(const CServerPath& path);
 
-	int GetEngineId() const {return m_engine_id; }
+	unsigned int GetEngineId() const {return m_engine_id; }
 
 	CTransferStatusManager transfer_status_;
 protected:
@@ -111,6 +111,7 @@ protected:
 
 	void SendQueuedLogs(bool reset_flag = false);
 	void ClearQueuedLogs(bool reset_flag);
+	void ClearQueuedLogs(fz::scoped_lock& lock, bool reset_flag);
 	bool ShouldQueueLogsFromOptions() const;
 
 	int CheckCommandPreconditions(CCommand const& command, bool checkBusy);
@@ -120,16 +121,16 @@ protected:
 	void OnSetAsyncRequestReplyEvent(std::unique_ptr<CAsyncRequestNotification> const& reply);
 
 	// Command handlers, only called by CFileZillaEngine::Command
-	int Connect(const CConnectCommand &command);
-	int Disconnect(const CDisconnectCommand &command);
-	int List(const CListCommand &command);
-	int FileTransfer(const CFileTransferCommand &command);
-	int RawCommand(const CRawCommand& command);
+	int Connect(CConnectCommand const& command);
+	int Disconnect(CDisconnectCommand const& command);
+	int List(CListCommand const&command);
+	int FileTransfer(CFileTransferCommand const& command);
+	int RawCommand(CRawCommand const& command);
 	int Delete(CDeleteCommand& command);
-	int RemoveDir(const CRemoveDirCommand& command);
-	int Mkdir(const CMkdirCommand& command);
-	int Rename(const CRenameCommand& command);
-	int Chmod(const CChmodCommand& command);
+	int RemoveDir(CRemoveDirCommand const& command);
+	int Mkdir(CMkdirCommand const& command);
+	int Rename(CRenameCommand const& command);
+	int Chmod(CChmodCommand const& command);
 
 	void DoCancel();
 
@@ -137,7 +138,7 @@ protected:
 
 	void operator()(fz::event_base const& ev);
 	void OnEngineEvent(EngineNotificationType type);
-	void OnTimer(int timer_id);
+	void OnTimer(fz::timer_id);
 	void OnCommandEvent();
 
 	// General mutex for operations on the engine
@@ -145,11 +146,11 @@ protected:
 	static fz::mutex mutex_;
 
 	// Used to synchronize access to the notification list
-	fz::mutex notification_mutex_;
+	fz::mutex notification_mutex_{false};
 
-	wxEvtHandler *m_pEventHandler{};
+	EngineNotificationHandler& notification_handler_;
 
-	int m_engine_id;
+	unsigned int const m_engine_id;
 
 	static std::vector<CFileZillaEnginePrivate*> m_engineList;
 
@@ -202,6 +203,8 @@ protected:
 
 	bool queue_logs_{true};
 	std::deque<CLogmsgNotification*> queued_logs_;
+
+	fz::thread_pool & thread_pool_;
 };
 
 struct command_event_type{};
@@ -210,4 +213,4 @@ typedef fz::simple_event<command_event_type> CCommandEvent;
 struct async_request_reply_event_type{};
 typedef fz::simple_event<async_request_reply_event_type, std::unique_ptr<CAsyncRequestNotification>> CAsyncRequestReplyEvent;
 
-#endif //__FILEZILLAENGINEPRIVATE_H__
+#endif
