@@ -6,7 +6,7 @@
 #include "externalipresolver.h"
 #include "rtt.h"
 
-#include <wx/regex.h>
+#include <regex>
 
 #define RECVBUFFERSIZE 4096
 #define MAXLINELEN 2000
@@ -31,35 +31,35 @@ protected:
 	virtual int ResetOperation(int nErrorCode);
 
 	virtual int Connect(const CServer &server);
-	virtual int List(CServerPath path = CServerPath(), wxString subDir = _T(""), int flags = 0);
+	virtual int List(CServerPath path = CServerPath(), std::wstring const& subDir = std::wstring(), int flags = 0);
 	int ListParseResponse();
 	int ListSubcommandResult(int prevResult);
 	int ListSend();
 	int ListCheckTimezoneDetection(CDirectoryListing& listing);
 
-	int ChangeDir(CServerPath path = CServerPath(), wxString subDir = _T(""), bool link_discovery = false);
+	int ChangeDir(CServerPath path = CServerPath(), std::wstring subDir = std::wstring(), bool link_discovery = false);
 	int ChangeDirParseResponse();
 	int ChangeDirSubcommandResult(int prevResult);
 	int ChangeDirSend();
 
-	virtual int FileTransfer(const wxString localFile, const CServerPath &remotePath,
-							 const wxString &remoteFile, bool download,
-							 const CFileTransferCommand::t_transferSettings& transferSettings);
+	virtual int FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
+							 std::wstring const& remoteFile, bool download,
+							 CFileTransferCommand::t_transferSettings const& transferSettings);
 	int FileTransferParseResponse();
 	int FileTransferSubcommandResult(int prevResult);
 	int FileTransferSend();
 	int FileTransferTestResumeCapability();
 
-	virtual int RawCommand(const wxString& command);
+	virtual int RawCommand(std::wstring const& command);
 	int RawCommandSend();
 	int RawCommandParseResponse();
 
-	virtual int Delete(const CServerPath& path, std::deque<wxString>&& files);
+	virtual int Delete(const CServerPath& path, std::deque<std::wstring>&& files);
 	int DeleteSubcommandResult(int prevResult);
 	int DeleteSend();
 	int DeleteParseResponse();
 
-	virtual int RemoveDir(const CServerPath& path, const wxString& subDir);
+	virtual int RemoveDir(CServerPath const& path, std::wstring const& subDir);
 	int RemoveDirSubcommandResult(int prevResult);
 	int RemoveDirSend();
 	int RemoveDirParseResponse();
@@ -78,23 +78,23 @@ protected:
 	virtual int ChmodSubcommandResult(int prevResult);
 	virtual int ChmodSend();
 
-	virtual int Transfer(const wxString& cmd, CFtpTransferOpData* oldData);
+	virtual int Transfer(std::wstring const& cmd, CFtpTransferOpData* oldData);
 	virtual int TransferParseResponse();
 	virtual int TransferSend();
 
 	virtual void OnConnect();
 	virtual void OnReceive();
 
-	bool SendCommand(wxString const& str, bool maskArgs = false, bool measureRTT = true);
+	bool SendCommand(std::wstring const& str, bool maskArgs = false, bool measureRTT = true);
 
 	// Parse the latest reply line from the server
-	void ParseLine(wxString line);
+	void ParseLine(std::wstring line);
 
 	// Parse the actual response and delegate it to the handlers.
 	// It's the last line in a multi-line response.
 	void ParseResponse();
 
-	void ParseFeat(wxString line);
+	void ParseFeat(std::wstring line);
 
 	virtual int SendNextCommand();
 	virtual int ParseSubcommandResult(int prevResult);
@@ -105,7 +105,7 @@ protected:
 	int LogonParseResponse();
 	int LogonSend();
 
-	wxString GetPassiveCommand(CRawTransferOpData& data);
+	std::wstring GetPassiveCommand(CRawTransferOpData& data);
 	bool ParsePasvResponse(CRawTransferOpData* pData);
 	bool ParseEpsvResponse(CRawTransferOpData* pData);
 
@@ -114,7 +114,7 @@ protected:
 	// Other servers return "550 No files found."
 	bool IsMisleadingListResponse() const;
 
-	int GetExternalIPAddress(wxString& address);
+	int GetExternalIPAddress(std::string& address);
 
 	// Checks if listing2 is a subset of listing1. Compares only filenames.
 	bool CheckInclusion(const CDirectoryListing& listing1, const CDirectoryListing& listing2);
@@ -123,9 +123,9 @@ protected:
 
 	bool GetLoginSequence(const CServer& server);
 
-	wxString m_Response;
-	wxString m_MultilineResponseCode;
-	std::vector<wxString> m_MultilineResponseLines;
+	std::wstring m_Response;
+	std::wstring m_MultilineResponseCode;
+	std::vector<std::wstring> m_MultilineResponseLines;
 
 	CTransferSocket *m_pTransferSocket;
 
@@ -159,7 +159,7 @@ protected:
 	void OnExternalIPAddress();
 	void OnTimer(fz::timer_id id);
 
-	wxRegEx m_pasvReplyRegex; // Have it as class member to avoid recompiling the regex on each transfer or listing
+	std::unique_ptr<std::wregex> m_pasvReplyRegex; // Have it as class member to avoid recompiling the regex on each transfer or listing
 };
 
 class CIOThread;
@@ -180,27 +180,11 @@ public:
 class CFtpFileTransferOpData final : public CFileTransferOpData, public CFtpTransferOpData
 {
 public:
-	CFtpFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path);
+	CFtpFileTransferOpData(bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path);
 	virtual ~CFtpFileTransferOpData();
 
 	CIOThread *pIOThread{};
 	bool fileDidExist{true};
 };
 
-class CRawTransferOpData final : public COpData
-{
-public:
-	CRawTransferOpData();
-	wxString cmd;
-
-	CFtpTransferOpData* pOldData;
-
-	bool bPasv;
-	bool bTriedPasv;
-	bool bTriedActive;
-
-	wxString host;
-	int port;
-};
-
 #endif
