@@ -4,11 +4,14 @@
 #include "engineprivate.h"
 #include "httpcontrolsocket.h"
 #include "tlssocket.h"
+#include "uri.h"
 
 #include <libfilezilla/file.hpp>
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
+#include <wx/string.h>
+
 #define FZ_REPLY_REDIRECTED FZ_REPLY_ALREADYCONNECTED
 
 // Connect is special for HTTP: It is done on a per-command basis, so we need
@@ -40,8 +43,8 @@ public:
 
 	bool m_gotHeader{};
 	int m_responseCode{-1};
-	wxString m_responseString;
-	wxURI m_newLocation;
+	std::wstring m_responseString;
+	fz::uri m_newLocation;
 	int m_redirectionCount{};
 
 	int64_t m_totalSize{-1};
@@ -68,8 +71,9 @@ public:
 class CHttpFileTransferOpData : public CFileTransferOpData, public CHttpOpData
 {
 public:
-	CHttpFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path)
-		: CFileTransferOpData(is_download, local_file, remote_file, remote_path), CHttpOpData(this)
+	CHttpFileTransferOpData(bool is_download, std::wstring const& local_file, std::wstring const& remote_file, const CServerPath& remote_path)
+		: CFileTransferOpData(is_download, local_file, remote_file, remote_path)
+		, CHttpOpData(this)
 	{
 	}
 
@@ -79,31 +83,24 @@ public:
 CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate & engine)
 	: CRealControlSocket(engine)
 {
-	m_pRecvBuffer = 0;
-	m_recvBufferPos = 0;
-	m_pTlsSocket = 0;
-	m_pHttpOpData = 0;
 }
 
 CHttpControlSocket::~CHttpControlSocket()
 {
 	remove_handler();
 	DoClose();
-	delete [] m_pRecvBuffer;
 }
 
 int CHttpControlSocket::SendNextCommand()
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::SendNextCommand()"));
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("SendNextCommand called without active operation"));
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (m_pCurOpData->waitForAsyncRequest)
-	{
+	if (m_pCurOpData->waitForAsyncRequest) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Waiting for async request, ignoring SendNextCommand"));
 		return FZ_REPLY_WOULDBLOCK;
 	}
@@ -190,14 +187,13 @@ void CHttpControlSocket::OnReceive()
 
 int CHttpControlSocket::DoReceive()
 {
-	do
-	{
+	do {
 		const CSocket::SocketState state = m_pSocket->GetState();
-		if (state != CSocket::connected && state != CSocket::closing)
+		if (state != CSocket::connected && state != CSocket::closing) {
 			return 0;
+		}
 
-		if (!m_pRecvBuffer)
-		{
+		if (!m_pRecvBuffer) {
 			m_pRecvBuffer = new char[m_recvBufferLen];
 			m_recvBufferPos = 0;
 		}
@@ -205,10 +201,8 @@ int CHttpControlSocket::DoReceive()
 		unsigned int len = m_recvBufferLen - m_recvBufferPos;
 		int error;
 		int read = m_pBackend->Read(m_pRecvBuffer + m_recvBufferPos, len, error);
-		if (read == -1)
-		{
-			if (error != EAGAIN)
-			{
+		if (read == -1) {
+			if (error != EAGAIN) {
 				ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 			}
 			return 0;
@@ -250,7 +244,7 @@ int CHttpControlSocket::DoReceive()
 		{
 			if (!read)
 			{
-				wxASSERT(!m_recvBufferPos);
+				assert(!m_recvBufferPos);
 				ProcessData(0, 0);
 				return 0;
 			}
@@ -269,7 +263,7 @@ int CHttpControlSocket::DoReceive()
 
 void CHttpControlSocket::OnConnect()
 {
-	wxASSERT(GetCurrentCommandId() == Command::connect);
+	assert(GetCurrentCommandId() == Command::connect);
 
 	CHttpConnectOpData *pData = static_cast<CHttpConnectOpData *>(m_pCurOpData);
 
@@ -312,22 +306,19 @@ enum filetransferStates
 	filetransfer_transfer
 };
 
-int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath &remotePath,
-							  const wxString &remoteFile, bool download,
-							  const CFileTransferCommand::t_transferSettings&)
+int CHttpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
+									std::wstring const& remoteFile, bool download,
+									CFileTransferCommand::t_transferSettings const&)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransfer()"));
 
 	LogMessage(MessageType::Status, _("Downloading %s"), remotePath.FormatFilename(remoteFile));
 
-	if (!download)
-	{
-		ResetOperation(FZ_REPLY_CRITICALERROR | FZ_REPLY_NOTSUPPORTED);
+	if (!download) {
 		return FZ_REPLY_ERROR;
 	}
 
-	if (m_pCurOpData)
-	{
+	if (m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("deleting nonzero pData"));
 		delete m_pCurOpData;
 	}
@@ -336,28 +327,37 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 	m_pCurOpData = pData;
 	m_pHttpOpData = pData;
 
-	m_current_uri = wxURI(m_pCurrentServer->Format(ServerFormat::with_user_and_optional_port) + pData->remotePath.FormatFilename(pData->remoteFile));
+	// TODO: Ordinarily we need to percent-encode the filename. With the current API we then however would not be able to pass the query part of the URL
+	m_current_uri = fz::uri(fz::to_utf8(m_pCurrentServer->Format(ServerFormat::url)) + fz::to_utf8(pData->remotePath.FormatFilename(pData->remoteFile)));
+	if (m_current_uri.empty()) {
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
 
 	if (!localFile.empty()) {
 		pData->localFileSize = fz::local_filesys::get_size(fz::to_native(pData->localFile));
 
 		pData->opState = filetransfer_waitfileexists;
 		int res = CheckOverwriteFile();
-		if (res != FZ_REPLY_OK)
+		if (res != FZ_REPLY_OK) {
 			return res;
+		}
 
 		pData->opState = filetransfer_transfer;
 
 		res = OpenFile(pData);
-		if( res != FZ_REPLY_OK )
+		if (res != FZ_REPLY_OK) {
 			return res;
+		}
 	}
-	else
+	else {
 		pData->opState = filetransfer_transfer;
+	}
 
 	int res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(), m_pCurrentServer->GetProtocol() == HTTPS);
-	if (res != FZ_REPLY_OK)
+	if (res != FZ_REPLY_OK) {
 		return res;
+	}
 
 	return FileTransferSend();
 }
@@ -366,15 +366,13 @@ int CHttpControlSocket::FileTransferSubcommandResult(int prevResult)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransferSubcommandResult(%d)"), prevResult);
 
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (prevResult != FZ_REPLY_OK)
-	{
+	if (prevResult != FZ_REPLY_OK) {
 		ResetOperation(prevResult);
 		return FZ_REPLY_ERROR;
 	}
@@ -392,8 +390,8 @@ int CHttpControlSocket::FileTransferSend()
 		return FZ_REPLY_ERROR;
 	}
 
-	if (!m_current_uri.HasScheme() || !m_current_uri.HasServer() || !m_current_uri.HasPath()) {
-		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Invalid URI: %s"), m_current_uri.BuildURI());
+	if (m_current_uri.scheme_.empty() || m_current_uri.host_.empty() || !m_current_uri.is_absolute()) {
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, "Invalid URI: %s", m_current_uri.to_string());
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -414,18 +412,11 @@ int CHttpControlSocket::FileTransferSend()
 		}
 	}
 
-	std::wstring location = m_current_uri.GetPath().ToStdWstring();
-	if (m_current_uri.HasQuery()) {
-		location += L"?" + m_current_uri.GetQuery().ToStdWstring();
-	}
-	std::wstring action = fz::sprintf(L"GET %s HTTP/1.1", location);
+	std::string action = fz::sprintf("GET %s HTTP/1.1", m_current_uri.get_request());
 	LogMessageRaw(MessageType::Command, action);
 
-	std::string hostWithPort = fz::to_utf8(m_current_uri.GetServer());
-	if (m_current_uri.HasPort()) {
-		hostWithPort += ":" + fz::to_utf8(m_current_uri.GetPort());
-	}
-	std::string command = fz::sprintf("%s\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n", fz::to_utf8(action), hostWithPort, PACKAGE_STRING);
+	std::string host = m_current_uri.get_authority(false);
+	std::string command = fz::sprintf("%s\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n", action, host, PACKAGE_STRING);
 	if (pData->resume) {
 		command += fz::sprintf("Range: bytes=%d-\r\n", pData->localFileSize);
 	}
@@ -438,7 +429,7 @@ int CHttpControlSocket::FileTransferSend()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-int CHttpControlSocket::InternalConnect(wxString host, unsigned short port, bool tls)
+int CHttpControlSocket::InternalConnect(std::wstring host, unsigned short port, bool tls)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::InternalConnect()"));
 
@@ -448,7 +439,7 @@ int CHttpControlSocket::InternalConnect(wxString host, unsigned short port, bool
 	pData->port = port;
 	pData->tls = tls;
 
-	if (fz::get_address_type(host.ToStdWstring()) == fz::address_type::unknown) {
+	if (fz::get_address_type(host) == fz::address_type::unknown) {
 		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
 	}
 
@@ -510,7 +501,7 @@ int CHttpControlSocket::FileTransferParseResponse(char* p, unsigned int len)
 		engine_.AddNotification(new CDataNotification(q, len));
 	}
 	else {
-		wxASSERT(pData->file.opened());
+		assert(pData->file.opened());
 
 		auto write = static_cast<int64_t>(len);
 		if (pData->file.write(p, write) != write) {
@@ -534,12 +525,9 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 	for (;;) {
 		// Find line ending
 		unsigned int i = 0;
-		for (i = 0; (i + 1) < m_recvBufferPos; i++)
-		{
-			if (m_pRecvBuffer[i] == '\r')
-			{
-				if (m_pRecvBuffer[i + 1] != '\n')
-				{
+		for (i = 0; (i + 1) < m_recvBufferPos; i++) {
+			if (m_pRecvBuffer[i] == '\r') {
+				if (m_pRecvBuffer[i + 1] != '\n') {
 					LogMessage(MessageType::Error, _("Malformed reply, server not sending proper line endings"));
 					ResetOperation(FZ_REPLY_ERROR);
 					return FZ_REPLY_ERROR;
@@ -547,10 +535,8 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				break;
 			}
 		}
-		if ((i + 1) >= m_recvBufferPos)
-		{
-			if (m_recvBufferPos == m_recvBufferLen)
-			{
+		if ((i + 1) >= m_recvBufferPos) {
+			if (m_recvBufferPos == m_recvBufferLen) {
 				// We don't support header lines larger than 4096
 				LogMessage(MessageType::Error, _("Too long header line"));
 				ResetOperation(FZ_REPLY_ERROR);
@@ -560,15 +546,14 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 		}
 
 		m_pRecvBuffer[i] = 0;
-		wxString const line = wxString(m_pRecvBuffer, wxConvLocal);
-		if (!line.empty())
+		std::wstring const line = wxString(m_pRecvBuffer, wxConvLocal).ToStdWstring();
+		if (!line.empty()) {
 			LogMessageRaw(MessageType::Response, line);
+		}
 
-		if (pData->m_responseCode == -1)
-		{
+		if (pData->m_responseCode == -1) {
 			pData->m_responseString = line;
-			if (m_recvBufferPos < 16 || memcmp(m_pRecvBuffer, "HTTP/1.", 7))
-			{
+			if (m_recvBufferPos < 16 || memcmp(m_pRecvBuffer, "HTTP/1.", 7)) {
 				// Invalid HTTP Status-Line
 				LogMessage(MessageType::Error, _("Invalid HTTP Response"));
 				ResetOperation(FZ_REPLY_ERROR);
@@ -587,14 +572,14 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 
 			pData->m_responseCode = (m_pRecvBuffer[9] - '0') * 100 + (m_pRecvBuffer[10] - '0') * 10 + m_pRecvBuffer[11] - '0';
 
-			if( pData->m_responseCode == 416 ) {
+			if (pData->m_responseCode == 416) {
 				CHttpFileTransferOpData* pTransfer = static_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
-				if( pTransfer->resume ) {
+				if (pTransfer->resume) {
 					// Sad, the server does not like our attempt to resume.
 					// Get full file instead.
 					pTransfer->resume = false;
 					int res = OpenFile(pTransfer);
-					if( res != FZ_REPLY_OK ) {
+					if (res != FZ_REPLY_OK) {
 						return res;
 					}
 					pData->m_newLocation = m_current_uri;
@@ -602,30 +587,25 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				}
 			}
 
-			if (pData->m_responseCode >= 400)
-			{
+			if (pData->m_responseCode >= 400) {
 				// Failed request
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
 
-			if (pData->m_responseCode == 305)
-			{
+			if (pData->m_responseCode == 305) {
 				// Unsupported redirect
 				LogMessage(MessageType::Error, _("Unsupported redirect"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
 		}
-		else
-		{
-			if (!i)
-			{
+		else {
+			if (!i) {
 				// End of header, data from now on
 
 				// Redirect if neccessary
-				if (pData->m_responseCode >= 300)
-				{
+				if (pData->m_responseCode >= 300) {
 					if (pData->m_redirectionCount++ == 6) {
 						LogMessage(MessageType::Error, _("Too many redirects"));
 						ResetOperation(FZ_REPLY_ERROR);
@@ -635,43 +615,47 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 					ResetSocket();
 					ResetHttpData(pData);
 
-					if( !pData->m_newLocation.HasScheme() || !pData->m_newLocation.HasServer() || !pData->m_newLocation.HasPath() ) {
-						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported URI: %s"), m_current_uri.BuildURI());
+					if (pData->m_newLocation.scheme_.empty() || pData->m_newLocation.host_.empty() || !pData->m_newLocation.is_absolute()) {
+						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported URI: %s"), m_current_uri.to_string());
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
 					}
 
-					ServerProtocol protocol = CServer::GetProtocolFromPrefix(pData->m_newLocation.GetScheme());
-					if( protocol != HTTP && protocol != HTTPS ) {
-						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported address: %s"), pData->m_newLocation.BuildURI());
+					ServerProtocol protocol = CServer::GetProtocolFromPrefix(fz::to_wstring_from_utf8(pData->m_newLocation.scheme_));
+					if (protocol != HTTP && protocol != HTTPS) {
+						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported address: %s"), pData->m_newLocation.to_string());
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
 					}
 
-					long port = CServer::GetDefaultPort(protocol);
-					if( pData->m_newLocation.HasPort() && (!pData->m_newLocation.GetPort().ToLong(&port) || port < 1 || port > 65535) ) {
-						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported address: %s"), pData->m_newLocation.BuildURI());
-						ResetOperation(FZ_REPLY_ERROR);
-						return FZ_REPLY_ERROR;
+					unsigned short port = CServer::GetDefaultPort(protocol);
+					if (pData->m_newLocation.port_ != 0) {
+						port = pData->m_newLocation.port_;
 					}
 
 					m_current_uri = pData->m_newLocation;
 
 					// International domain names
-					wxString host = ConvertDomainName(m_current_uri.GetServer());
+					std::wstring host = ConvertDomainName(fz::to_wstring_from_utf8(m_current_uri.host_));
+					if (host.empty()) {
+						LogMessage(MessageType::Error, _("Invalid hostname: %s"), pData->m_newLocation.to_string());
+						ResetOperation(FZ_REPLY_ERROR);
+						return FZ_REPLY_ERROR;
+					}
 
-					int res = InternalConnect(host, static_cast<unsigned short>(port), protocol == HTTPS);
-					if (res == FZ_REPLY_WOULDBLOCK)
+					int res = InternalConnect(host, port, protocol == HTTPS);
+					if (res == FZ_REPLY_WOULDBLOCK) {
 						res |= FZ_REPLY_REDIRECTED;
+					}
 					return res;
 				}
 
-				if( pData->m_pOpData && pData->m_pOpData->opId == Command::transfer) {
+				if (pData->m_pOpData && pData->m_pOpData->opId == Command::transfer) {
 					CHttpFileTransferOpData* pTransfer = static_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
-					if( pTransfer->resume && pData->m_responseCode != 206 ) {
+					if (pTransfer->resume && pData->m_responseCode != 206) {
 						pTransfer->resume = false;
 						int res = OpenFile(pTransfer);
-						if( res != FZ_REPLY_OK ) {
+						if (res != FZ_REPLY_OK) {
 							return res;
 						}
 					}
@@ -682,13 +666,12 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				memmove(m_pRecvBuffer, m_pRecvBuffer + 2, m_recvBufferPos - 2);
 				m_recvBufferPos -= 2;
 
-				if (m_recvBufferPos)
-				{
+				if (m_recvBufferPos) {
 					int res;
-					if (pData->m_transferEncoding == pData->chunked)
+					if (pData->m_transferEncoding == pData->chunked) {
 						res = OnChunkedData(pData);
-					else
-					{
+					}
+					else {
 						pData->m_receivedData += m_recvBufferPos;
 						res = ProcessData(m_pRecvBuffer, m_recvBufferPos);
 						m_recvBufferPos = 0;
@@ -698,28 +681,28 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 
 				return FZ_REPLY_WOULDBLOCK;
 			}
-			if (m_recvBufferPos > 12 && !memcmp(m_pRecvBuffer, "Location: ", 10))
-			{
-				pData->m_newLocation = wxURI(wxString(m_pRecvBuffer + 10, wxConvLocal));
-				pData->m_newLocation.Resolve(m_current_uri);
+			if (m_recvBufferPos > 12 && !memcmp(m_pRecvBuffer, "Location: ", 10)) {
+				pData->m_newLocation = fz::uri(m_pRecvBuffer + 10);
+				if (!pData->m_newLocation.empty()) {
+					pData->m_newLocation.resolve(m_current_uri);
+				}
 			}
-			else if (m_recvBufferPos > 21 && !memcmp(m_pRecvBuffer, "Transfer-Encoding: ", 19))
-			{
-				if (!strcmp(m_pRecvBuffer + 19, "chunked"))
+			else if (m_recvBufferPos > 21 && !memcmp(m_pRecvBuffer, "Transfer-Encoding: ", 19)) {
+				if (!strcmp(m_pRecvBuffer + 19, "chunked")) {
 					pData->m_transferEncoding = CHttpOpData::chunked;
-				else if (!strcmp(m_pRecvBuffer + 19, "identity"))
+				}
+				else if (!strcmp(m_pRecvBuffer + 19, "identity")) {
 					pData->m_transferEncoding = CHttpOpData::identity;
-				else
+				}
+				else {
 					pData->m_transferEncoding = CHttpOpData::unknown;
+				}
 			}
-			else if (i > 16 && !memcmp(m_pRecvBuffer, "Content-Length: ", 16))
-			{
+			else if (i > 16 && !memcmp(m_pRecvBuffer, "Content-Length: ", 16)) {
 				pData->m_totalSize = 0;
 				char* p = m_pRecvBuffer + 16;
-				while (*p)
-				{
-					if (*p < '0' || *p > '9')
-					{
+				while (*p) {
+					if (*p < '0' || *p > '9') {
 						LogMessage(MessageType::Error, _("Malformed header: %s"), _("Invalid Content-Length"));
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
@@ -732,8 +715,9 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 		memmove(m_pRecvBuffer, m_pRecvBuffer + i + 2, m_recvBufferPos - i - 2);
 		m_recvBufferPos -= i + 2;
 
-		if (!m_recvBufferPos)
+		if (!m_recvBufferPos) {
 			break;
+		}
 	}
 
 	return FZ_REPLY_WOULDBLOCK;
@@ -922,12 +906,25 @@ void CHttpControlSocket::OnClose(int error)
 	ProcessData(0, 0);
 }
 
+void CHttpControlSocket::ResetSocket()
+{
+	delete[] m_pRecvBuffer;
+	m_pRecvBuffer = 0;
+	m_recvBufferPos = 0;
+
+	if (m_pTlsSocket) {
+		if (m_pTlsSocket != m_pBackend) {
+			delete m_pTlsSocket;
+		}
+		m_pTlsSocket = 0;
+	}
+
+	CRealControlSocket::ResetSocket();
+}
+
 void CHttpControlSocket::ResetHttpData(CHttpOpData* pData)
 {
-	wxASSERT(pData);
-
-	delete [] m_pRecvBuffer;
-	m_pRecvBuffer = 0;
+	assert(pData);
 
 	pData->m_gotHeader = false;
 	pData->m_responseCode = -1;
@@ -957,7 +954,7 @@ int CHttpControlSocket::ProcessData(char* p, int len)
 		break;
 	}
 
-	wxASSERT(p || !m_pCurOpData);
+	assert(p || !m_pCurOpData);
 
 	return res;
 }
