@@ -1,7 +1,7 @@
-#ifndef __HTTPCONTROLSOCKET_H__
-#define __HTTPCONTROLSOCKET_H__
+#ifndef FILEZILLA_ENGINE_HTTPCONTROLSOCKET_HEADER
+#define FILEZILLA_ENGINE_HTTPCONTROLSOCKET_HEADER
 
-#include <wx/uri.h>
+#include "uri.h"
 
 class CHttpOpData;
 class CTlsSocket;
@@ -20,14 +20,14 @@ protected:
 	virtual int SendNextCommand();
 	virtual int ParseSubcommandResult(int prevResult);
 
-	virtual int FileTransfer(const wxString localFile, const CServerPath &remotePath,
-							 const wxString &remoteFile, bool download,
-							 const CFileTransferCommand::t_transferSettings& transferSettings);
+	virtual int FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
+		std::wstring const& remoteFile, bool download,
+		CFileTransferCommand::t_transferSettings const& transferSettings);
 	virtual int FileTransferSend();
 	virtual int FileTransferParseResponse(char* p, unsigned int len);
 	virtual int FileTransferSubcommandResult(int prevResult);
 
-	int InternalConnect(wxString host, unsigned short port, bool tls);
+	int InternalConnect(std::wstring host, unsigned short port, bool tls);
 	int DoInternalConnect();
 
 	virtual void OnConnect();
@@ -39,6 +39,7 @@ protected:
 
 	virtual int ResetOperation(int nErrorCode);
 
+	virtual void ResetSocket();
 	virtual void ResetHttpData(CHttpOpData* pData);
 
 	int OpenFile( CHttpFileTransferOpData* pData);
@@ -48,15 +49,15 @@ protected:
 
 	int ProcessData(char* p, int len);
 
-	char* m_pRecvBuffer;
-	unsigned int m_recvBufferPos;
+	char* m_pRecvBuffer{};
+	unsigned int m_recvBufferPos{};
 	static const unsigned int m_recvBufferLen = 4096;
 
-	CHttpOpData* m_pHttpOpData;
+	CHttpOpData* m_pHttpOpData{};
 
-	CTlsSocket* m_pTlsSocket;
+	CTlsSocket* m_pTlsSocket{};
 
-	wxURI m_current_uri;
+	fz::uri m_current_uri;
 };
 
-#endif //__HTTPCONTROLSOCKET_H__
+#endif
