@@ -4,7 +4,7 @@
 
 #include <libfilezilla/file.hpp>
 
-#include <wx/log.h>
+#include <assert.h>
 
 CIOThread::CIOThread()
 {
@@ -34,9 +34,9 @@ void CIOThread::Close()
 	}
 }
 
-bool CIOThread::Create(std::unique_ptr<fz::file> && pFile, bool read, bool binary)
+bool CIOThread::Create(fz::thread_pool& pool, std::unique_ptr<fz::file> && pFile, bool read, bool binary)
 {
-	wxASSERT(pFile);
+	assert(pFile);
 
 	Close();
 
@@ -56,24 +56,27 @@ bool CIOThread::Create(std::unique_ptr<fz::file> && pFile, bool read, bool binar
 #ifdef SIMULATE_IO
 	size_ = m_pFile->size();
 #endif
-	
+
 	m_running = true;
-	
-	if (!run()) {
+
+	thread_ = pool.spawn([this]() { entry(); });
+	if (!thread_) {
 		m_running = false;
 		return false;
 	}
-	
+
 	return true;
 }
 
 void CIOThread::entry()
 {
 	if (m_read) {
+		fz::scoped_lock l(m_mutex);
 		while (m_running) {
-			auto len = ReadFromFile(m_buffers[m_curThreadBuf], BUFFERSIZE);
 
-			fz::scoped_lock l(m_mutex);
+			l.unlock();
+			auto len = ReadFromFile(m_buffers[m_curThreadBuf], BUFFERSIZE);
+			l.lock();
 
 			if (m_appWaiting) {
 				if (!m_evtHandler) {
@@ -84,13 +87,13 @@ void CIOThread::entry()
 				m_evtHandler->send_event<CIOThreadEvent>();
 			}
 
-			if (len == -1) {
+			if (len <= -1) {
 				m_error = true;
 				m_running = false;
 				break;
 			}
 
-			m_bufferLens[m_curThreadBuf] = len;
+			m_bufferLens[m_curThreadBuf] = static_cast<unsigned int>(len);
 
 			if (!len) {
 				m_running = false;
@@ -189,7 +192,7 @@ int CIOThread::GetNextWriteBuffer(char** pBuffer)
 
 bool CIOThread::Finalize(int len)
 {
-	wxASSERT(m_pFile);
+	assert(m_pFile);
 
 	Destroy();
 
@@ -205,7 +208,7 @@ bool CIOThread::Finalize(int len)
 	if (!WriteToFile(m_buffers[m_curAppBuf], len))
 		return false;
 
-#ifndef __WXMSW__
+#ifndef FZ_WINDOWS
 	if (!m_binary && m_wasCarriageReturn) {
 		const char CR = '\r';
 		if (m_pFile->write(&CR, 1) != 1)
@@ -220,7 +223,7 @@ bool CIOThread::Finalize(int len)
 
 int CIOThread::GetNextReadBuffer(char** pBuffer)
 {
-	wxASSERT(m_read);
+	assert(m_read);
 
 	int newBuf = (m_curAppBuf + 1) % BUFFERCOUNT;
 
@@ -261,7 +264,7 @@ void CIOThread::Destroy()
 		}
 	}
 
-	join();
+	thread_.join();
 }
 
 int64_t CIOThread::ReadFromFile(char* pBuffer, int64_t maxLen)
@@ -277,12 +280,12 @@ int64_t CIOThread::ReadFromFile(char* pBuffer, int64_t maxLen)
 	// In binary mode, no conversion has to be done.
 	// Also, under Windows the native newline format is already identical
 	// to the newline format of the FTP protocol
-#ifndef __WXMSW__
+#ifndef FZ_WINDOWS
 	if (m_binary)
 #endif
 		return m_pFile->read(pBuffer, maxLen);
 
-#ifndef __WXMSW__
+#ifndef FZ_WINDOWS
 
 	// In the worst case, length will doubled: If reading
 	// only LFs from the file
@@ -324,11 +327,11 @@ bool CIOThread::WriteToFile(char* pBuffer, int64_t len)
 	// In binary mode, no conversion has to be done.
 	// Also, under Windows the native newline format is already identical
 	// to the newline format of the FTP protocol
-#ifndef __WXMSW__
+#ifndef FZ_WINDOWS
 	if (m_binary) {
 #endif
 		return DoWrite(pBuffer, len);
-#ifndef __WXMSW__
+#ifndef FZ_WINDOWS
 	}
 	else {
 
@@ -384,9 +387,9 @@ bool CIOThread::DoWrite(const char* pBuffer, int64_t len)
 		return true;
 	}
 
-	int code = wxSysErrorCode();
+	auto err = GetSystemErrorCode();
 
-	std::wstring const error = wxSysErrorMsg(code);
+	std::wstring const error = fz::to_wstring(GetSystemErrorDescription(err));
 
 	fz::scoped_lock locker(m_mutex);
 	m_error_description = error;
