@@ -2,9 +2,10 @@
 #define __IOTHREAD_H__
 
 #include <libfilezilla/event.hpp>
+#include <libfilezilla/thread_pool.hpp>
 
-#define BUFFERCOUNT 5
-#define BUFFERSIZE 128*1024
+#define BUFFERCOUNT 8
+#define BUFFERSIZE 256*1024
 
 // Does not actually read from or write to file
 // Useful for benchmarks to avoid IO bottleneck
@@ -25,14 +26,14 @@ namespace fz {
 class file;
 }
 
-class CIOThread final : protected fz::thread
+class CIOThread final
 {
 public:
 	CIOThread();
-	virtual ~CIOThread();
+	~CIOThread();
 
-	bool Create(std::unique_ptr<fz::file> && pFile, bool read, bool binary);
-	virtual void Destroy(); // Only call that might be blocking
+	bool Create(fz::thread_pool& pool, std::unique_ptr<fz::file> && pFile, bool read, bool binary);
+	void Destroy(); // Only call that might be blocking
 
 	// Call before first call to one of the GetNext*Buffer functions
 	// This handler will receive the CIOThreadEvent events. The events
@@ -55,12 +56,12 @@ public:
 
 	bool Finalize(int len);
 
-	wxString GetError();
+	std::wstring GetError();
 
 private:
 	void Close();
 
-	virtual void entry();
+	void entry();
 
 	int64_t ReadFromFile(char* pBuffer, int64_t maxLen);
 	bool WriteToFile(char* pBuffer, int64_t len);
@@ -75,7 +76,7 @@ private:
 	char* m_buffers[BUFFERCOUNT];
 	unsigned int m_bufferLens[BUFFERCOUNT];
 
-	fz::mutex m_mutex;
+	fz::mutex m_mutex{false};
 	fz::condition m_condition;
 
 	int m_curAppBuf{};
@@ -88,11 +89,13 @@ private:
 
 	bool m_wasCarriageReturn{};
 
-	wxString m_error_description;
+	std::wstring m_error_description;
 
 #ifdef SIMULATE_IO
 	int64_t size_{};
 #endif
+
+	fz::async_task thread_;
 };
 
 #endif //__IOTHREAD_H__
