@@ -25,9 +25,9 @@ public:
 
 		proxytype_count
 	};
-	static wxString Name(ProxyType t);
+	static std::wstring Name(ProxyType t);
 
-	int Handshake(ProxyType type, const wxString& host, unsigned int port, const wxString& user, const wxString& pass);
+	int Handshake(ProxyType type, std::wstring const& host, unsigned int port, std::wstring const& user, std::wstring const& pass);
 
 	ProxyState GetState() const { return m_proxyState; }
 
@@ -42,18 +42,18 @@ public:
 	bool Detached() const { return m_pSocket == 0; }
 
 	ProxyType GetProxyType() const { return m_proxyType; }
-	wxString GetUser() const { return m_user; }
-	wxString GetPass() const { return m_pass; }
+	std::wstring GetUser() const;
+	std::wstring GetPass() const;
 
 protected:
 	CSocket* m_pSocket;
 	CControlSocket* m_pOwner;
 
 	ProxyType m_proxyType{unknown};
-	wxString m_host;
+	std::wstring m_host;
 	int m_port{};
-	wxString m_user;
-	wxString m_pass;
+	std::string m_user;
+	std::string m_pass;
 
 	ProxyState m_proxyState{noconn};
 
@@ -68,7 +68,7 @@ protected:
 
 	virtual void operator()(fz::event_base const& ev);
 	void OnSocketEvent(CSocketEventSource* source, SocketEventType t, int error);
-	void OnHostAddress(CSocketEventSource* source, wxString const& address);
+	void OnHostAddress(CSocketEventSource* source, std::string const& address);
 
 	void OnReceive();
 	void OnSend();
