@@ -50,8 +50,3 @@ void CLatencyMeasurement::Reset()
 	m_measurements = 0;
 	m_start = fz::monotonic_clock();
 }
-
-void CLatencyMeasurement::cb()
-{
-	Stop();
-}
