@@ -3,7 +3,7 @@
 
 #include "socket.h"
 
-class CLatencyMeasurement final : public CCallback
+class CLatencyMeasurement final
 {
 public:
 	CLatencyMeasurement();
@@ -20,15 +20,13 @@ public:
 
 	void Reset();
 
-	virtual void cb();
-
 protected:
 	fz::monotonic_clock m_start;
 
 	int64_t m_summed_latency{};
 	int m_measurements{};
 
-	mutable fz::mutex m_sync;
+	mutable fz::mutex m_sync{false};
 };
 
 #endif
