@@ -1,12 +1,14 @@
-#include <wx/defs.h>
 #include <libfilezilla/libfilezilla.hpp>
 #ifdef FZ_WINDOWS
   #include <libfilezilla/private/windows.hpp>
   #include <winsock2.h>
   #include <ws2tcpip.h>
+  #include <mstcpip.h>
 #endif
 #include <filezilla.h>
+#include <libfilezilla/format.hpp>
 #include <libfilezilla/mutex.hpp>
+#include <libfilezilla/thread_pool.hpp>
 #include "socket.h"
 #ifndef FZ_WINDOWS
   #define mutex mutex_override // Sadly on some platforms system headers include conflicting names
@@ -14,6 +16,7 @@
   #include <sys/socket.h>
   #include <netdb.h>
   #include <fcntl.h>
+  #include <unistd.h>
   #include <arpa/inet.h>
   #include <netinet/in.h>
   #include <netinet/tcp.h>
@@ -23,6 +26,8 @@
   #undef mutex
 #endif
 
+#include <string.h>
+
 // Fixups needed on FreeBSD
 #if !defined(EAI_ADDRFAMILY) && defined(EAI_FAMILY)
   #define EAI_ADDRFAMILY EAI_FAMILY
@@ -164,32 +169,9 @@ int GetLastSocketError()
 #else
 inline int GetLastSocketError() { return errno; }
 #endif
-
-#ifdef ERRORCODETEST
-class CErrorCodeTest
-{
-public:
-	CErrorCodeTest()
-	{
-		int errors[] = {
-			0
-		};
-		for (int i = 0; errors[i]; ++i) {
-#ifdef FZ_WINDOWS
-			int code = ConvertMSWErrorCode(errors[i]);
-#else
-			int code = errors[i];
-#endif
-			if (CSocket::GetErrorDescription(code).Len() < 15)
-				wxMessageBoxEx(CSocket::GetErrorDescription(code));
-		}
-
-	}
-};
-#endif
 }
 
-class CSocketThread final : protected fz::thread
+class CSocketThread final
 {
 	friend class CSocket;
 public:
@@ -207,9 +189,9 @@ public:
 		}
 	}
 
-	virtual ~CSocketThread()
+	~CSocketThread()
 	{
-		join();
+		thread_.join();
 #ifdef FZ_WINDOWS
 		if (m_sync_event != WSA_INVALID_EVENT)
 			WSACloseEvent(m_sync_event);
@@ -237,20 +219,19 @@ public:
 		m_waiting = 0;
 	}
 
-	int Connect(wxString const& bind)
+	int Connect(std::string const& bind)
 	{
-		wxASSERT(m_pSocket);
-
-		auto const hostBuf = m_pSocket->m_host.mb_str();
-		auto const bindBuf = bind.mb_str();
-		if (!hostBuf || !bindBuf) {
-			m_bind.clear();
-			m_host.clear();
-			m_port.clear();
+		assert(m_pSocket);
+		if (!m_pSocket) {
 			return EINVAL;
 		}
-		m_host = hostBuf;
-		m_bind = bindBuf;
+
+		m_host = fz::to_utf8(m_pSocket->m_host);
+		if (m_host.empty()) {
+			return EINVAL;
+		}
+
+		m_bind = bind;
 
 		// Connect method of CSocket ensures port is in range
 		char tmp[7];
@@ -267,7 +248,7 @@ public:
 	{
 		if (m_started) {
 			fz::scoped_lock l(m_sync);
-			wxASSERT(m_threadwait);
+			assert(m_threadwait);
 			m_waiting = 0;
 			WakeupThread(l);
 			return 0;
@@ -285,11 +266,9 @@ public:
 		}
 #endif
 
-		if (!run()) {
-			return 1;
-		}
+		thread_ = m_pSocket->thread_pool_.spawn([this]() { entry(); });
 
-		return 0;
+		return thread_ ? 0 : 1;
 	}
 
 	// Cancels select or idle wait
@@ -378,7 +357,7 @@ protected:
 			(void)bind(fd, &bindAddr.sockaddr, sizeof(bindAddr));
 		}
 
-		CSocket::DoSetFlags(fd, m_pSocket->m_flags, m_pSocket->m_flags);
+		CSocket::DoSetFlags(fd, m_pSocket->m_flags, m_pSocket->m_flags, m_pSocket->m_keepalive_interval);
 		CSocket::DoSetBufferSizes(fd, m_pSocket->m_buffer_sizes[0], m_pSocket->m_buffer_sizes[1]);
 
 		int res = connect(fd, addr.ai_addr, addr.ai_addrlen);
@@ -554,16 +533,21 @@ protected:
 		for (;;) {
 #ifdef FZ_WINDOWS
 			int wait_events = FD_CLOSE;
-			if (m_waiting & WAIT_CONNECT)
+			if (m_waiting & WAIT_CONNECT) {
 				wait_events |= FD_CONNECT;
-			if (m_waiting & WAIT_READ)
+			}
+			if (m_waiting & WAIT_READ) {
 				wait_events |= FD_READ;
-			if (m_waiting & WAIT_WRITE)
+			}
+			if (m_waiting & WAIT_WRITE) {
 				wait_events |= FD_WRITE;
-			if (m_waiting & WAIT_ACCEPT)
+			}
+			if (m_waiting & WAIT_ACCEPT) {
 				wait_events |= FD_ACCEPT;
-			if (m_waiting & WAIT_CLOSE)
+			}
+			if (m_waiting & WAIT_CLOSE) {
 				wait_events |= FD_CLOSE;
+			}
 			WSAEventSelect(m_pSocket->m_fd, m_sync_event, wait_events);
 			l.unlock();
 			WSAWaitForMultipleEvents(1, &m_sync_event, false, WSA_INFINITE, false);
@@ -625,17 +609,19 @@ protected:
 			FD_ZERO(&writefds);
 
 			FD_SET(m_pipe[0], &readfds);
-			if (!(m_waiting & WAIT_CONNECT))
+			if (!(m_waiting & WAIT_CONNECT)) {
 				FD_SET(m_pSocket->m_fd, &readfds);
+			}
 
-			if (m_waiting & (WAIT_WRITE | WAIT_CONNECT))
+			if (m_waiting & (WAIT_WRITE | WAIT_CONNECT)) {
 				FD_SET(m_pSocket->m_fd, &writefds);
+			}
 
-			int max = wxMax(m_pipe[0], m_pSocket->m_fd) + 1;
+			int maxfd = std::max(m_pipe[0], m_pSocket->m_fd) + 1;
 
 			l.unlock();
 
-			int res = select(max, &readfds, &writefds, 0, 0);
+			int res = select(maxfd, &readfds, &writefds, 0, 0);
 
 			l.lock();
 
@@ -649,15 +635,15 @@ protected:
 				return false;
 			}
 
-			if (!res)
+			if (!res) {
 				continue;
+			}
 			if (res == -1) {
 				res = errno;
-				//printf("select failed: %s\n", (const char *)CSocket::GetErrorDescription(res).mb_str());
-				//fflush(stdout);
 
-				if (res == EINTR)
+				if (res == EINTR) {
 					continue;
+				}
 
 				return false;
 			}
@@ -666,9 +652,10 @@ protected:
 				if (FD_ISSET(m_pSocket->m_fd, &writefds)) {
 					int error;
 					socklen_t len = sizeof(error);
-					int res = getsockopt(m_pSocket->m_fd, SOL_SOCKET, SO_ERROR, &error, &len);
-					if (res)
+					int getsockopt_res = getsockopt(m_pSocket->m_fd, SOL_SOCKET, SO_ERROR, &error, &len);
+					if (getsockopt_res) {
 						error = errno;
+					}
 					m_triggered |= WAIT_CONNECT;
 					m_triggered_errors[0] = error;
 					m_waiting &= ~WAIT_CONNECT;
@@ -693,8 +680,9 @@ protected:
 				}
 			}
 
-			if (m_triggered || !m_waiting)
+			if (m_triggered || !m_waiting) {
 				return true;
+			}
 #endif
 		}
 	}
@@ -704,8 +692,6 @@ protected:
 		if (!m_pSocket || !m_pSocket->m_pEvtHandler)
 			return;
 		if (m_triggered & WAIT_READ) {
-			if (m_pSocket->m_synchronous_read_cb)
-				m_pSocket->m_synchronous_read_cb->cb();
 			m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, SocketEventType::read, m_triggered_errors[1]);
 			m_triggered &= ~WAIT_READ;
 		}
@@ -765,7 +751,7 @@ protected:
 		return true;
 	}
 
-	virtual void entry()
+	void entry()
 	{
 		fz::scoped_lock l(m_sync);
 		for (;;) {
@@ -854,20 +840,15 @@ protected:
 
 	// Thread waits for instructions
 	bool m_threadwait{};
+
+	fz::async_task thread_;
 };
 
-CSocket::CSocket(fz::event_handler* pEvtHandler)
-	: m_pEvtHandler(pEvtHandler)
+CSocket::CSocket(fz::thread_pool & pool, fz::event_handler* pEvtHandler)
+	: thread_pool_(pool)
+	, m_pEvtHandler(pEvtHandler)
+	, m_keepalive_interval(fz::duration::from_hours(2))
 {
-#ifdef ERRORCODETEST
-	CErrorCodeTest test;
-#endif
-	m_fd = -1;
-	m_state = none;
-	m_pSocketThread = 0;
-	m_flags = 0;
-
-	m_port = 0;
 	m_family = AF_UNSPEC;
 
 	m_buffer_sizes[0] = -1;
@@ -879,15 +860,17 @@ CSocket::~CSocket()
 	if (m_state != none)
 		Close();
 
-	DetachThread();
+	if (m_pSocketThread) {
+		fz::scoped_lock l(m_pSocketThread->m_sync);
+		DetachThread(l);
+	}
 }
 
-void CSocket::DetachThread()
+void CSocket::DetachThread(fz::scoped_lock & l)
 {
 	if (!m_pSocketThread)
 		return;
 
-	fz::scoped_lock l(m_pSocketThread->m_sync);
 	m_pSocketThread->SetSocket(0, l);
 	if (m_pSocketThread->m_finished) {
 		m_pSocketThread->WakeupThread(l);
@@ -913,7 +896,7 @@ void CSocket::DetachThread()
 	Cleanup(false);
 }
 
-int CSocket::Connect(wxString const& host, unsigned int port, address_family family, wxString const& bind)
+int CSocket::Connect(fz::native_string const& host, unsigned int port, address_family family, std::string const& bind)
 {
 	if (m_state != none)
 		return EISCONN;
@@ -921,16 +904,22 @@ int CSocket::Connect(wxString const& host, unsigned int port, address_family fam
 	if (port < 1 || port > 65535)
 		return EINVAL;
 
+	if (host.empty()) {
+		return EINVAL;
+	}
+
+	int af{};
+
 	switch (family)
 	{
 	case unspec:
-		m_family = AF_UNSPEC;
+		af = AF_UNSPEC;
 		break;
 	case ipv4:
-		m_family = AF_INET;
+		af = AF_INET;
 		break;
 	case ipv6:
-		m_family = AF_INET6;
+		af = AF_INET6;
 		break;
 	default:
 		return EINVAL;
@@ -939,17 +928,9 @@ int CSocket::Connect(wxString const& host, unsigned int port, address_family fam
 	if (m_pSocketThread && m_pSocketThread->m_started) {
 		fz::scoped_lock l(m_pSocketThread->m_sync);
 		if (!m_pSocketThread->m_threadwait) {
-			m_pSocketThread->WakeupThread(l);
-			l.unlock();
-			// Wait a small amount of time
-			wxMilliSleep(100);
-
-			l.lock();
-			if (!m_pSocketThread->m_threadwait) {
-				// Inside a blocking call, e.g. getaddrinfo
-				l.unlock();
-				DetachThread();
-			}
+			// Possibly inside a blocking call, e.g. getaddrinfo.
+			// Detach the thread so that we can continue.
+			DetachThread(l);
 		}
 	}
 	if (!m_pSocketThread) {
@@ -957,6 +938,7 @@ int CSocket::Connect(wxString const& host, unsigned int port, address_family fam
 		m_pSocketThread->SetSocket(this);
 	}
 
+	m_family = af;
 	m_state = connecting;
 
 	m_host = host;
@@ -1018,113 +1000,113 @@ void CSocket::SetEventHandler(fz::event_handler* pEvtHandler)
 	}
 }
 
-#define ERRORDECL(c, desc) { c, _T(#c), desc },
+#define ERRORDECL(c, desc) { c, #c, desc },
 
 struct Error_table
 {
 	int code;
-	const wxChar* const name;
-	const wxChar* const description;
+	char const* const name;
+	char const* const description;
 };
 
 static Error_table const error_table[] =
 {
-	ERRORDECL(EACCES, TRANSLATE_T("Permission denied"))
-	ERRORDECL(EADDRINUSE, TRANSLATE_T("Local address in use"))
-	ERRORDECL(EAFNOSUPPORT, TRANSLATE_T("The specified address family is not supported"))
-	ERRORDECL(EINPROGRESS, TRANSLATE_T("Operation in progress"))
-	ERRORDECL(EINVAL, TRANSLATE_T("Invalid argument passed"))
-	ERRORDECL(EMFILE, TRANSLATE_T("Process file table overflow"))
-	ERRORDECL(ENFILE, TRANSLATE_T("System limit of open files exceeded"))
-	ERRORDECL(ENOBUFS, TRANSLATE_T("Out of memory"))
-	ERRORDECL(ENOMEM, TRANSLATE_T("Out of memory"))
-	ERRORDECL(EPERM, TRANSLATE_T("Permission denied"))
-	ERRORDECL(EPROTONOSUPPORT, TRANSLATE_T("Protocol not supported"))
-	ERRORDECL(EAGAIN, TRANSLATE_T("Resource temporarily unavailable"))
-	ERRORDECL(EALREADY, TRANSLATE_T("Operation already in progress"))
-	ERRORDECL(EBADF, TRANSLATE_T("Bad file descriptor"))
-	ERRORDECL(ECONNREFUSED, TRANSLATE_T("Connection refused by server"))
-	ERRORDECL(EFAULT, TRANSLATE_T("Socket address outside address space"))
-	ERRORDECL(EINTR, TRANSLATE_T("Interrupted by signal"))
-	ERRORDECL(EISCONN, TRANSLATE_T("Socket already connected"))
-	ERRORDECL(ENETUNREACH, TRANSLATE_T("Network unreachable"))
-	ERRORDECL(ENOTSOCK, TRANSLATE_T("File descriptor not a socket"))
-	ERRORDECL(ETIMEDOUT, TRANSLATE_T("Connection attempt timed out"))
-	ERRORDECL(EHOSTUNREACH, TRANSLATE_T("No route to host"))
-	ERRORDECL(ENOTCONN, TRANSLATE_T("Socket not connected"))
-	ERRORDECL(ENETRESET, TRANSLATE_T("Connection reset by network"))
-	ERRORDECL(EOPNOTSUPP, TRANSLATE_T("Operation not supported"))
-	ERRORDECL(ESHUTDOWN, TRANSLATE_T("Socket has been shut down"))
-	ERRORDECL(EMSGSIZE, TRANSLATE_T("Message too large"))
-	ERRORDECL(ECONNABORTED, TRANSLATE_T("Connection aborted"))
-	ERRORDECL(ECONNRESET, TRANSLATE_T("Connection reset by peer"))
-	ERRORDECL(EPIPE, TRANSLATE_T("Local endpoint has been closed"))
-	ERRORDECL(EHOSTDOWN, TRANSLATE_T("Host is down"))
+	ERRORDECL(EACCES, fztranslate_mark("Permission denied"))
+	ERRORDECL(EADDRINUSE, fztranslate_mark("Local address in use"))
+	ERRORDECL(EAFNOSUPPORT, fztranslate_mark("The specified address family is not supported"))
+	ERRORDECL(EINPROGRESS, fztranslate_mark("Operation in progress"))
+	ERRORDECL(EINVAL, fztranslate_mark("Invalid argument passed"))
+	ERRORDECL(EMFILE, fztranslate_mark("Process file table overflow"))
+	ERRORDECL(ENFILE, fztranslate_mark("System limit of open files exceeded"))
+	ERRORDECL(ENOBUFS, fztranslate_mark("Out of memory"))
+	ERRORDECL(ENOMEM, fztranslate_mark("Out of memory"))
+	ERRORDECL(EPERM, fztranslate_mark("Permission denied"))
+	ERRORDECL(EPROTONOSUPPORT, fztranslate_mark("Protocol not supported"))
+	ERRORDECL(EAGAIN, fztranslate_mark("Resource temporarily unavailable"))
+	ERRORDECL(EALREADY, fztranslate_mark("Operation already in progress"))
+	ERRORDECL(EBADF, fztranslate_mark("Bad file descriptor"))
+	ERRORDECL(ECONNREFUSED, fztranslate_mark("Connection refused by server"))
+	ERRORDECL(EFAULT, fztranslate_mark("Socket address outside address space"))
+	ERRORDECL(EINTR, fztranslate_mark("Interrupted by signal"))
+	ERRORDECL(EISCONN, fztranslate_mark("Socket already connected"))
+	ERRORDECL(ENETUNREACH, fztranslate_mark("Network unreachable"))
+	ERRORDECL(ENOTSOCK, fztranslate_mark("File descriptor not a socket"))
+	ERRORDECL(ETIMEDOUT, fztranslate_mark("Connection attempt timed out"))
+	ERRORDECL(EHOSTUNREACH, fztranslate_mark("No route to host"))
+	ERRORDECL(ENOTCONN, fztranslate_mark("Socket not connected"))
+	ERRORDECL(ENETRESET, fztranslate_mark("Connection reset by network"))
+	ERRORDECL(EOPNOTSUPP, fztranslate_mark("Operation not supported"))
+	ERRORDECL(ESHUTDOWN, fztranslate_mark("Socket has been shut down"))
+	ERRORDECL(EMSGSIZE, fztranslate_mark("Message too large"))
+	ERRORDECL(ECONNABORTED, fztranslate_mark("Connection aborted"))
+	ERRORDECL(ECONNRESET, fztranslate_mark("Connection reset by peer"))
+	ERRORDECL(EPIPE, fztranslate_mark("Local endpoint has been closed"))
+	ERRORDECL(EHOSTDOWN, fztranslate_mark("Host is down"))
 
 	// Getaddrinfo related
 #ifdef EAI_ADDRFAMILY
-	ERRORDECL(EAI_ADDRFAMILY, TRANSLATE_T("Network host does not have any network addresses in the requested address family"))
+	ERRORDECL(EAI_ADDRFAMILY, fztranslate_mark("Network host does not have any network addresses in the requested address family"))
 #endif
-	ERRORDECL(EAI_AGAIN, TRANSLATE_T("Temporary failure in name resolution"))
-	ERRORDECL(EAI_BADFLAGS, TRANSLATE_T("Invalid value for ai_flags"))
+	ERRORDECL(EAI_AGAIN, fztranslate_mark("Temporary failure in name resolution"))
+	ERRORDECL(EAI_BADFLAGS, fztranslate_mark("Invalid value for ai_flags"))
 #ifdef EAI_BADHINTS
-	ERRORDECL(EAI_BADHINTS, TRANSLATE_T("Invalid value for hints"))
+	ERRORDECL(EAI_BADHINTS, fztranslate_mark("Invalid value for hints"))
 #endif
-	ERRORDECL(EAI_FAIL, TRANSLATE_T("Nonrecoverable failure in name resolution"))
-	ERRORDECL(EAI_FAMILY, TRANSLATE_T("The ai_family member is not supported"))
-	ERRORDECL(EAI_MEMORY, TRANSLATE_T("Memory allocation failure"))
+	ERRORDECL(EAI_FAIL, fztranslate_mark("Nonrecoverable failure in name resolution"))
+	ERRORDECL(EAI_FAMILY, fztranslate_mark("The ai_family member is not supported"))
+	ERRORDECL(EAI_MEMORY, fztranslate_mark("Memory allocation failure"))
 #ifdef EAI_NODATA
-	ERRORDECL(EAI_NODATA, TRANSLATE_T("No address associated with nodename"))
+	ERRORDECL(EAI_NODATA, fztranslate_mark("No address associated with nodename"))
 #endif
-	ERRORDECL(EAI_NONAME, TRANSLATE_T("Neither nodename nor servname provided, or not known"))
+	ERRORDECL(EAI_NONAME, fztranslate_mark("Neither nodename nor servname provided, or not known"))
 #ifdef EAI_OVERFLOW
-	ERRORDECL(EAI_OVERFLOW, TRANSLATE_T("Argument buffer overflow"))
+	ERRORDECL(EAI_OVERFLOW, fztranslate_mark("Argument buffer overflow"))
 #endif
 #ifdef EAI_PROTOCOL
-	ERRORDECL(EAI_PROTOCOL, TRANSLATE_T("Resolved protocol is unknown"))
+	ERRORDECL(EAI_PROTOCOL, fztranslate_mark("Resolved protocol is unknown"))
 #endif
-	ERRORDECL(EAI_SERVICE, TRANSLATE_T("The servname parameter is not supported for ai_socktype"))
-	ERRORDECL(EAI_SOCKTYPE, TRANSLATE_T("The ai_socktype member is not supported"))
+	ERRORDECL(EAI_SERVICE, fztranslate_mark("The servname parameter is not supported for ai_socktype"))
+	ERRORDECL(EAI_SOCKTYPE, fztranslate_mark("The ai_socktype member is not supported"))
 #ifdef EAI_SYSTEM
-	ERRORDECL(EAI_SYSTEM, TRANSLATE_T("Other system error"))
+	ERRORDECL(EAI_SYSTEM, fztranslate_mark("Other system error"))
 #endif
 
 	// Codes that have no POSIX equivalence
 #ifdef FZ_WINDOWS
-	ERRORDECL(WSANOTINITIALISED, TRANSLATE_T("Not initialized, need to call WSAStartup"))
-	ERRORDECL(WSAENETDOWN, TRANSLATE_T("System's network subsystem has failed"))
-	ERRORDECL(WSAEPROTOTYPE, TRANSLATE_T("Protocol not supported on given socket type"))
-	ERRORDECL(WSAESOCKTNOSUPPORT, TRANSLATE_T("Socket type not supported for address family"))
-	ERRORDECL(WSAEADDRNOTAVAIL, TRANSLATE_T("Cannot assign requested address"))
-	ERRORDECL(ERROR_NETNAME_DELETED, TRANSLATE_T("The specified network name is no longer available"))
+	ERRORDECL(WSANOTINITIALISED, fztranslate_mark("Not initialized, need to call WSAStartup"))
+	ERRORDECL(WSAENETDOWN, fztranslate_mark("System's network subsystem has failed"))
+	ERRORDECL(WSAEPROTOTYPE, fztranslate_mark("Protocol not supported on given socket type"))
+	ERRORDECL(WSAESOCKTNOSUPPORT, fztranslate_mark("Socket type not supported for address family"))
+	ERRORDECL(WSAEADDRNOTAVAIL, fztranslate_mark("Cannot assign requested address"))
+	ERRORDECL(ERROR_NETNAME_DELETED, fztranslate_mark("The specified network name is no longer available"))
 #endif
 	{ 0, 0, 0 }
 };
 
-wxString CSocket::GetErrorString(int error)
+std::string CSocket::GetErrorString(int error)
 {
-	for (int i = 0; error_table[i].code; ++i)
-	{
-		if (error != error_table[i].code)
+	for (int i = 0; error_table[i].code; ++i) {
+		if (error != error_table[i].code) {
 			continue;
+		}
 
 		return error_table[i].name;
 	}
 
-	return wxString::Format(_T("%d"), error);
+	return fz::sprintf("%d", error);
 }
 
-wxString CSocket::GetErrorDescription(int error)
+fz::native_string CSocket::GetErrorDescription(int error)
 {
-	for (int i = 0; error_table[i].code; ++i)
-	{
-		if (error != error_table[i].code)
+	for (int i = 0; error_table[i].code; ++i) {
+		if (error != error_table[i].code) {
 			continue;
+		}
 
-		return wxString(error_table[i].name) + _T(" - ") + wxGetTranslation(error_table[i].description);
+		return fz::to_native(fz::to_native(std::string(error_table[i].name)) + fzT(" - ") + fz::to_native(fz::translate(error_table[i].description)));
 	}
 
-	return wxString::Format(_T("%d"), error);
+	return fz::sprintf(fzT("%d"), error);
 }
 
 int CSocket::Close()
@@ -1137,7 +1119,7 @@ int CSocket::Close()
 		m_pSocketThread->m_host.clear();
 		m_pSocketThread->m_port.clear();
 
-			m_pSocketThread->WakeupThread(l);
+		m_pSocketThread->WakeupThread(l);
 
 		CSocketThread::CloseSocketFd(fd);
 		m_state = none;
@@ -1276,40 +1258,41 @@ int CSocket::Write(const void* buffer, unsigned int size, int& error)
 	return res;
 }
 
-wxString CSocket::AddressToString(const struct sockaddr* addr, int addr_len, bool with_port /*=true*/, bool strip_zone_index/*=false*/)
+std::string CSocket::AddressToString(const struct sockaddr* addr, int addr_len, bool with_port, bool strip_zone_index)
 {
 	char hostbuf[NI_MAXHOST];
 	char portbuf[NI_MAXSERV];
 
 	int res = getnameinfo(addr, addr_len, hostbuf, NI_MAXHOST, portbuf, NI_MAXSERV, NI_NUMERICHOST | NI_NUMERICSERV);
 	if (res) // Should never fail
-		return wxString();
+		return std::string();
 
-	wxString host = wxString(hostbuf, wxConvLibc);
-	wxString port = wxString(portbuf, wxConvLibc);
+	std::string host = hostbuf;
+	std::string port = portbuf;
 
 	// IPv6 uses colons as separator, need to enclose address
 	// to avoid ambiguity if also showing port
 	if (addr->sa_family == AF_INET6) {
 		if (strip_zone_index) {
-			int pos = host.Find('%');
-			if (pos != -1)
-				host.Truncate(pos);
+			auto pos = host.find('%');
+			if (pos != std::string::npos) {
+				host = host.substr(0, pos);
+			}
 		}
 		if (with_port)
-			host = _T("[") + host + _T("]");
+			host = "[" + host + "]";
 	}
 
 	if (with_port)
-		return host + _T(":") + port;
+		return host + ":" + port;
 	else
 		return host;
 }
 
-wxString CSocket::AddressToString(char const* buf, int buf_len)
+std::string CSocket::AddressToString(char const* buf, int buf_len)
 {
 	if (buf_len != 4 && buf_len != 16) {
-		return wxString();
+		return std::string();
 	}
 
 	sockaddr_u addr;
@@ -1322,28 +1305,29 @@ wxString CSocket::AddressToString(char const* buf, int buf_len)
 		addr.in4.sin_family = AF_INET;
 	}
 
-
 	return AddressToString(&addr.sockaddr, sizeof(addr), false, true);
 }
 
-wxString CSocket::GetLocalIP(bool strip_zone_index /*=false*/) const
+std::string CSocket::GetLocalIP(bool strip_zone_index) const
 {
 	struct sockaddr_storage addr;
 	socklen_t addr_len = sizeof(addr);
 	int res = getsockname(m_fd, (sockaddr*)&addr, &addr_len);
-	if (res)
-		return wxString();
+	if (res) {
+		return std::string();
+	}
 
 	return AddressToString((sockaddr *)&addr, addr_len, false, strip_zone_index);
 }
 
-wxString CSocket::GetPeerIP(bool strip_zone_index /*=false*/) const
+std::string CSocket::GetPeerIP(bool strip_zone_index) const
 {
 	struct sockaddr_storage addr;
 	socklen_t addr_len = sizeof(addr);
 	int res = getpeername(m_fd, (sockaddr*)&addr, &addr_len);
-	if (res)
-		return wxString();
+	if (res) {
+		return std::string();
+	}
 
 	return AddressToString((sockaddr *)&addr, addr_len, false, strip_zone_index);
 }
@@ -1353,8 +1337,9 @@ CSocket::address_family CSocket::GetAddressFamily() const
 	sockaddr_u addr;
 	socklen_t addr_len = sizeof(addr);
 	int res = getsockname(m_fd, &addr.sockaddr, &addr_len);
-	if (res)
+	if (res) {
 		return unspec;
+	}
 
 	switch (addr.sockaddr.sa_family)
 	{
@@ -1519,7 +1504,7 @@ CSocket* CSocket::Accept(int &error)
 
 	DoSetBufferSizes(fd, m_buffer_sizes[0], m_buffer_sizes[1]);
 
-	CSocket* pSocket = new CSocket(0);
+	CSocket* pSocket = new CSocket(thread_pool_, 0);
 	pSocket->m_state = connected;
 	pSocket->m_fd = fd;
 	pSocket->m_pSocketThread = new CSocketThread();
@@ -1553,18 +1538,21 @@ int CSocket::SetNonblocking(int fd)
 
 void CSocket::SetFlags(int flags)
 {
-	if (m_pSocketThread)
+	if (m_pSocketThread) {
 		m_pSocketThread->m_sync.lock();
+	}
 
-	if (m_fd != -1)
-		DoSetFlags(m_fd, flags, flags ^ m_flags);
+	if (m_fd != -1) {
+		DoSetFlags(m_fd, flags, flags ^ m_flags, m_keepalive_interval);
+	}
 	m_flags = flags;
 
-	if (m_pSocketThread)
+	if (m_pSocketThread) {
 		m_pSocketThread->m_sync.unlock();
+	}
 }
 
-int CSocket::DoSetFlags(int fd, int flags, int flags_mask)
+int CSocket::DoSetFlags(int fd, int flags, int flags_mask, fz::duration const& keepalive_interval)
 {
 	if (flags_mask & flag_nodelay) {
 		const int value = (flags & flag_nodelay) ? 1 : 0;
@@ -1574,28 +1562,47 @@ int CSocket::DoSetFlags(int fd, int flags, int flags_mask)
 		}
 	}
 	if (flags_mask & flag_keepalive) {
+#if FZ_WINDOWS
+		tcp_keepalive v{};
+		v.onoff = (flags & flag_keepalive) ? 1 : 0;
+		v.keepalivetime = keepalive_interval.get_milliseconds();
+		v.keepaliveinterval = 1000;
+		DWORD tmp{};
+		int res = WSAIoctl(fd, SIO_KEEPALIVE_VALS, &v, sizeof(v), 0, 0, &tmp, 0, 0);
+		if (res != 0) {
+			return GetLastSocketError();
+		}
+#else
 		const int value = (flags & flag_keepalive) ? 1 : 0;
 		int res = setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (const char*)&value, sizeof(value));
 		if (res != 0) {
 			return GetLastSocketError();
 		}
+#ifdef TCP_KEEPIDLE
+		int const idle = keepalive_interval.get_seconds();
+		res = setsockopt(fd, IPPROTO_TCP, TCP_KEEPINTVL, (const char*)&idle, sizeof(idle));
+		if (res != 0) {
+			return GetLastSocketError();
+		}
+#endif
+#endif
 	}
 
 	return 0;
 }
 
-int CSocket::SetBufferSizes(int size_read, int size_write)
+int CSocket::SetBufferSizes(int size_receive, int size_send)
 {
 	int ret = 0;
 
 	if (m_pSocketThread)
 		m_pSocketThread->m_sync.lock();
 
-	m_buffer_sizes[0] = size_read;
-	m_buffer_sizes[1] = size_write;
+	m_buffer_sizes[0] = size_receive;
+	m_buffer_sizes[1] = size_send;
 
 	if (m_fd != -1)
-		ret = DoSetBufferSizes(m_fd, size_read, size_write);
+		ret = DoSetBufferSizes(m_fd, size_receive, size_send);
 
 	if (m_pSocketThread)
 		m_pSocketThread->m_sync.unlock();
@@ -1603,10 +1610,45 @@ int CSocket::SetBufferSizes(int size_read, int size_write)
 	return ret;
 }
 
+int CSocket::GetIdealSendBufferSize()
+{
+	int size = -1;
+
+#ifdef FZ_WINDOWS
+	if (m_pSocketThread)
+		m_pSocketThread->m_sync.lock();
+
+	if (m_fd != -1) {
+		// MSDN says this:
+		// "Dynamic send buffering for TCP was added on Windows 7 and Windows
+		// Server 2008 R2.By default, dynamic send buffering for TCP is
+		// enabled unless an application sets the SO_SNDBUF socket option on
+		// the stream socket"
+		//
+		// Guess what: It doesn't do it by itself. Programs need to
+		// periodically and manually update SO_SNDBUF based on what
+		// SIO_IDEAL_SEND_BACKLOG_QUERY returns.
+#ifndef SIO_IDEAL_SEND_BACKLOG_QUERY
+#define SIO_IDEAL_SEND_BACKLOG_QUERY 0x4004747b
+#endif
+		ULONG v{};
+		DWORD outlen{};
+		if (!WSAIoctl(m_fd, SIO_IDEAL_SEND_BACKLOG_QUERY, 0, 0, &v, sizeof(v), &outlen, 0, 0)) {
+			size = v;
+		}
+	}
+
+	if (m_pSocketThread)
+		m_pSocketThread->m_sync.unlock();
+#endif
+
+	return size;
+}
+
+
 int CSocket::DoSetBufferSizes(int fd, int size_read, int size_write)
 {
 	int ret = 0;
-
 	if (size_read != -1) {
 		int res = setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (const char*)&size_read, sizeof(size_read));
 		if (res != 0) {
@@ -1624,18 +1666,27 @@ int CSocket::DoSetBufferSizes(int fd, int size_read, int size_write)
 	return ret;
 }
 
-void CSocket::SetSynchronousReadCallback(CCallback* cb)
-{
-	if (m_pSocketThread)
-		m_pSocketThread->m_sync.lock();
-
-	m_synchronous_read_cb = cb;
-
-	if (m_pSocketThread)
-		m_pSocketThread->m_sync.unlock();
-}
-
-wxString CSocket::GetPeerHost() const
+fz::native_string CSocket::GetPeerHost() const
 {
 	return m_host;
 }
+
+void CSocket::SetKeepaliveInterval(fz::duration const& d)
+{
+	if (d < fz::duration::from_minutes(1)) {
+		return;
+	}
+
+	if (m_pSocketThread) {
+		m_pSocketThread->m_sync.lock();
+	}
+
+	m_keepalive_interval = d;
+	if (m_fd != -1) {
+		DoSetFlags(m_fd, m_flags, flag_keepalive, m_keepalive_interval);
+	}
+
+	if (m_pSocketThread) {
+		m_pSocketThread->m_sync.unlock();
+	}
+}
