@@ -7,6 +7,10 @@
 
 #include <gnutls/x509.h>
 
+#include <algorithm>
+
+#include <string.h>
+
 #if FZ_USE_GNUTLS_SYSTEM_CIPHERS
 char const ciphers[] = "@SYSTEM";
 #else
@@ -19,14 +23,110 @@ char const ciphers[] = "SECURE256:+SECURE128:-ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-A
 CControlSocket* pLoggingControlSocket;
 void log_func(int level, const char* msg)
 {
-	if (!msg || !pLoggingControlSocket)
+	if (!msg || !pLoggingControlSocket) {
 		return;
-	wxString s(msg, wxConvLocal);
-	s.Trim();
-	pLoggingControlSocket->LogMessage(MessageType::Debug_Debug, _T("tls: %d %s"), level, s);
+	}
+	std::wstring s = fz::to_wstring(msg);
+	fz::trim(s);
+	pLoggingControlSocket->LogMessage(MessageType::Debug_Debug, L"tls: %d %s", level, s);
 }
 #endif
 
+class CTlsSocketCallbacks
+{
+public:
+	static int handshake_hook_func(gnutls_session_t session, unsigned int htype, unsigned int post, unsigned int incoming)
+	{
+		if (!session) {
+			return 0;
+		}
+		auto* tls = reinterpret_cast<CTlsSocket*>(gnutls_session_get_ptr(session));
+		if (!tls) {
+			return 0;
+		}
+
+		char const* prefix;
+		if (incoming) {
+			if (post) {
+				prefix = "Processed";
+			}
+			else {
+				prefix = "Received";
+			}
+		}
+		else {
+			if (post) {
+				prefix = "Sent";
+			}
+			else {
+				prefix = "About to send";
+			}
+		}
+
+		char const* name = gnutls_handshake_description_get_name(static_cast<gnutls_handshake_description_t>(htype));
+
+		tls->m_pOwner->LogMessage(MessageType::Debug_Debug, _T("TLS handshake: %s %s"), prefix, name);
+
+		return 0;
+	}
+};
+
+namespace {
+extern "C" int handshake_hook_func(gnutls_session_t session, unsigned int htype, unsigned int post, unsigned int incoming, gnutls_datum_t const*)
+{
+	return CTlsSocketCallbacks::handshake_hook_func(session, htype, post, incoming);
+}
+
+struct cert_list_holder final
+{
+	cert_list_holder() = default;
+	~cert_list_holder() {
+		for (unsigned int i = 0; i < certs_size; ++i) {
+			gnutls_x509_crt_deinit(certs[i]);
+		}
+		gnutls_free(certs);
+	}
+
+	cert_list_holder(cert_list_holder const&) = delete;
+	cert_list_holder& operator=(cert_list_holder const&) = delete;
+
+	gnutls_x509_crt_t * certs{};
+	unsigned int certs_size{};
+};
+
+struct datum_holder final : gnutls_datum_t
+{
+	datum_holder() {
+		data = 0;
+	}
+
+	~datum_holder() {
+		gnutls_free(data);
+	}
+
+	datum_holder(datum_holder const&) = delete;
+	datum_holder& operator=(datum_holder const&) = delete;
+
+};
+
+void clone_cert(gnutls_x509_crt_t in, gnutls_x509_crt_t &out)
+{
+	gnutls_x509_crt_deinit(out);
+	out = 0;
+
+	if (in) {
+		datum_holder der;
+		if (gnutls_x509_crt_export2(in, GNUTLS_X509_FMT_DER, &der) == GNUTLS_E_SUCCESS) {
+			gnutls_x509_crt_init(&out);
+			if (gnutls_x509_crt_import(out, &der, GNUTLS_X509_FMT_DER) != GNUTLS_E_SUCCESS) {
+				gnutls_x509_crt_deinit(out);
+				out = 0;
+			}
+		}
+	}
+}
+}
+
 CTlsSocket::CTlsSocket(event_handler* pEvtHandler, CSocket& socket, CControlSocket* pOwner)
 	: event_handler(pOwner->event_loop_)
 	, CBackend(pEvtHandler)
@@ -72,6 +172,9 @@ bool CTlsSocket::Init()
 		return false;
 	}
 
+	// Disable time checks. We allow expired/not yet valid certificates, though only after explicit user confirmation
+	gnutls_certificate_set_verify_flags(m_certCredentials, gnutls_certificate_get_verify_flags(m_certCredentials) | GNUTLS_VERIFY_DISABLE_TIME_CHECKS | GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS);
+
 	if (!InitSession())
 		return false;
 
@@ -113,6 +216,9 @@ bool CTlsSocket::InitSession()
 		return false;
 	}
 
+	// For use in callbacks
+	gnutls_session_set_ptr(m_session, this);
+
 	// Even though the name gnutls_db_set_cache_expiration
 	// implies expiration of some cache, it also governs
 	// the actual session lifetime, independend whether the
@@ -177,7 +283,7 @@ void CTlsSocket::UninitSession()
 }
 
 
-void CTlsSocket::LogError(int code, const wxString& function, MessageType logLevel)
+void CTlsSocket::LogError(int code, std::wstring const& function, MessageType logLevel)
 {
 	if (code == GNUTLS_E_WARNING_ALERT_RECEIVED || code == GNUTLS_E_FATAL_ALERT_RECEIVED) {
 		PrintAlert(logLevel);
@@ -201,17 +307,20 @@ void CTlsSocket::LogError(int code, const wxString& function, MessageType logLev
 	else {
 		const char* error = gnutls_strerror(code);
 		if (error) {
-			wxString str(error, wxConvLocal);
-			if (function.empty())
-				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d: %s"), code, str);
-			else
-				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d in %s: %s"), code, function, str);
+			if (function.empty()) {
+				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d: %s"), code, error);
+			}
+			else {
+				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d in %s: %s"), code, function, error);
+			}
 		}
 		else {
-			if (function.empty())
+			if (function.empty()) {
 				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d"), code);
-			else
+			}
+			else {
 				m_pOwner->LogMessage(logLevel, _("GnuTLS error %d in %s"), code, function);
+			}
 		}
 	}
 }
@@ -221,11 +330,11 @@ void CTlsSocket::PrintAlert(MessageType logLevel)
 	gnutls_alert_description_t last_alert = gnutls_alert_get(m_session);
 	const char* alert = gnutls_alert_get_name(last_alert);
 	if (alert) {
-		wxString str(alert, wxConvLocal);
-		m_pOwner->LogMessage(logLevel, _("Received TLS alert from the server: %s (%d)"), str, last_alert);
+		m_pOwner->LogMessage(logLevel, _("Received TLS alert from the server: %s (%d)"), alert, last_alert);
 	}
-	else
+	else {
 		m_pOwner->LogMessage(logLevel, _("Received unknown TLS alert %d from the server"), last_alert);
+	}
 }
 
 ssize_t CTlsSocket::PushFunction(gnutls_transport_ptr_t ptr, const void* data, size_t len)
@@ -424,7 +533,7 @@ void CTlsSocket::OnSend()
 
 bool CTlsSocket::CopySessionData(const CTlsSocket* pPrimarySocket)
 {
-	gnutls_datum_t d;
+	datum_holder d;
 	int res = gnutls_session_get_data2(pPrimarySocket->m_session, &d);
 	if (res) {
 		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("gnutls_session_get_data2 on primary socket failed: %d"), res);
@@ -433,15 +542,16 @@ bool CTlsSocket::CopySessionData(const CTlsSocket* pPrimarySocket)
 
 	// Set session data
 	res = gnutls_session_set_data(m_session, d.data, d.size );
-	gnutls_free(d.data);
 	if (res) {
 		m_pOwner->LogMessage(MessageType::Debug_Info, _T("gnutls_session_set_data failed: %d. Going to reinitialize session."), res);
 		UninitSession();
-		if (!InitSession())
+		if (!InitSession()) {
 			return false;
+		}
 	}
-	else
+	else {
 		m_pOwner->LogMessage(MessageType::Debug_Info, _T("Trying to resume existing TLS session."));
+	}
 
 	return true;
 }
@@ -454,13 +564,21 @@ bool CTlsSocket::ResumedSession() const
 int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::Handshake()"));
-	wxASSERT(m_session);
+	if (!m_session) {
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Called CTlsSocket::Handshake without session"));
+		return FZ_REPLY_ERROR;
+	}
 
 	m_tlsState = TlsState::handshake;
 
-	wxString hostname;
+	fz::native_string hostname;
 
 	if (pPrimarySocket) {
+		if (!pPrimarySocket->m_session) {
+			m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Primary socket has no session"));
+			return FZ_REPLY_ERROR;
+		}
+
 		// Implicitly trust certificate of primary socket
 		unsigned int cert_list_size;
 		const gnutls_datum_t* const cert_list = gnutls_certificate_get_peers(pPrimarySocket->m_session, &cert_list_size);
@@ -482,24 +600,26 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 		hostname = m_socket.GetPeerHost();
 	}
 
-	if (!hostname.empty() && fz::get_address_type(hostname.ToStdWstring()) == fz::address_type::unknown) {
-		const wxWX2MBbuf utf8 = hostname.mb_str(wxConvUTF8);
-		if (utf8) {
-			int res = gnutls_server_name_set( m_session, GNUTLS_NAME_DNS, utf8, strlen(utf8) );
+	if (!hostname.empty() && fz::get_address_type(hostname) == fz::address_type::unknown) {
+		auto const utf8 = fz::to_utf8(hostname);
+		if (!utf8.empty()) {
+			int res = gnutls_server_name_set(m_session, GNUTLS_NAME_DNS, utf8.c_str(), utf8.size());
 			if (res) {
 				LogError(res, _T("gnutls_server_name_set"), MessageType::Debug_Warning );
 			}
 		}
 	}
 
+	gnutls_handshake_set_hook_function(m_session, GNUTLS_HANDSHAKE_ANY, GNUTLS_HOOK_BOTH, &handshake_hook_func);
+
 	return ContinueHandshake();
 }
 
 int CTlsSocket::ContinueHandshake()
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::ContinueHandshake()"));
-	wxASSERT(m_session);
-	wxASSERT(m_tlsState == TlsState::handshake);
+	assert(m_session);
+	assert(m_tlsState == TlsState::handshake);
 
 	int res = gnutls_handshake(m_session);
 	while (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) {
@@ -511,19 +631,21 @@ int CTlsSocket::ContinueHandshake()
 	if (!res) {
 		m_pOwner->LogMessage(MessageType::Debug_Info, _T("TLS Handshake successful"));
 
-		if (ResumedSession())
+		if (ResumedSession()) {
 			m_pOwner->LogMessage(MessageType::Debug_Info, _T("TLS Session resumed"));
+		}
 
-		const wxString protocol = GetProtocolName();
-		const wxString keyExchange = GetKeyExchange();
-		const wxString cipherName = GetCipherName();
-		const wxString macName = GetMacName();
+		std::wstring const protocol = GetProtocolName();
+		std::wstring const keyExchange = GetKeyExchange();
+		std::wstring const cipherName = GetCipherName();
+		std::wstring const macName = GetMacName();
 
 		m_pOwner->LogMessage(MessageType::Debug_Info, _T("Protocol: %s, Key exchange: %s, Cipher: %s, MAC: %s"), protocol, keyExchange, cipherName, macName);
 
 		res = VerifyCertificate();
-		if (res != FZ_REPLY_OK)
+		if (res != FZ_REPLY_OK) {
 			return res;
+		}
 
 		if (m_shutdown_requested) {
 			int error = Shutdown();
@@ -534,8 +656,9 @@ int CTlsSocket::ContinueHandshake()
 
 		return FZ_REPLY_OK;
 	}
-	else if (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED)
+	else if (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) {
 		return FZ_REPLY_WOULDBLOCK;
+	}
 
 	Failure(res, true);
 
@@ -558,7 +681,7 @@ int CTlsSocket::Read(void *buffer, unsigned int len, int& error)
 	}
 
 	if (m_peekDataLen) {
-		unsigned int min = wxMin(len, m_peekDataLen);
+		auto min = std::min(len, m_peekDataLen);
 		memcpy(buffer, m_peekData, min);
 
 		if (min == m_peekDataLen) {
@@ -700,7 +823,7 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 		m_canTriggerWrite = true;
 	}
 	if (m_lastReadFailed) {
-		wxASSERT(!m_peekData);
+		assert(!m_peekData);
 
 		m_peekDataLen = 65536;
 		m_peekData = new char[m_peekDataLen];
@@ -728,7 +851,7 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 	}
 }
 
-void CTlsSocket::Failure(int code, bool send_close, const wxString& function)
+void CTlsSocket::Failure(int code, bool send_close, std::wstring const& function)
 {
 	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::Failure(%d)"), code);
 	if (code) {
@@ -754,7 +877,7 @@ void CTlsSocket::Failure(int code, bool send_close, const wxString& function)
 int CTlsSocket::Peek(void *buffer, unsigned int len, int& error)
 {
 	if (m_peekData) {
-		int min = wxMin(len, m_peekDataLen);
+		auto min = std::min(len, m_peekDataLen);
 		memcpy(buffer, m_peekData, min);
 
 		error = 0;
@@ -855,34 +978,24 @@ void CTlsSocket::TrustCurrentCert(bool trusted)
 	Failure(0, true);
 }
 
-static wxString bin2hex(const unsigned char* in, size_t size)
+static std::wstring bin2hex(const unsigned char* in, size_t size)
 {
-	wxString str;
-	for (size_t i = 0; i < size; i++)
-	{
-		if (i)
-			str += _T(":");
-		str += wxString::Format(_T("%.2x"), (int)in[i]);
+	std::wstring str;
+	str.reserve(size * 3);
+	for (size_t i = 0; i < size; ++i) {
+		if (i) {
+			str += ':';
+		}
+		str += fz::int_to_hex_char<wchar_t>(in[i] >> 4);
+		str += fz::int_to_hex_char<wchar_t>(in[i] & 0xf);
 	}
 
 	return str;
 }
 
 
-bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
+bool CTlsSocket::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& out)
 {
-	gnutls_x509_crt_t cert;
-	if (gnutls_x509_crt_init(&cert)) {
-		m_pOwner->LogMessage(MessageType::Error, _("Could not initialize structure for peer certificates, gnutls_x509_crt_init failed"));
-		return false;
-	}
-
-	if (gnutls_x509_crt_import(cert, datum, GNUTLS_X509_FMT_DER)) {
-		m_pOwner->LogMessage(MessageType::Error, _("Could not import peer certificates, gnutls_x509_crt_import failed"));
-		gnutls_x509_crt_deinit(cert);
-		return false;
-	}
-
 	fz::datetime expirationTime(gnutls_x509_crt_get_expiration_time(cert), fz::datetime::seconds);
 	fz::datetime activationTime(gnutls_x509_crt_get_activation_time(cert), fz::datetime::seconds);
 
@@ -890,54 +1003,56 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 	unsigned char buffer[40];
 	size_t size = sizeof(buffer);
 	int res = gnutls_x509_crt_get_serial(cert, buffer, &size);
-	if( res != 0 ) {
+	if (res != 0) {
 		size = 0;
 	}
 
-	wxString serial = bin2hex(buffer, size);
+	std::wstring serial = bin2hex(buffer, size);
 
 	unsigned int pkBits;
 	int pkAlgo = gnutls_x509_crt_get_pk_algorithm(cert, &pkBits);
-	wxString pkAlgoName;
+	std::wstring pkAlgoName;
 	if (pkAlgo >= 0) {
 		const char* pAlgo = gnutls_pk_algorithm_get_name((gnutls_pk_algorithm_t)pkAlgo);
-		if (pAlgo)
-			pkAlgoName = wxString(pAlgo, wxConvUTF8);
+		if (pAlgo) {
+			pkAlgoName = fz::to_wstring_from_utf8(pAlgo);
+		}
 	}
 
 	int signAlgo = gnutls_x509_crt_get_signature_algorithm(cert);
-	wxString signAlgoName;
+	std::wstring signAlgoName;
 	if (signAlgo >= 0) {
 		const char* pAlgo = gnutls_sign_algorithm_get_name((gnutls_sign_algorithm_t)signAlgo);
-		if (pAlgo)
-			signAlgoName = wxString(pAlgo, wxConvUTF8);
+		if (pAlgo) {
+			signAlgoName = fz::to_wstring_from_utf8(pAlgo);
+		}
 	}
 
-	wxString subject, issuer;
+	std::wstring subject, issuer;
 
 	size = 0;
 	res = gnutls_x509_crt_get_dn(cert, 0, &size);
 	if (size) {
 		char* dn = new char[size + 1];
 		dn[size] = 0;
-		if (!(res = gnutls_x509_crt_get_dn(cert, dn, &size)))
-		{
+		if (!(res = gnutls_x509_crt_get_dn(cert, dn, &size))) {
 			dn[size] = 0;
-			subject = wxString(dn, wxConvUTF8);
+			subject = fz::to_wstring_from_utf8(dn);
 		}
-		else
+		else {
 			LogError(res, _T("gnutls_x509_crt_get_dn"));
+		}
 		delete [] dn;
 	}
-	else
+	else {
 		LogError(res, _T("gnutls_x509_crt_get_dn"));
+	}
 	if (subject.empty()) {
 		m_pOwner->LogMessage(MessageType::Error, _("Could not get distinguished name of certificate subject, gnutls_x509_get_dn failed"));
-		gnutls_x509_crt_deinit(cert);
 		return false;
 	}
 
-	std::vector<wxString> alt_subject_names = GetCertSubjectAltNames(cert);
+	std::vector<std::wstring> alt_subject_names = GetCertSubjectAltNames(cert);
 
 	size = 0;
 	res = gnutls_x509_crt_get_issuer_dn(cert, 0, &size);
@@ -946,22 +1061,23 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 		dn[size] = 0;
 		if (!(res = gnutls_x509_crt_get_issuer_dn(cert, dn, &size))) {
 			dn[size] = 0;
-			issuer = wxString(dn, wxConvUTF8);
+			issuer = fz::to_wstring_from_utf8(dn);
 		}
-		else
+		else {
 			LogError(res, _T("gnutls_x509_crt_get_issuer_dn"));
+		}
 		delete [] dn;
 	}
-	else
+	else {
 		LogError(res, _T("gnutls_x509_crt_get_issuer_dn"));
+	}
 	if (issuer.empty() ) {
 		m_pOwner->LogMessage(MessageType::Error, _("Could not get distinguished name of certificate issuer, gnutls_x509_get_issuer_dn failed"));
-		gnutls_x509_crt_deinit(cert);
 		return false;
 	}
 
-	wxString fingerprint_sha256;
-	wxString fingerprint_sha1;
+	std::wstring fingerprint_sha256;
+	std::wstring fingerprint_sha1;
 
 	unsigned char digest[100];
 	size = sizeof(digest) - 1;
@@ -975,10 +1091,15 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 		fingerprint_sha1 = bin2hex(digest, size);
 	}
 
-	gnutls_x509_crt_deinit(cert);
+	datum_holder der;
+	if (gnutls_x509_crt_export2(cert, GNUTLS_X509_FMT_DER, &der) != GNUTLS_E_SUCCESS || !der.data || !der.size) {
+		m_pOwner->LogMessage(MessageType::Error, _T("gnutls_x509_crt_get_issuer_dn"));
+		return false;
+	}
+	std::vector<uint8_t> data(der.data, der.data + der.size);
 
 	out = CCertificate(
-		datum->data, datum->size,
+		std::move(data),
 		activationTime, expirationTime,
 		serial,
 		pkAlgoName, pkBits,
@@ -987,15 +1108,15 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 		fingerprint_sha1,
 		issuer,
 		subject,
-		alt_subject_names);
+		std::move(alt_subject_names));
 
 	return true;
 }
 
 
-std::vector<wxString> CTlsSocket::GetCertSubjectAltNames(gnutls_x509_crt_t cert)
+std::vector<std::wstring> CTlsSocket::GetCertSubjectAltNames(gnutls_x509_crt_t cert)
 {
-	std::vector<wxString> ret;
+	std::vector<std::wstring> ret;
 
 	char san[4096];
 	for (unsigned int i = 0; i < 10000; ++i) { // I assume this is a sane limit
@@ -1009,19 +1130,18 @@ std::vector<wxString> CTlsSocket::GetCertSubjectAltNames(gnutls_x509_crt_t cert)
 		}
 
 		if (type_or_error == GNUTLS_SAN_DNSNAME || type_or_error == GNUTLS_SAN_RFC822NAME) {
-			wxString dns = wxString(san, wxConvUTF8);
+			std::wstring dns = fz::to_wstring_from_utf8(san);
 			if (!dns.empty()) {
-				ret.push_back(dns);
+				ret.emplace_back(std::move(dns));
 			}
 		}
 		else if (type_or_error == GNUTLS_SAN_IPADDRESS) {
-			wxString ip = CSocket::AddressToString(san, san_size);
+			std::wstring ip = fz::to_wstring(CSocket::AddressToString(san, san_size));
 			if (!ip.empty()) {
-				ret.push_back(ip);
+				ret.emplace_back(std::move(ip));
 			}
 		}
 	}
-
 	return ret;
 }
 
@@ -1085,6 +1205,66 @@ int CTlsSocket::GetAlgorithmWarnings()
 }
 
 
+bool CTlsSocket::GetSortedPeerCertificates(gnutls_x509_crt_t *& certs, unsigned int & certs_size)
+{
+	certs = 0;
+	certs_size = 0;
+
+	// First get unsorted list of peer certificates in DER
+	unsigned int cert_list_size;
+	const gnutls_datum_t* cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
+	if (!cert_list || !cert_list_size) {
+		m_pOwner->LogMessage(MessageType::Error, _("gnutls_certificate_get_peers returned no certificates"));
+		return false;
+	}
+
+	// Convert them all to PEM
+	gnutls_datum_t *pem_cert_list = new gnutls_datum_t[cert_list_size];
+	for (unsigned i = 0; i < cert_list_size; ++i) {
+		if (gnutls_pem_base64_encode2("CERTIFICATE", cert_list + i, pem_cert_list + i) != 0) {
+			for (unsigned int j = 0; j < i; ++j) {
+				gnutls_free(pem_cert_list[j].data);
+			}
+			delete [] pem_cert_list;
+			m_pOwner->LogMessage(MessageType::Error, _("gnutls_pem_base64_encode2 failed"));
+			return false;
+		}
+	}
+
+	// Concatenate them
+	gnutls_datum_t concated_certs{};
+	for (unsigned i = 0; i < cert_list_size; ++i) {
+		concated_certs.size += pem_cert_list[i].size;
+	}
+	concated_certs.data = new unsigned char[concated_certs.size];
+	concated_certs.size = 0;
+	for (unsigned i = 0; i < cert_list_size; ++i) {
+		memcpy(concated_certs.data + concated_certs.size, pem_cert_list[i].data, pem_cert_list[i].size);
+		concated_certs.size += pem_cert_list[i].size;
+	}
+
+	for (unsigned i = 0; i < cert_list_size; ++i) {
+		gnutls_free(pem_cert_list[i].data);
+	}
+	delete[] pem_cert_list;
+
+	// And now import the certificates
+	int res = gnutls_x509_crt_list_import2(&certs, &certs_size, &concated_certs, GNUTLS_X509_FMT_PEM, GNUTLS_X509_CRT_LIST_FAIL_IF_UNSORTED);
+	if (res == GNUTLS_E_CERTIFICATE_LIST_UNSORTED) {
+		m_pOwner->LogMessage(MessageType::Error, _("Server sent unsorted certificate chain in violation of the TLS specifications"));
+		res = gnutls_x509_crt_list_import2(&certs, &certs_size, &concated_certs, GNUTLS_X509_FMT_PEM, GNUTLS_X509_CRT_LIST_SORT);
+	}
+
+	delete[] concated_certs.data;
+
+	if (res != GNUTLS_E_SUCCESS) {
+		m_pOwner->LogMessage(MessageType::Error, _("Could not sort peer certificates"));
+		return false;
+	}
+
+	return true;
+}
+
 int CTlsSocket::VerifyCertificate()
 {
 	if (m_tlsState != TlsState::handshake) {
@@ -1100,8 +1280,41 @@ int CTlsSocket::VerifyCertificate()
 		return FZ_REPLY_ERROR;
 	}
 
+	cert_list_holder certs;
+	if (!GetSortedPeerCertificates(certs.certs, certs.certs_size)) {
+		Failure(0, true);
+		return FZ_REPLY_ERROR;
+	}
+
+	gnutls_x509_crt_t root{};
+	clone_cert(certs.certs[certs.certs_size - 1], root);
+	if (!root) {
+		m_pOwner->LogMessage(MessageType::Error, _("Could not copy certificate"));
+		Failure(0, true);
+		return FZ_REPLY_ERROR;
+	}
+
+	// Our trust-model is user-guided TOFU on the host's certificate.
+	//
+	// Here we validate the certificate chain sent by the server on the assumption
+	// that it's signed by a trusted CA.
+	//
+	// For now, add the highest certificate from the chain to trust list. Otherweise
+	// gnutls_certificate_verify_peers2 always stops with GNUTLS_CERT_SIGNER_NOT_FOUND
+	// at the highest certificate in the chain.
+	//
+	// Actual trust decision is done later by the user.
+	gnutls_x509_trust_list_t tlist;
+	gnutls_certificate_get_trust_list(m_certCredentials, &tlist);
+	if (gnutls_x509_trust_list_add_cas(tlist, &root, 1, 0) != 1) {
+		m_pOwner->LogMessage(MessageType::Error, _("Could not add certificate to temporary trust list"));
+		Failure(0, true);
+		return FZ_REPLY_ERROR;
+	}
+
 	unsigned int status = 0;
 	int const verifyResult = gnutls_certificate_verify_peers2(m_session, &status);
+
 	if (verifyResult < 0) {
 		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("gnutls_certificate_verify_peers2 returned %d with status %u"), verifyResult, status);
 		m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
@@ -1109,29 +1322,39 @@ int CTlsSocket::VerifyCertificate()
 		return FZ_REPLY_ERROR;
 	}
 
-	if (status & GNUTLS_CERT_REVOKED) {
-		m_pOwner->LogMessage(MessageType::Error, _("Beware! Certificate has been revoked"));
+	if (status != 0) {
+		if (status & GNUTLS_CERT_REVOKED) {
+			m_pOwner->LogMessage(MessageType::Error, _("Beware! Certificate has been revoked"));
+		}
+		else if (status & GNUTLS_CERT_SIGNATURE_FAILURE) {
+			m_pOwner->LogMessage(MessageType::Error, _("Certificate signature verification failed"));
+		}
+		else if (status & GNUTLS_CERT_INSECURE_ALGORITHM) {
+			m_pOwner->LogMessage(MessageType::Error, _("A certificate in the chain was signed using an insecure algorithm"));
+		}
+		else if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
+			m_pOwner->LogMessage(MessageType::Error, _("An issuer in the certificate chain is not a certificate authority"));
+		}
+		else if (status & GNUTLS_CERT_UNEXPECTED_OWNER) {
+			m_pOwner->LogMessage(MessageType::Error, _("The server's hostname does not match the certificate's hostname"));
+		}
+		else {
+			m_pOwner->LogMessage(MessageType::Error, _("Received certificate chain could not be verified. Verification status is %d."), status);
+		}
+
 		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
-		m_pOwner->LogMessage(MessageType::Error, _("Incomplete chain, top certificate is not self-signed certificate authority certificate"));
-		Failure(0, true);
-		return FZ_REPLY_ERROR;
-	}
-
-	unsigned int cert_list_size;
-	const gnutls_datum_t* cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
-	if (!cert_list || !cert_list_size) {
-		m_pOwner->LogMessage(MessageType::Error, _("gnutls_certificate_get_peers returned no certificates"));
+	datum_holder cert_der{};
+	if (gnutls_x509_crt_export2(certs.certs[0], GNUTLS_X509_FMT_DER, &cert_der) != GNUTLS_E_SUCCESS) {
 		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
 	if (m_implicitTrustedCert.data) {
-		if (m_implicitTrustedCert.size != cert_list[0].size ||
-			memcmp(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size))
+		if (m_implicitTrustedCert.size != cert_der.size ||
+			memcmp(m_implicitTrustedCert.data, cert_der.data, cert_der.size))
 		{
 			m_pOwner->LogMessage(MessageType::Error, _("Primary connection and data connection certificates don't match."));
 			Failure(0, true);
@@ -1148,16 +1371,16 @@ int CTlsSocket::VerifyCertificate()
 	m_pOwner->LogMessage(MessageType::Status, _("Verifying certificate..."));
 
 	std::vector<CCertificate> certificates;
-	for (unsigned int i = 0; i < cert_list_size; ++i) {
+	certificates.reserve(certs.certs_size);
+	for (unsigned int i = 0; i < certs.certs_size; ++i) {
 		CCertificate cert;
-		if (ExtractCert(cert_list, cert))
+		if (ExtractCert(certs.certs[i], cert)) {
 			certificates.push_back(cert);
+		}
 		else {
 			Failure(0, true);
 			return FZ_REPLY_ERROR;
 		}
-
-		++cert_list;
 	}
 
 	if (CertificateIsBlacklisted(certificates)) {
@@ -1175,8 +1398,9 @@ int CTlsSocket::VerifyCertificate()
 		GetCipherName(),
 		GetMacName(),
 		algorithmWarnings,
-		certificates);
+		std::move(certificates));
 
+	// Finally, ask user to verify the certificate chain
 	m_pOwner->SendAsyncRequest(pNotification);
 
 	return FZ_REPLY_WOULDBLOCK;
@@ -1186,81 +1410,107 @@ void CTlsSocket::OnRateAvailable(CRateLimiter::rate_direction)
 {
 }
 
-wxString CTlsSocket::GetProtocolName()
+std::wstring CTlsSocket::GetProtocolName()
 {
-	wxString protocol = _("unknown");
+	std::wstring ret;
 
 	const char* s = gnutls_protocol_get_name( gnutls_protocol_get_version( m_session ) );
-	if (s && *s)
-		protocol = wxString(s, wxConvUTF8);
+	if (s && *s) {
+		ret = fz::to_wstring_from_utf8(s);
+	}
 
-	return protocol;
+	if (ret.empty()) {
+		ret = _("unknown");
+	}
+
+	return ret;
 }
 
-wxString CTlsSocket::GetKeyExchange()
+std::wstring CTlsSocket::GetKeyExchange()
 {
-	wxString keyExchange = _("unknown");
+	std::wstring ret;
 
 	const char* s = gnutls_kx_get_name( gnutls_kx_get( m_session ) );
-	if (s && *s)
-		keyExchange = wxString(s, wxConvUTF8);
+	if (s && *s) {
+		ret = fz::to_wstring_from_utf8(s);
+	}
 
-	return keyExchange;
+	if (ret.empty()) {
+		ret = _("unknown");
+	}
+
+	return ret;
 }
 
-wxString CTlsSocket::GetCipherName()
+std::wstring CTlsSocket::GetCipherName()
 {
+	std::wstring ret;
+
 	const char* cipher = gnutls_cipher_get_name(gnutls_cipher_get(m_session));
-	if (cipher && *cipher)
-		return wxString(cipher, wxConvUTF8);
-	else
-		return _("unknown");
+	if (cipher && *cipher) {
+		ret = fz::to_wstring_from_utf8(cipher);
+	}
+
+	if (ret.empty()) {
+		ret = _("unknown");
+	}
+
+	return ret;
 }
 
-wxString CTlsSocket::GetMacName()
+std::wstring CTlsSocket::GetMacName()
 {
+	std::wstring ret;
+
 	const char* mac = gnutls_mac_get_name(gnutls_mac_get(m_session));
-	if (mac && *mac)
-		return wxString(mac, wxConvUTF8);
-	else
-		return _T("unknown");
+	if (mac && *mac) {
+		ret = fz::to_wstring_from_utf8(mac);
+	}
+
+	if (ret.empty()) {
+		ret = _("unknown");
+	}
+
+	return ret;
 }
 
-wxString CTlsSocket::ListTlsCiphers(wxString priority)
+std::string CTlsSocket::ListTlsCiphers(std::string priority)
 {
-	if (priority.empty())
-		priority = wxString::FromUTF8(ciphers);
+	if (priority.empty()) {
+		priority = ciphers;
+	}
 
-	wxString list = wxString::Format(_T("Ciphers for %s:\n"), priority);
+	auto list = fz::sprintf("Ciphers for %s:\n", priority);
 
 	gnutls_priority_t pcache;
 	const char *err = 0;
-	int ret = gnutls_priority_init(&pcache, priority.mb_str(), &err);
+	int ret = gnutls_priority_init(&pcache, priority.c_str(), &err);
 	if (ret < 0) {
-		list += wxString::Format(_T("gnutls_priority_init failed with code %d: %s"), ret, err ? wxString::FromUTF8(err) : _T("Unknown error"));
+		list += fz::sprintf("gnutls_priority_init failed with code %d: %s", ret, err ? err : "Unknown error");
 		return list;
 	}
 	else {
-		for (size_t i = 0; ; i++) {
+		for (size_t i = 0; ; ++i) {
 			unsigned int idx;
 			ret = gnutls_priority_get_cipher_suite_index(pcache, i, &idx);
-			if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
+			if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
 				break;
-			if (ret == GNUTLS_E_UNKNOWN_CIPHER_SUITE)
+			}
+			if (ret == GNUTLS_E_UNKNOWN_CIPHER_SUITE) {
 				continue;
+			}
 
 			gnutls_protocol_t version;
 			unsigned char id[2];
 			const char* name = gnutls_cipher_suite_info(idx, id, NULL, NULL, NULL, &version);
 
-			if (name != 0)
-			{
-				list += wxString::Format(
-					_T("%-50s    0x%02x, 0x%02x    %s\n"),
-					wxString::FromUTF8(name),
+			if (name != 0) {
+				list += fz::sprintf(
+					"%-50s    0x%02x, 0x%02x    %s\n",
+					name,
 					(unsigned char)id[0],
 					(unsigned char)id[1],
-					wxString::FromUTF8(gnutls_protocol_get_name(version)));
+					gnutls_protocol_get_name(version));
 			}
 		}
 	}
