@@ -1,6 +1,9 @@
 #ifndef __TLSSOCKET_H__
 #define __TLSSOCKET_H__
 
+// TODO: Figure our ssize_t mess
+#include <wx/defs.h>
+
 #include <gnutls/gnutls.h>
 #include "backend.h"
 #include "socket.h"
@@ -37,15 +40,15 @@ public:
 
 	TlsState GetState() const { return m_tlsState; }
 
-	wxString GetProtocolName();
-	wxString GetKeyExchange();
-	wxString GetCipherName();
-	wxString GetMacName();
+	std::wstring GetProtocolName();
+	std::wstring GetKeyExchange();
+	std::wstring GetCipherName();
+	std::wstring GetMacName();
 	int GetAlgorithmWarnings();
 
 	bool ResumedSession() const;
 
-	static wxString ListTlsCiphers(wxString priority);
+	static std::string ListTlsCiphers(std::string priority);
 
 	bool SetClientCertificate(fz::native_string const& keyfile, fz::native_string const& certs, fz::native_string const& password);
 
@@ -72,11 +75,11 @@ protected:
 
 	gnutls_certificate_credentials_t m_certCredentials{};
 
-	void LogError(int code, const wxString& function, MessageType logLegel = MessageType::Error);
+	void LogError(int code, std::wstring const& function, MessageType logLegel = MessageType::Error);
 	void PrintAlert(MessageType logLevel);
 
 	// Failure logs the error, uninits the session and sends a close event
-	void Failure(int code, bool send_close, const wxString& function = wxString());
+	void Failure(int code, bool send_close, std::wstring const& function = std::wstring());
 
 	static ssize_t PushFunction(gnutls_transport_ptr_t ptr, const void* data, size_t len);
 	static ssize_t PullFunction(gnutls_transport_ptr_t ptr, void* data, size_t len);
@@ -96,7 +99,7 @@ protected:
 	bool GetSortedPeerCertificates(gnutls_x509_crt_t *& certs, unsigned int & certs_size);
 
 	bool ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& out);
-	std::vector<wxString> GetCertSubjectAltNames(gnutls_x509_crt_t cert);
+	std::vector<std::wstring> GetCertSubjectAltNames(gnutls_x509_crt_t cert);
 
 	bool m_canReadFromSocket{true};
 	bool m_canWriteToSocket{true};
