@@ -1,18 +1,19 @@
 #ifndef __DIRECTORYLISTING_H__
 #define __DIRECTORYLISTING_H__
 
-#include "optional.h"
+#include <libfilezilla/optional.hpp>
+#include <libfilezilla/shared.hpp>
 #include <libfilezilla/time.hpp>
 
-#include <map>
+#include <unordered_map>
 
 class CDirentry
 {
 public:
 	std::wstring name;
 	int64_t size;
-	CRefcountObject<std::wstring> permissions;
-	CRefcountObject<std::wstring> ownerGroup;
+	fz::shared_value<std::wstring> permissions;
+	fz::shared_value<std::wstring> ownerGroup;
 
 	enum _flags
 	{
@@ -39,29 +40,27 @@ public:
 
 	inline bool has_date() const
 	{
-		return time.empty();;
+		return !time.empty();
 	}
 
 	inline bool has_time() const
 	{
-		return time.empty() && time.get_accuracy() >= fz::datetime::hours;
+		return !time.empty() && time.get_accuracy() >= fz::datetime::hours;
 	}
 
 	inline bool has_seconds() const
 	{
-		return time.empty() && time.get_accuracy() >= fz::datetime::seconds;
+		return !time.empty() && time.get_accuracy() >= fz::datetime::seconds;
 	}
 
-	CSparseOptional<std::wstring> target; // Set to linktarget it link is true
+	fz::sparse_optional<std::wstring> target; // Set to linktarget it link is true
 
 	fz::datetime time;
 
-	wxString dump() const;
+	std::wstring dump() const;
 	bool operator==(const CDirentry &op) const;
 };
 
-#include "refcount.h"
-
 class CDirectoryListing final
 {
 public:
@@ -74,17 +73,18 @@ public:
 	CDirectoryListing& operator=(CDirectoryListing const&) = default;
 	CDirectoryListing& operator=(CDirectoryListing &&) noexcept = default;
 
-	const CDirentry& operator[](unsigned int index) const;
+	CDirentry const& operator[](unsigned int index) const;
 
 	// Word of caution: You MUST NOT change the name of the returned
 	// entry if you do not call ClearFindMap afterwards
-	CDirentry& operator[](unsigned int index);
+	CDirentry& get(unsigned int index);
 
-	void SetCount(unsigned int count);
-	unsigned int GetCount() const { return m_entryCount; }
+	unsigned int GetCount() const { return m_entries ? m_entries->size() : 0; }
 
-	int FindFile_CmpCase(const wxString& name) const;
-	int FindFile_CmpNoCase(wxString name) const;
+	void Append(CDirentry&& entry);
+
+	int FindFile_CmpCase(std::wstring const& name) const;
+	int FindFile_CmpNoCase(std::wstring const& name) const;
 
 	void ClearFindMap();
 
@@ -126,7 +126,7 @@ public:
 	bool has_perms() const { return (m_flags & listing_has_perms) != 0; }
 	bool has_usergroup() const { return (m_flags & listing_has_usergroup) != 0; }
 
-	void Assign(std::deque<CRefcountObject<CDirentry>> & entries);
+	void Assign(std::deque<fz::shared_value<CDirentry>> & entries);
 
 	bool RemoveEntry(unsigned int index);
 
@@ -134,12 +134,10 @@ public:
 
 protected:
 
-	CRefcountObject_Uninitialized<std::vector<CRefcountObject<CDirentry> > > m_entries;
+	fz::shared_optional<std::vector<fz::shared_value<CDirentry> > > m_entries;
 
-	mutable CRefcountObject_Uninitialized<std::multimap<std::wstring, unsigned int> > m_searchmap_case;
-	mutable CRefcountObject_Uninitialized<std::multimap<std::wstring, unsigned int> > m_searchmap_nocase;
-
-	unsigned int m_entryCount{};
+	mutable fz::shared_optional<std::unordered_multimap<std::wstring, unsigned int> > m_searchmap_case;
+	mutable fz::shared_optional<std::unordered_multimap<std::wstring, unsigned int> > m_searchmap_nocase;
 };
 
 #endif
