@@ -76,6 +76,10 @@ enum engineOptions
 	OPTION_SIZE_USETHOUSANDSEP,
 	OPTION_SIZE_DECIMALPLACES,
 
+	OPTION_TCP_KEEPALIVE_INTERVAL,
+
+	OPTION_CACHE_TTL,
+
 	OPTIONS_ENGINE_NUM
 };
 
@@ -84,10 +88,10 @@ class COptionsBase
 public:
 	inline virtual ~COptionsBase() {};
 	virtual int GetOptionVal(unsigned int nID) = 0;
-	virtual wxString GetOption(unsigned int nID) = 0;
+	virtual std::wstring GetOption(unsigned int nID) = 0;
 
 	virtual bool SetOption(unsigned int nID, int value) = 0;
-	virtual bool SetOption(unsigned int nID, wxString const& value) = 0;
+	virtual bool SetOption(unsigned int nID, std::wstring const& value) = 0;
 };
 
 #endif
