@@ -1,5 +1,5 @@
-#ifndef __SERVER_H__
-#define __SERVER_H__
+#ifndef FILEZILLA_ENGINE_SERVER_HEADER
+#define FILEZILLA_ENGINE_SERVER_HEADER
 
 enum ServerProtocol
 {
@@ -76,22 +76,20 @@ public:
 
 	// No error checking is done in the constructors
 	CServer();
-	CServer(wxString host, unsigned int);
-	CServer(wxString host, unsigned int, wxString user, wxString pass = wxString());
-	CServer(ServerProtocol protocol, ServerType type, wxString host, unsigned int);
-	CServer(ServerProtocol protocol, ServerType type, wxString host, unsigned int, wxString user, wxString pass = wxString(), wxString account = wxString());
+	CServer(ServerProtocol protocol, ServerType type, std::wstring const& host, unsigned int);
+	CServer(ServerProtocol protocol, ServerType type, std::wstring const& host, unsigned int, std::wstring const& user, std::wstring const& pass = std::wstring(), std::wstring const& account = std::wstring());
 
 	void SetType(ServerType type);
 
 	ServerProtocol GetProtocol() const;
 	ServerType GetType() const;
-	wxString GetHost() const;
+	std::wstring GetHost() const;
 	unsigned int GetPort() const;
 	LogonType GetLogonType() const;
-	wxString GetUser() const;
-	wxString GetPass() const;
-	wxString GetAccount() const;
-	wxString GetKeyFile() const;
+	std::wstring GetUser() const;
+	std::wstring GetPass() const;
+	std::wstring GetAccount() const;
+	std::wstring GetKeyFile() const;
 	int GetTimezoneOffset() const;
 	PasvMode GetPasvMode() const;
 	int MaximumMultipleConnections() const;
@@ -99,16 +97,16 @@ public:
 
 	// Return true if URL could be parsed correctly, false otherwise.
 	// If parsing fails, pError is filled with the reason and the CServer instance may be left an undefined state.
-	bool ParseUrl(wxString host, unsigned int port, wxString user, wxString pass, wxString &error, CServerPath &path);
-	bool ParseUrl(wxString host, wxString port, wxString user, wxString pass, wxString &error, CServerPath &path);
+	bool ParseUrl(std::wstring host, unsigned int port, std::wstring user, std::wstring pass, std::wstring &error, CServerPath &path);
+	bool ParseUrl(std::wstring const& host, std::wstring const& port, std::wstring const& user, std::wstring const& pass, std::wstring &error, CServerPath &path);
 
 	void SetProtocol(ServerProtocol serverProtocol);
-	bool SetHost(wxString Host, unsigned int port);
+	bool SetHost(std::wstring const& host, unsigned int port);
 
 	void SetLogonType(LogonType logonType);
-	bool SetUser(const wxString& user, const wxString& pass = wxString());
-	bool SetAccount(const wxString& account);
-	bool SetKeyFile(const wxString& keyFile);
+	bool SetUser(std::wstring const& user, std::wstring const& pass = std::wstring());
+	bool SetAccount(std::wstring const& account);
+	bool SetKeyFile(std::wstring const& keyFile);
 
 	CServer& operator=(const CServer &op);
 	bool operator==(const CServer &op) const;
@@ -120,29 +118,29 @@ public:
 	void SetPasvMode(PasvMode pasvMode);
 	void MaximumMultipleConnections(int maximum);
 
-	wxString Format(ServerFormat formatType) const;
+	std::wstring Format(ServerFormat formatType) const;
 
-	bool SetEncodingType(CharsetEncoding type, const wxString& encoding = wxString());
-	bool SetCustomEncoding(const wxString& encoding);
+	bool SetEncodingType(CharsetEncoding type, std::wstring const& encoding = std::wstring());
+	bool SetCustomEncoding(std::wstring const& encoding);
 	CharsetEncoding GetEncodingType() const;
-	wxString GetCustomEncoding() const;
+	std::wstring GetCustomEncoding() const;
 
 	static unsigned int GetDefaultPort(ServerProtocol protocol);
 	static ServerProtocol GetProtocolFromPort(unsigned int port, bool defaultOnly = false);
 
-	static wxString GetProtocolName(ServerProtocol protocol);
-	static ServerProtocol GetProtocolFromName(const wxString& name);
+	static std::wstring GetProtocolName(ServerProtocol protocol);
+	static ServerProtocol GetProtocolFromName(std::wstring const& name);
 
-	static ServerProtocol GetProtocolFromPrefix(const wxString& prefix);
-	static wxString GetPrefixFromProtocol(const ServerProtocol protocol);
+	static ServerProtocol GetProtocolFromPrefix(std::wstring const& prefix);
+	static std::wstring GetPrefixFromProtocol(ServerProtocol const protocol);
 
 	// Some protocol distinguish between ASCII and binary files for line-ending
 	// conversion.
 	static bool ProtocolHasDataTypeConcept(ServerProtocol const protocol);
 
 	// These commands will be executed after a successful login.
-	const std::vector<wxString>& GetPostLoginCommands() const { return m_postLoginCommands; }
-	bool SetPostLoginCommands(const std::vector<wxString>& postLoginCommands);
+	std::vector<std::wstring> const& GetPostLoginCommands() const { return m_postLoginCommands; }
+	bool SetPostLoginCommands(std::vector<std::wstring> const& postLoginCommands);
 	static bool SupportsPostLoginCommands(ServerProtocol const protocol);
 
 	void SetBypassProxy(bool val);
@@ -152,11 +150,11 @@ public:
 	void SetName(std::wstring const& name) { m_name = name; }
 	std::wstring GetName() const { return m_name; }
 
-	static wxString GetNameFromServerType(ServerType type);
-	static ServerType GetServerTypeFromName(const wxString& name);
+	static std::wstring GetNameFromServerType(ServerType type);
+	static ServerType GetServerTypeFromName(std::wstring const& name);
 
-	static wxString GetNameFromLogonType(LogonType type);
-	static LogonType GetLogonTypeFromName(const wxString& name);
+	static std::wstring GetNameFromLogonType(LogonType type);
+	static LogonType GetLogonTypeFromName(std::wstring const& name);
 
 	explicit operator bool() const { return !m_host.empty(); }
 
@@ -165,21 +163,21 @@ protected:
 
 	ServerProtocol m_protocol;
 	ServerType m_type;
-	wxString m_host;
+	std::wstring m_host;
 	unsigned int m_port;
 	LogonType m_logonType;
-	wxString m_user;
-	wxString m_pass;
-	wxString m_account;
-	wxString m_keyFile;
+	std::wstring m_user;
+	std::wstring m_pass;
+	std::wstring m_account;
+	std::wstring m_keyFile;
 	int m_timezoneOffset;
 	PasvMode m_pasvMode;
 	int m_maximumMultipleConnections;
 	CharsetEncoding m_encodingType;
-	wxString m_customEncoding;
+	std::wstring m_customEncoding;
 	std::wstring m_name;
 
-	std::vector<wxString> m_postLoginCommands;
+	std::vector<std::wstring> m_postLoginCommands;
 	bool m_bypassProxy;
 };
 
