@@ -16,17 +16,19 @@
 #include <wx/msw/registry.h>
 #include "volume_enumerator.h"
 #endif
-#include "edithandler.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
+#include "edithandler.h"
 #include "filelist_statusbar.h"
+#include "graphics.h"
+#include "local_recursive_operation.h"
 #include "sizeformatting.h"
 #include "timeformatting.h"
 
 #include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/recursive_remove.hpp>
 
-class CLocalListViewDropTarget : public CScrollableDropTarget<wxListCtrlEx>
+class CLocalListViewDropTarget final : public CScrollableDropTarget<wxListCtrlEx>
 {
 public:
 	CLocalListViewDropTarget(CLocalListView* pLocalListView)
@@ -61,59 +63,65 @@ public:
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
+		{
 			return def;
+		}
 
-		if (m_pLocalListView->m_fileData.empty())
+		if (m_pLocalListView->m_fileData.empty()) {
 			return wxDragError;
+		}
 
-		if (def != wxDragCopy && def != wxDragMove)
+		if (def != wxDragCopy && def != wxDragMove) {
 			return wxDragError;
+		}
 
 		CDragDropManager* pDragDropManager = CDragDropManager::Get();
-		if (pDragDropManager)
+		if (pDragDropManager) {
 			pDragDropManager->pDropTarget = m_pLocalListView;
+		}
 
-		wxString subdir;
+		std::wstring subdir;
 		int flags;
 		int hit = m_pLocalListView->HitTest(wxPoint(x, y), flags, 0);
-		if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM))
-		{
+		if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM)) {
 			const CLocalFileData* const data = m_pLocalListView->GetData(hit);
-			if (data && data->dir)
+			if (data && data->dir) {
 				subdir = data->name;
+			}
 		}
 
-		CLocalPath dir = m_pLocalListView->m_pState->GetLocalDir();
-		if (!subdir.empty())
-		{
-			if (!dir.ChangePath(subdir))
+		CLocalPath dir = m_pLocalListView->m_state.GetLocalDir();
+		if (!subdir.empty()) {
+			if (!dir.ChangePath(subdir)) {
 				return wxDragError;
+			}
 		}
 
-		if (!dir.IsWriteable())
+		if (!dir.IsWriteable()) {
 			return wxDragError;
+		}
 
-		if (!GetData())
+		if (!GetData()) {
 			return wxDragError;
+		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
-			m_pLocalListView->m_pState->HandleDroppedFiles(m_pFileDataObject, dir, def == wxDragCopy);
-		else
-		{
-			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
-			{
+		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
+			m_pLocalListView->m_state.HandleDroppedFiles(m_pFileDataObject, dir, def == wxDragCopy);
+		}
+		else {
+			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalListView->m_pState->GetServer()))
-			{
+			if (!m_pLocalListView->m_state.GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalListView->m_state.GetServer())) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_pState->DownloadDroppedFiles(m_pRemoteDataObject, dir))
+			if (!m_pLocalListView->m_state.DownloadDroppedFiles(m_pRemoteDataObject, dir)) {
 				return wxDragNone;
+			}
 		}
 
 		return def;
@@ -193,28 +201,29 @@ public:
 			return def;
 		}
 
-		if (m_pLocalListView->m_fileData.empty())
-		{
+		if (m_pLocalListView->m_fileData.empty()) {
 			ClearDropHighlight();
 			return wxDragNone;
 		}
 
-		const wxString& subdir = DoDisplayDropHighlight(wxPoint(x, y));
+		std::wstring const subdir = DoDisplayDropHighlight(wxPoint(x, y)).ToStdWstring();
 
-		CLocalPath dir = m_pLocalListView->m_pState->GetLocalDir();
+		CLocalPath dir = m_pLocalListView->m_state.GetLocalDir();
 		if (subdir.empty()) {
 			const CDragDropManager* pDragDropManager = CDragDropManager::Get();
-			if (pDragDropManager && pDragDropManager->localParent == m_pLocalListView->m_dir)
+			if (pDragDropManager && pDragDropManager->localParent == m_pLocalListView->m_dir) {
 				return wxDragNone;
+			}
 		}
-		else
-		{
-			if (!dir.ChangePath(subdir))
+		else {
+			if (!dir.ChangePath(subdir)) {
 				return wxDragNone;
+			}
 		}
 
-		if (!dir.IsWriteable())
+		if (!dir.IsWriteable()) {
 			return wxDragNone;
+		}
 
 		return def;
 	}
@@ -260,14 +269,15 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CLocalListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQueue)
-	: CFileListCtrl<CLocalFileData>(pParent, pState, pQueue),
-	CStateEventHandler(pState)
+CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQueue)
+	: CFileListCtrl<CLocalFileData>(pParent, pQueue),
+	CStateEventHandler(state)
 {
-	wxGetApp().AddStartupProfileRecord(_T("CLocalListView::CLocalListView"));
-	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
-	m_pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
-	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
+	wxGetApp().AddStartupProfileRecord("CLocalListView::CLocalListView");
+	m_state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	m_state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	m_state.RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
+	m_state.RegisterHandler(this, STATECHANGE_SERVER);
 
 	m_dropTarget = -1;
 
@@ -279,8 +289,6 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQ
 	AddColumn(_("Last modified"), wxLIST_FORMAT_LEFT, widths[3]);
 	LoadColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
 
-	InitSort(OPTION_LOCALFILELIST_SORTORDER);
-
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
 #ifdef __WXMSW__
@@ -289,15 +297,19 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQ
 
 	InitHeaderSortImageList();
 
+	InitSort(OPTION_LOCALFILELIST_SORTORDER);
+
 	SetDropTarget(new CLocalListViewDropTarget(this));
 
 	EnablePrefixSearch(true);
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*GetMainWindow());
 }
 
 CLocalListView::~CLocalListView()
 {
 	wxString str = wxString::Format(_T("%d %d"), m_sortDirection, m_sortColumn);
-	COptions::Get()->SetOption(OPTION_LOCALFILELIST_SORTORDER, str);
+	COptions::Get()->SetOption(OPTION_LOCALFILELIST_SORTORDER, str.ToStdWstring());
 
 #ifdef __WXMSW__
 	delete m_pVolumeEnumeratorThread;
@@ -306,37 +318,39 @@ CLocalListView::~CLocalListView()
 
 bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 {
-	
+
 	CancelLabelEdit();
 
 	wxString focused;
 	std::list<wxString> selectedNames;
 	bool ensureVisible = false;
-	if (m_dir != dirname)
-	{
+	if (m_dir != dirname) {
 		ResetSearchPrefix();
 
-		if (IsComparing())
+		if (IsComparing()) {
 			ExitComparisonMode();
+		}
 
 		ClearSelection();
-		focused = m_pState->GetPreviouslyVisitedLocalSubdir();
+		focused = m_state.GetPreviouslyVisitedLocalSubdir();
 		ensureVisible = !focused.empty();
-		if (focused.empty())
+		if (focused.empty()) {
 			focused = _T("..");
+		}
 
-		if (GetItemCount())
+		if (GetItemCount()) {
 			EnsureVisible(0);
+		}
 		m_dir = dirname;
 	}
-	else
-	{
+	else {
 		// Remember which items were selected
 		selectedNames = RememberSelectedItems(focused);
 	}
 
-	if (m_pFilelistStatusBar)
+	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->UnselectAll();
+	}
 
 	const int oldItemCount = m_indexMapping.size();
 
@@ -358,11 +372,11 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 	if (m_dir.GetPath() == _T("\\")) {
 		DisplayDrives();
 	}
-	else if (m_dir.GetPath().Left(2) == _T("\\\\"))
-	{
-		int pos = m_dir.GetPath().Mid(2).Find('\\');
-		if (pos != -1 && pos + 3 != (int)m_dir.GetPath().Len())
+	else if (m_dir.GetPath().substr(0, 2) == _T("\\\\")) {
+		auto pos = m_dir.GetPath().find('\\', 2);
+		if (pos != std::wstring::npos && pos + 1 < m_dir.GetPath().size()) {
 			goto regular_dir;
+		}
 
 		// UNC path without shares
 		DisplayShares(m_dir.GetPath());
@@ -397,7 +411,7 @@ regular_dir:
 				continue;
 			}
 
-			data.name = name;
+			data.name = fz::to_wstring(name);
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 				if (data.dir)
@@ -479,21 +493,20 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 	bool back = false;
 
 	int item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 
 		count++;
 
-		if (!item && m_hasParent)
+		if (!item && m_hasParent) {
 			back = true;
+		}
 	}
-	if (count > 1)
-	{
-		if (back)
-		{
+	if (count > 1) {
+		if (back) {
 			wxBell();
 			return;
 		}
@@ -506,30 +519,29 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 	item = event.GetIndex();
 
 	CLocalFileData *data = GetData(item);
-	if (!data)
+	if (!data) {
 		return;
+	}
 
-	if (data->dir)
-	{
+	if (data->dir) {
 		const int action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
-		if (action == 3)
-		{
+		if (action == 3) {
 			// No action
 			wxBell();
 			return;
 		}
 
-		if (!action || data->name == _T(".."))
-		{
+		if (!action || data->name == _T("..")) {
 			// Enter action
 
-			wxString error;
-			if (!m_pState->SetLocalDir(data->name, &error))
-			{
-				if (!error.empty())
+			std::wstring error;
+			if (!m_state.SetLocalDir(data->name, &error)) {
+				if (!error.empty()) {
 					wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-				else
+				}
+				else {
 					wxBell();
+				}
 			}
 			return;
 		}
@@ -539,38 +551,33 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	if (data->comparison_flags == fill)
-	{
+	if (data->comparison_flags == fill) {
 		wxBell();
 		return;
 	}
 
 	const int action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_FILE);
-	if (action == 3)
-	{
+	if (action == 3) {
 		// No action
 		wxBell();
 		return;
 	}
 
-	if (action == 2)
-	{
+	if (action == 2) {
 		// View / Edit action
 		wxCommandEvent evt;
 		OnMenuEdit(evt);
 		return;
 	}
 
-	const CServer* pServer = m_pState->GetServer();
-	if (!pServer)
-	{
+	const CServer* pServer = m_state.GetServer();
+	if (!pServer) {
 		wxBell();
 		return;
 	}
 
-	CServerPath path = m_pState->GetRemotePath();
-	if (path.empty())
-	{
+	CServerPath path = m_state.GetRemotePath();
+	if (path.empty()) {
 		wxBell();
 		return;
 	}
@@ -600,12 +607,14 @@ void CLocalListView::OnMenuEnter(wxCommandEvent &)
 		return;
 	}
 
-	wxString error;
-	if (!m_pState->SetLocalDir(data->name, &error)) {
-		if (!error.empty())
+	std::wstring error;
+	if (!m_state.SetLocalDir(data->name, &error)) {
+		if (!error.empty()) {
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-		else
+		}
+		else {
 			wxBell();
+		}
 	}
 }
 
@@ -614,17 +623,19 @@ void CLocalListView::DisplayDrives()
 {
 	int count = m_fileData.size();
 
-	std::list<wxString> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
-	for( std::list<wxString>::const_iterator it = drives.begin(); it != drives.end(); ++it ) {
+	std::list<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
+	for (auto it = drives.begin(); it != drives.end(); ++it) {
 		wxString drive = *it;
-		if (drive.Right(1) == _T("\\"))
+		if (drive.Right(1) == _T("\\")) {
 			drive.RemoveLast();
+		}
 
 		CLocalFileData data;
 		data.name = drive;
-		data.label = CSparseOptional<wxString>(data.name);
+		data.label = fz::sparse_optional<wxString>(data.name);
 		data.dir = true;
 		data.size = -1;
+		data.icon = GetIconIndex(iconType::dir, _T(""), false);
 
 		m_fileData.push_back(data);
 		m_indexMapping.push_back(count);
@@ -678,7 +689,7 @@ void CLocalListView::DisplayShares(wxString computer)
 			CLocalFileData data;
 			data.name = p->shi1_netname;
 #ifdef __WXMSW__
-			data.label = CSparseOptional<wxString>(data.name);
+			data.label = fz::sparse_optional<wxString>(data.name);
 #endif
 			data.dir = true;
 			data.size = -1;
@@ -724,8 +735,7 @@ CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComp
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
 	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
 
-	if (!m_sortDirection)
-	{
+	if (!m_sortDirection) {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -735,8 +745,7 @@ CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComp
 		else
 			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
-	else
-	{
+	else {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -750,8 +759,7 @@ CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComp
 
 void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 {
-	if (GetEditControl())
-	{
+	if (GetEditControl()) {
 		event.Skip();
 		return;
 	}
@@ -760,9 +768,8 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	if (!pMenu)
 		return;
 
-	const bool connected = m_pState->IsRemoteConnected();
-	if (!connected)
-	{
+	const bool connected = m_state.IsRemoteConnected();
+	if (!connected) {
 		pMenu->Enable(XRCID("ID_EDIT"), COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL) == 0);
 		pMenu->Enable(XRCID("ID_UPLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
@@ -772,12 +779,10 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	int count = 0;
 	int fillCount = 0;
 	bool selectedDir = false;
-	while (index != -1)
-	{
-		count++;
+	while (index != -1) {
+		++count;
 		const CLocalFileData* const data = GetData(index);
-		if (!data || (!index && m_hasParent))
-		{
+		if (!data || (!index && m_hasParent)) {
 			pMenu->Enable(XRCID("ID_OPEN"), false);
 			pMenu->Enable(XRCID("ID_RENAME"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
@@ -788,8 +793,7 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 			selectedDir = true;
 		index = GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-	if (!count || fillCount == count)
-	{
+	if (!count || fillCount == count) {
 		pMenu->Delete(XRCID("ID_ENTER"));
 		pMenu->Enable(XRCID("ID_UPLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
@@ -797,19 +801,22 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 	}
-	else if (count > 1)
-	{
+	else if (count > 1) {
 		pMenu->Delete(XRCID("ID_ENTER"));
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 	}
-	else
-	{
+	else {
 		// Exactly one item selected
 		if (!selectedDir)
 			pMenu->Delete(XRCID("ID_ENTER"));
 	}
-	if (selectedDir)
+	if (selectedDir) {
 		pMenu->Enable(XRCID("ID_EDIT"), false);
+		if (m_state.GetLocalRecursiveOperation() && m_state.GetLocalRecursiveOperation()->IsActive()) {
+			pMenu->Enable(XRCID("ID_UPLOAD"), false);
+			pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
+		}
+	}
 
 	PopupMenu(pMenu);
 	delete pMenu;
@@ -817,7 +824,7 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 {
-	const CServer* pServer = m_pState->GetServer();
+	const CServer* pServer = m_state.GetServer();
 	if (!pServer) {
 		wxBell();
 		return;
@@ -827,6 +834,14 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 
 	bool queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
 
+	auto recursiveOperation = m_state.GetLocalRecursiveOperation();
+	if (!recursiveOperation || recursiveOperation->IsActive()) {
+		wxBell();
+		return;
+	}
+
+	local_recursion_root root;
+
 	long item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
@@ -842,28 +857,37 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 		if (data->comparison_flags == fill)
 			continue;
 
-		CServerPath path = m_pState->GetRemotePath();
-		if (path.empty()) {
+		CServerPath remotePath = m_state.GetRemotePath();
+		if (remotePath.empty()) {
 			wxBell();
 			break;
 		}
 
 		if (data->dir) {
-			if (!path.ChangePath(data->name)) {
+			CLocalPath localPath = m_dir;
+			if (!localPath.ChangePath(data->name)) {
+				continue;
+			}
+			if (!remotePath.ChangePath(data->name)) {
 				continue;
 			}
 
-			CLocalPath localPath(m_dir);
-			localPath.AddSegment(data->name);
-			m_pQueue->QueueFolder(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, localPath, path, *pServer);
+			root.add_dir_to_visit(localPath, remotePath);
 		}
 		else {
-			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, *pServer, data->size);
+			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, *pServer, data->size);
 			added = true;
 		}
 	}
+
 	if (added)
 		m_pQueue->QueueFile_Finish(!queue_only);
+
+	if (!root.empty()) {
+		recursiveOperation->AddRecursionRoot(std::move(root));
+		CFilterManager filter;
+		recursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
+	}
 }
 
 // Create a new Directory
@@ -871,26 +895,27 @@ void CLocalListView::OnMenuMkdir(wxCommandEvent&)
 {
 	wxString newdir = MenuMkdir();
 	if (!newdir.empty()) {
-		m_pState->RefreshLocal();
+		m_state.RefreshLocal();
 	}
 }
 
 // Create a new Directory and enter the new Directory
 void CLocalListView::OnMenuMkdirChgDir(wxCommandEvent&)
 {
-	wxString newdir = MenuMkdir();
+	std::wstring newdir = MenuMkdir().ToStdWstring();
 	if (newdir.empty()) {
 		return;
 	}
 
 	// OnMenuEnter
-	wxString error;
-	if (!m_pState->SetLocalDir(newdir, &error))
-	{
-		if (!error.empty())
+	std::wstring error;
+	if (!m_state.SetLocalDir(newdir, &error)) {
+		if (!error.empty()) {
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-		else
+		}
+		else {
 			wxBell();
+		}
 	}
 }
 
@@ -899,14 +924,15 @@ void CLocalListView::OnMenuMkdirChgDir(wxCommandEvent&)
 wxString CLocalListView::MenuMkdir()
 {
 	CInputDialog dlg;
-	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:")))
+	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:"))) {
 		return wxString();
+	}
 
-	if (dlg.ShowModal() != wxID_OK)
+	if (dlg.ShowModal() != wxID_OK) {
 		return wxString();
+	}
 
-	if (dlg.GetValue().empty())
-	{
+	if (dlg.GetValue().empty()) {
 		wxBell();
 		return wxString();
 	}
@@ -950,7 +976,7 @@ void CLocalListView::OnMenuDelete(wxCommandEvent&)
 	gui_recursive_remove rmd(this);
 	rmd.remove(pathsToDelete);
 
-	m_pState->SetLocalDir(m_dir);
+	m_state.SetLocalDir(m_dir);
 }
 
 void CLocalListView::OnMenuRename(wxCommandEvent&)
@@ -1021,7 +1047,7 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 
 bool CLocalListView::OnBeginRename(const wxListEvent& event)
 {
-	if (!m_pState->GetLocalDir().IsWriteable())
+	if (!m_state.GetLocalDir().IsWriteable())
 		return false;
 
 	if (event.GetIndex() == 0 && m_hasParent)
@@ -1043,7 +1069,7 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 	if (event.GetLabel().empty())
 		return false;
 
-	if (!m_pState->GetLocalDir().IsWriteable())
+	if (!m_state.GetLocalDir().IsWriteable())
 		return false;
 
 	CLocalFileData *const data = GetData(event.GetIndex());
@@ -1065,7 +1091,7 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 #ifdef __WXMSW__
 	data->label.clear();
 #endif
-	m_pState->RefreshLocal();
+	CallAfter([&](){m_state.RefreshLocal();});
 
 	return true;
 }
@@ -1074,18 +1100,21 @@ void CLocalListView::ApplyCurrentFilter()
 {
 	CFilterManager filter;
 
-	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing())
+	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing()) {
 		ExitComparisonMode();
+	}
 
 	unsigned int min = m_hasParent ? 1 : 0;
-	if (m_fileData.size() <= min)
+	if (m_fileData.size() <= min) {
 		return;
+	}
 
 	wxString focused;
 	const std::list<wxString>& selectedNames = RememberSelectedItems(focused);
 
-	if (m_pFilelistStatusBar)
+	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->UnselectAll();
+	}
 
 	int64_t totalSize{};
 	int unknown_sizes = 0;
@@ -1235,23 +1264,26 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 		if (i == (int)m_indexMapping.size())
 			break;
 	}
-	if (!focused.empty())
-	{
-		if (firstSelected != -1)
+	if (!focused.empty()) {
+		if (firstSelected != -1) {
 			SetItemState(firstSelected, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
-		else
+		}
+		else {
 			SetItemState(0, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+		}
 	}
 }
 
-void CLocalListView::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString& data, const void*)
+void CLocalListView::OnStateChange(t_statechange_notifications notification, const wxString& data, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
-		DisplayDir(m_pState->GetLocalDir());
+		DisplayDir(m_state.GetLocalDir());
 	else if (notification == STATECHANGE_APPLYFILTER)
 		ApplyCurrentFilter();
-	else
-	{
+	else if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
+	else {
 		wxASSERT(notification == STATECHANGE_LOCAL_REFRESH_FILE);
 		RefreshFile(data);
 	}
@@ -1259,6 +1291,10 @@ void CLocalListView::OnStateChange(CState*, enum t_statechange_notifications not
 
 void CLocalListView::OnBeginDrag(wxListEvent&)
 {
+	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+		return;
+	}
+
 	long item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
@@ -1311,7 +1347,7 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 	if (!handled_internally && (res == wxDragCopy || res == wxDragMove)) {
 		// We only need to refresh local side if the operation got handled
 		// externally, the internal handlers do this for us already
-		m_pState->RefreshLocal();
+		m_state.RefreshLocal();
 	}
 }
 
@@ -1320,7 +1356,7 @@ void CLocalListView::RefreshFile(const wxString& file)
 	CLocalFileData data;
 
 	bool wasLink;
-	enum fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(m_dir.GetPath() + file), wasLink, &data.size, &data.time, &data.attributes);
+	fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(m_dir.GetPath() + file), wasLink, &data.size, &data.time, &data.attributes);
 	if (type == fz::local_filesys::unknown)
 		return;
 
@@ -1474,8 +1510,7 @@ wxListItemAttr* CLocalListView::OnGetItemAttr(long item) const
 
 void CLocalListView::StartComparison()
 {
-	if (m_sortDirection || m_sortColumn)
-	{
+	if (m_sortDirection || m_sortColumn != 0) {
 		wxASSERT(m_originalIndexMapping.empty());
 		SortList(0, 0);
 	}
@@ -1567,7 +1602,7 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 			return wxString();
 
 		if (data->fileType.empty())
-			data->fileType = GetType(data->name, data->dir, m_dir.GetPath());
+			data->fileType = GetType(data->name, data->dir, m_dir.GetPath()).ToStdWstring();
 
 		return data->fileType;
 	}
@@ -1582,16 +1617,16 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 	CServer server;
 	CServerPath path;
 
-	if (!m_pState->GetServer()) {
+	if (!m_state.GetServer()) {
 		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
 			wxMessageBoxEx(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
 	else {
-		server = *m_pState->GetServer();
+		server = *m_state.GetServer();
 
-		path = m_pState->GetRemotePath();
+		path = m_state.GetRemotePath();
 		if (path.empty()) {
 			wxMessageBoxEx(_("Cannot edit file, remote path unknown."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
@@ -1692,9 +1727,10 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 		bool program_exists = false;
 		wxString cmd = GetSystemOpenCommand(fn.GetFullPath(), program_exists);
 		if (cmd.empty()) {
-			int pos = data.name.Find('.') == -1;
-			if (pos == -1 || (pos == 0 && data.name.Mid(1).Find('.') == -1))
+			auto pos = data.name.find('.');
+			if (pos == std::wstring::npos || (pos == 0 && data.name.find('.', 1) == std::wstring::npos)) {
 				cmd = pEditHandler->GetOpenCommand(fn.GetFullPath(), program_exists);
+			}
 		}
 		if (cmd.empty()) {
 			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
@@ -1727,8 +1763,9 @@ int64_t CLocalListView::ItemGetSize(int index) const
 
 void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 {
-	if (!m_pVolumeEnumeratorThread)
+	if (!m_pVolumeEnumeratorThread) {
 		return;
+	}
 
 	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
 	volumeInfo = m_pVolumeEnumeratorThread->GetVolumes();
@@ -1738,22 +1775,30 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	if (m_dir.GetPath() != _T("\\"))
+	if (m_dir.GetPath() != _T("\\")) {
 		return;
+	}
 
 	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter) {
 		wxString drive = iter->volume;
 
 		unsigned int item, index;
-		for (item = 1; item < m_indexMapping.size(); ++item) {
+		for (item = m_hasParent ? 1 : 0; item < m_indexMapping.size(); ++item) {
 			index = m_indexMapping[item];
-			if (m_fileData[index].name == drive || m_fileData[index].name.Left(drive.Len() + 1) == drive + _T(" "))
+			if (m_fileData[index].name == drive || m_fileData[index].name.substr(0, drive.Len() + 1) == drive + _T(" ")) {
 				break;
+			}
 		}
-		if (item >= m_indexMapping.size())
+		if (item >= m_indexMapping.size()) {
 			continue;
+		}
 
-		m_fileData[index].label = CSparseOptional<wxString>(drive + _T(" (") + iter->volumeName + _T(")"));
+		if (!iter->volumeName.empty()) {
+			m_fileData[index].label = fz::sparse_optional<wxString>(drive + _T(" (") + iter->volumeName + _T(")"));
+		}
+		if (iter->icon != -1) {
+			m_fileData[index].icon = iter->icon;
+		}
 
 		RefreshItem(item);
 	}
@@ -1763,26 +1808,25 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 
 void CLocalListView::OnMenuRefresh(wxCommandEvent&)
 {
-	m_pState->RefreshLocal();
+	m_state.RefreshLocal();
 }
 
 void CLocalListView::OnNavigationEvent(bool forward)
 {
-	if (!forward)
-	{
-		if (!m_hasParent)
-		{
+	if (!forward) {
+		if (!m_hasParent) {
 			wxBell();
 			return;
 		}
 
-		wxString error;
-		if (!m_pState->SetLocalDir(_T(".."), &error))
-		{
-			if (!error.empty())
+		std::wstring error;
+		if (!m_state.SetLocalDir(_T(".."), &error)) {
+			if (!error.empty()) {
 				wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-			else
+			}
+			else {
 				wxBell();
+			}
 		}
 	}
 }
