@@ -1,25 +1,26 @@
 #ifndef __LOCALTREEVIEW_H__
 #define __LOCALTREEVIEW_H__
 
+#include <option_change_event_handler.h>
 #include "systemimagelist.h"
 #include "state.h"
 #include "treectrlex.h"
-#include "filelistctrl.h"
 
 class CQueueView;
+class CWindowTinter;
 
 #ifdef __WXMSW__
 class CVolumeDescriptionEnumeratorThread;
 #endif
 
-class CLocalTreeView : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler
+class CLocalTreeView final : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler, COptionChangeEventHandler
 {
 	DECLARE_CLASS(CLocalTreeView)
 
 	friend class CLocalTreeViewDropTarget;
 
 public:
-	CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState, CQueueView *pQueueView);
+	CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView *pQueueView);
 	virtual ~CLocalTreeView();
 
 #ifdef __WXMSW__
@@ -28,7 +29,7 @@ public:
 #endif
 
 protected:
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
 
 	void SetDir(wxString localDir);
 	void RefreshListing();
@@ -44,11 +45,12 @@ protected:
 #endif
 
 	void UpdateSortMode();
-	virtual int OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId& item2);
+
+	virtual void OnOptionsChanged(changed_options_t const& options);
 
 	wxTreeItemId GetNearestParent(wxString& localDir);
 	wxTreeItemId GetSubdir(wxTreeItemId parent, const wxString& subDir);
-	void DisplayDir(wxTreeItemId parent, const wxString& dirname, const wxString& knownSubdir = _T(""));
+	void DisplayDir(wxTreeItemId parent, const wxString& dirname, std::wstring const& knownSubdir = std::wstring());
 	wxString HasSubdir(const wxString& dirname);
 	wxTreeItemId MakeSubdirs(wxTreeItemId parent, wxString dirname, wxString subDir);
 	wxString m_currentDir;
@@ -91,7 +93,8 @@ protected:
 
 	wxTreeItemId m_contextMenuItem;
 	wxTreeItemId m_dropHighlight;
-	enum CFileListCtrlSortBase::NameSortMode m_nameSortMode;
+
+	std::unique_ptr<CWindowTinter> m_windowTinter;
 };
 
 #endif
