@@ -1,54 +1,57 @@
+
 #include <filezilla.h>
 #include "Mainfrm.h"
 
+#include "aboutdialog.h"
+#include "asyncrequestqueue.h"
+#include "auto_ascii_files.h"
+#include "bookmarks_dialog.h"
+#include "buildinfo.h"
+#include "clearprivatedata.h"
+#include "cmdline.h"
+#include "commandqueue.h"
+#include "conditionaldialog.h"
+#include "context_control.h"
+#include "defaultfileexistsdlg.h"
+#include "edithandler.h"
+#include "export.h"
+#include "filelist_statusbar.h"
+#include "filezillaapp.h"
+#include "filter.h"
+#include "import.h"
+#include "inputdialog.h"
+#include "led.h"
+#include "local_recursive_operation.h"
 #include "LocalListView.h"
 #include "LocalTreeView.h"
+#include "loginmanager.h"
+#include "manual_transfer.h"
+#include "menu_bar.h"
+#include "netconfwizard.h"
+#include "Options.h"
+#include "power_management.h"
 #include "queue.h"
+#include "quickconnectbar.h"
+#include "remote_recursive_operation.h"
 #include "RemoteListView.h"
 #include "RemoteTreeView.h"
+#include "search.h"
+#include "settings/settingsdialog.h"
+#include "sitemanager_dialog.h"
+#include "speedlimits_dialog.h"
+#include "splitter.h"
 #include "StatusView.h"
 #include "state.h"
-#include "Options.h"
-#include "asyncrequestqueue.h"
-#include "commandqueue.h"
-#include "led.h"
-#include "sitemanager_dialog.h"
-#include "settings/settingsdialog.h"
 #include "themeprovider.h"
-#include "filezillaapp.h"
+#include "toolbar.h"
+#include "update_dialog.h"
+#include "updater.h"
 #include "view.h"
 #include "viewheader.h"
-#include "aboutdialog.h"
-#include "filter.h"
-#include "netconfwizard.h"
-#include "quickconnectbar.h"
-#include "updater.h"
-#include "update_dialog.h"
-#include "defaultfileexistsdlg.h"
-#include "loginmanager.h"
-#include "conditionaldialog.h"
-#include "clearprivatedata.h"
-#include "export.h"
-#include "import.h"
-#include "recursive_operation.h"
-#include <wx/tokenzr.h>
-#include "edithandler.h"
-#include "inputdialog.h"
-#include "window_state_manager.h"
-#include "cmdline.h"
-#include "buildinfo.h"
-#include "filelist_statusbar.h"
-#include "manual_transfer.h"
-#include "auto_ascii_files.h"
-#include "splitter.h"
-#include "bookmarks_dialog.h"
-#include "search.h"
-#include "power_management.h"
 #include "welcome_dialog.h"
-#include "context_control.h"
-#include "speedlimits_dialog.h"
-#include "toolbar.h"
-#include "menu_bar.h"
+#include "window_state_manager.h"
+
+#include <wx/tokenzr.h>
 
 #ifdef __WXMSW__
 #include <wx/module.h>
@@ -117,7 +120,6 @@ bool HandleKeyboardCommand(wxCommandEvent& event, wxWindow& parent)
 BEGIN_EVENT_TABLE(CMainFrame, wxNavigationEnabled<wxFrame>)
 	EVT_SIZE(CMainFrame::OnSize)
 	EVT_MENU(wxID_ANY, CMainFrame::OnMenuHandler)
-	EVT_FZ_NOTIFICATION(wxID_ANY, CMainFrame::OnEngineEvent)
 	EVT_COMMAND(wxID_ANY, fzEVT_UPDATE_LED_TOOLTIP, CMainFrame::OnUpdateLedTooltip)
 	EVT_TOOL(XRCID("ID_TOOLBAR_DISCONNECT"), CMainFrame::OnDisconnect)
 	EVT_MENU(XRCID("ID_MENU_SERVER_DISCONNECT"), CMainFrame::OnDisconnect)
@@ -181,11 +183,10 @@ BEGIN_EVENT_TABLE(CMainFrame, wxNavigationEnabled<wxFrame>)
 	EVT_MENU(XRCID("ID_MENU_FILE_CLOSETAB"), CMainFrame::OnMenuCloseTab)
 END_EVENT_TABLE()
 
-class CMainFrameStateEventHandler : public CStateEventHandler
+class CMainFrameStateEventHandler final : public CGlobalStateEventHandler
 {
 public:
 	CMainFrameStateEventHandler(CMainFrame* pMainFrame)
-		: CStateEventHandler(0)
 	{
 		m_pMainFrame = pMainFrame;
 
@@ -196,10 +197,9 @@ public:
 	}
 
 protected:
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*)
 	{
-		if (notification == STATECHANGE_CHANGEDCONTEXT)
-		{
+		if (notification == STATECHANGE_CHANGEDCONTEXT) {
 			// Update window title
 			const CServer* pServer = pState ? pState->GetServer() : 0;
 			if (!pServer)
@@ -213,36 +213,31 @@ protected:
 		if (!pState)
 			return;
 
-		CContextControl::_context_controls* controls = m_pMainFrame->m_pContextControl->GetControlsFromState(pState);
-		if (!controls)
+		if (!m_pMainFrame->m_pContextControl) {
 			return;
+		}
 
-		if (controls->tab_index == -1)
-		{
+		CContextControl::_context_controls* controls = m_pMainFrame->m_pContextControl->GetControlsFromState(pState);
+		if (!controls) {
+			return;
+		}
+
+		if (!controls->used()) {
 			if (notification == STATECHANGE_REMOTE_IDLE || notification == STATECHANGE_SERVER)
 				pState->Disconnect();
 
 			return;
 		}
 
-		if (notification == STATECHANGE_SERVER)
-		{
+		if (notification == STATECHANGE_SERVER) {
 			const CServer* pServer = pState->GetServer();
 
-			if (pState == CContextManager::Get()->GetCurrentContext())
-			{
-				if (!pServer)
+			if (pState == CContextManager::Get()->GetCurrentContext()) {
+				if (!pServer) {
 					m_pMainFrame->SetTitle(_T("FileZilla"));
-				else
-				{
+				}
+				else {
 					m_pMainFrame->SetTitle(pState->GetTitle() + _T(" - FileZilla"));
-					if (pServer->GetName().empty())
-					{
-						// Can only happen through quickconnect bar
-						CMenuBar* pMenuBar = wxDynamicCast(m_pMainFrame->GetMenuBar(), CMenuBar);
-						if (pMenuBar)
-							pMenuBar->ClearBookmarks();
-					}
 				}
 			}
 
@@ -259,7 +254,7 @@ CMainFrame::CMainFrame()
 {
 	m_pActivityLed[0] = m_pActivityLed[1] = 0;
 
-	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CMainFrame"));
+	wxGetApp().AddStartupProfileRecord("CMainFrame::CMainFrame");
 	wxRect screen_size = CWindowStateManager::GetScreenDimensions();
 
 	wxSize initial_size;
@@ -307,14 +302,16 @@ CMainFrame::CMainFrame()
 	m_closeEventTimer.SetOwner(this);
 
 	if (CFilterManager::HasActiveFilters(true)) {
-		if (COptions::Get()->GetOptionVal(OPTION_FILTERTOGGLESTATE))
+		if (COptions::Get()->GetOptionVal(OPTION_FILTERTOGGLESTATE)) {
 			CFilterManager::ToggleFilters();
+		}
 	}
 
 	CreateMenus();
 	CreateMainToolBar();
-	if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUICKCONNECT))
+	if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUICKCONNECT)) {
 		CreateQuickconnectBar();
+	}
 
 	m_pAsyncRequestQueue = new CAsyncRequestQueue(this);
 
@@ -430,9 +427,8 @@ CMainFrame::CMainFrame()
 
 	FixTabOrder();
 
-	RegisterOption(OPTION_LANGUAGE);
-	RegisterOption(OPTION_THEME);
-	RegisterOption(OPTION_THEME_ICONSIZE);
+	RegisterOption(OPTION_ICONS_THEME);
+	RegisterOption(OPTION_ICONS_SCALE);
 	RegisterOption(OPTION_MESSAGELOG_POSITION);
 	RegisterOption(OPTION_FILEPANE_LAYOUT);
 	RegisterOption(OPTION_FILEPANE_SWAP);
@@ -446,6 +442,9 @@ CMainFrame::~CMainFrame()
 
 	delete m_pStateEventHandler;
 
+	delete m_pContextControl;
+	m_pContextControl = 0;
+
 	CContextManager::Get()->DestroyAllStates();
 	delete m_pAsyncRequestQueue;
 #if FZ_MANUALUPDATECHECK
@@ -505,7 +504,7 @@ void CMainFrame::OnSize(wxSizeEvent &event)
 
 bool CMainFrame::CreateMenus()
 {
-	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CreateMenus"));
+	wxGetApp().AddStartupProfileRecord("CMainFrame::CreateMenus");
 	CMenuBar* old = m_pMenuBar;
 
 	m_pMenuBar = CMenuBar::Load(this);
@@ -523,7 +522,7 @@ bool CMainFrame::CreateMenus()
 
 bool CMainFrame::CreateQuickconnectBar()
 {
-	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CreateQuickconnectBar"));
+	wxGetApp().AddStartupProfileRecord("CMainFrame::CreateQuickconnectBar");
 	delete m_pQuickconnectBar;
 
 	m_pQuickconnectBar = new CQuickconnectBar();
@@ -597,7 +596,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			wxBell();
 			return;
 		}
-		pState->m_pCommandQueue->ProcessCommand(new CRawCommand(dlg.GetValue()));
+		pState->m_pCommandQueue->ProcessCommand(new CRawCommand(dlg.GetValue().ToStdWstring()));
 	}
 	else if (event.GetId() == XRCID("wxID_PREFERENCES")) {
 		OnMenuEditSettings(event);
@@ -612,12 +611,23 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		CInputDialog dlg;
 		dlg.Create(this, _T("Ciphers"), _T("Priority string:"));
 		dlg.AllowEmpty(true);
-		if (dlg.ShowModal() == wxID_OK)
-			wxMessageBoxEx(ListTlsCiphers(dlg.GetValue()), _T("Ciphers"));
+		if (dlg.ShowModal() == wxID_OK) {
+			std::string ciphers = ListTlsCiphers(fz::to_string(dlg.GetValue().ToStdWstring()));
+			wxMessageBoxEx(fz::to_wstring(ciphers), _T("Ciphers"));
+		}
 	}
 	else if (event.GetId() == XRCID("ID_CLEARCACHE_LAYOUT")) {
 		CWrapEngine::ClearCache();
 	}
+	else if (event.GetId() == XRCID("ID_CLEAR_UPDATER")) {
+#if FZ_MANUALUPDATECHECK
+		if (m_pUpdater) {
+			COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, std::wstring());
+			COptions::Get()->SetOption(OPTION_UPDATECHECK_NEWVERSION, std::wstring());
+			m_pUpdater->Init();
+		}
+#endif
+	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_FILEEXISTS")) {
 		CDefaultFileExistsDlg dlg;
 		if (!dlg.Load(this, false))
@@ -745,8 +755,9 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		}
 	}
 	else if (event.GetId() == XRCID("ID_VIEW_QUICKCONNECT")) {
-		if (!m_pQuickconnectBar)
+		if (!m_pQuickconnectBar) {
 			CreateQuickconnectBar();
+		}
 		else {
 			m_pQuickconnectBar->Destroy();
 			m_pQuickconnectBar = 0;
@@ -770,42 +781,37 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		if (!pState) {
 			return;
 		}
-		CServer const* pServer = pState->GetServer();
+
+		Site const old_site = pState->GetSite();
+		std::wstring sitePath = old_site.m_path;
 
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
-		if (!controls)
+		if (!controls) {
 			return;
-		if (!pServer && !controls->site_bookmarks->path.empty()) {
-			// Get server from site manager
-			std::unique_ptr<CSiteManagerItemData_Site> data = CSiteManager::GetSiteByPath(controls->site_bookmarks->path);
-			if (data) {
-				server = data->m_server;
-				pServer = &server;
-			}
-			else {
-				controls->site_bookmarks->path.clear();
-				controls->site_bookmarks->bookmarks.clear();
-				if (m_pMenuBar)
-					m_pMenuBar->UpdateBookmarkMenu();
-			}
 		}
 
 		// controls->last_bookmark_path can get modified if it's empty now
 		int res;
 		if (event.GetId() == XRCID("ID_BOOKMARK_ADD")) {
-			CNewBookmarkDialog dlg(this, controls->site_bookmarks->path, pServer);
+			CNewBookmarkDialog dlg(this, sitePath, old_site.m_server ? &old_site.m_server : 0);
 			res = dlg.Run(pState->GetLocalDir().GetPath(), pState->GetRemotePath());
 		}
 		else {
-			CBookmarksDialog dlg(this, controls->site_bookmarks->path, pServer);
-
+			CBookmarksDialog dlg(this, sitePath, old_site.m_server ? &old_site.m_server : 0);
 			res = dlg.Run();
 		}
 		if (res == wxID_OK) {
-			controls->site_bookmarks->bookmarks.clear();
-			CSiteManager::GetBookmarks(controls->site_bookmarks->path, controls->site_bookmarks->bookmarks);
-			if (m_pMenuBar)
-				m_pMenuBar->UpdateBookmarkMenu();
+			if (!sitePath.empty()) {
+				std::unique_ptr<Site> site = CSiteManager::GetSiteByPath(sitePath, false).first;
+				if (site) {
+					for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
+						CContextControl::_context_controls *tab_controls = m_pContextControl->GetControlsFromTabIndex(i);
+						if (tab_controls) {
+							tab_controls->pState->UpdateSite(old_site.m_path, *site);
+						}
+					}
+				}
+			}
 		}
 	}
 	else if (event.GetId() == XRCID("ID_MENU_HELP_WELCOME")) {
@@ -837,8 +843,9 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		COptions::Get()->SetOption(OPTION_COMPARISONMODE, old_mode ? 0 : 1);
 
 		CComparisonManager* pComparisonManager = pState->GetComparisonManager();
-		if (pComparisonManager && pComparisonManager->IsComparing())
+		if (pComparisonManager && pComparisonManager->IsComparing()) {
 			pComparisonManager->CompareListings();
+		}
 	}
 	else if (HandleKeyboardCommand(event, *this)) {
 		return;
@@ -859,23 +866,28 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			return;
 		}
 
-		std::unique_ptr<CSiteManagerItemData_Site> pData = CSiteManager::GetSiteById(event.GetId());
+		std::unique_ptr<Site> pData = CSiteManager::GetSiteById(event.GetId());
 
 		if (!pData) {
 			event.Skip();
 		}
 		else {
-			ConnectToSite(*pData);
+			ConnectToSite(*pData, pData->m_default_bookmark);
 		}
 	}
 }
 
-void CMainFrame::OnEngineEvent(wxFzEvent &event)
+void CMainFrame::OnEngineEvent(CFileZillaEngine* engine)
+{
+	CallAfter(&CMainFrame::DoOnEngineEvent, engine);
+}
+
+void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 {
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 	CState* pState = 0;
 	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
-		if ((*iter)->m_pEngine != event.engine_)
+		if ((*iter)->m_pEngine != engine)
 			continue;
 
 		pState = *iter;
@@ -925,7 +937,9 @@ void CMainFrame::OnEngineEvent(wxFzEvent &event)
 			}
 			break;
 		case nId_transferstatus:
-			m_pQueueView->ProcessNotification(pState->m_pEngine, std::move(pNotification));
+			if (m_pQueueView) {
+				m_pQueueView->ProcessNotification(pState->m_pEngine, std::move(pNotification));
+			}
 			break;
 		case nId_sftp_encryption:
 			{
@@ -971,7 +985,7 @@ void CMainFrame::OnUpdateLedTooltip(wxCommandEvent&)
 
 bool CMainFrame::CreateMainToolBar()
 {
-	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CreateMainToolBar"));
+	wxGetApp().AddStartupProfileRecord("CMainFrame::CreateMainToolBar");
 	if (m_pToolBar)
 	{
 #ifdef __WXMAC__
@@ -1020,10 +1034,9 @@ void CMainFrame::OnCancel(wxCommandEvent&)
 	if (!pState || pState->m_pCommandQueue->Idle())
 		return;
 
-	if (wxMessageBoxEx(_("Really cancel current operation?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) == wxYES)
-	{
+	if (wxMessageBoxEx(_("Really cancel current operation?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) == wxYES) {
 		pState->m_pCommandQueue->Cancel();
-		pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
+		pState->GetRemoteRecursiveOperation()->StopRecursiveOperation();
 	}
 }
 
@@ -1066,20 +1079,16 @@ bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 
 	int size = wxTopLevelWindows.size();
 	static wxTopLevelWindow* pLast = 0;
-	if (wxTopLevelWindows.size())
-	{
+	if (wxTopLevelWindows.size()) {
 		wxWindowList::reverse_iterator iter = wxTopLevelWindows.rbegin();
 		wxTopLevelWindow* pTop = (wxTopLevelWindow*)(*iter);
-		while (pTop != this && (size != prev_size || pLast != pTop))
-		{
-			wxDialog* pDialog = wxDynamicCast(pTop, wxDialog);
+		while (pTop != this && (size != prev_size || pLast != pTop)) {
+			wxDialog* pDialog = dynamic_cast<wxDialog*>(pTop);
 			if (pDialog)
 				pDialog->EndModal(wxID_CANCEL);
-			else
-			{
+			else {
 				wxWindow* pParent = pTop->GetParent();
-				if (m_pQueuePane && pParent == m_pQueuePane)
-				{
+				if (m_pQueuePane && pParent == m_pQueuePane) {
 					// It's the AUI frame manager hint window. Ignore it
 					++iter;
 					if (iter == wxTopLevelWindows.rend())
@@ -1217,8 +1226,11 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	bool res = true;
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 	for (CState* pState : *pStates) {
-		if( pState->GetRecursiveOperationHandler() ) {
-			pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
+		if (pState->GetLocalRecursiveOperation()) {
+			pState->GetLocalRecursiveOperation()->StopRecursiveOperation();
+		}
+		if (pState->GetRemoteRecursiveOperation()) {
+			pState->GetRemoteRecursiveOperation()->StopRecursiveOperation();
 		}
 
 		if (pState->m_pCommandQueue) {
@@ -1236,9 +1248,10 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 
 	CContextControl::_context_controls* controls = m_pContextControl ? m_pContextControl->GetCurrentControls() : 0;
 	if (controls) {
-		COptions::Get()->SetLastServer(controls->pState->GetLastServer());
+		Site site = controls->pState->GetLastSite();
+		COptions::Get()->SetLastServer(site.m_server);
 		COptions::Get()->SetOption(OPTION_LASTSERVERPATH, controls->pState->GetLastServerPath().GetSafePath());
-		COptions::Get()->SetOption(OPTION_LAST_CONNECTED_SITE, controls->site_bookmarks ? controls->site_bookmarks->path : wxString());
+		COptions::Get()->SetOption(OPTION_LAST_CONNECTED_SITE, site.m_path);
 	}
 
 	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
@@ -1262,21 +1275,19 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 void CMainFrame::OnReconnect(wxCommandEvent &)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
+	if (!pState) {
 		return;
-
-	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle())
-		return;
-
-	CServer server = pState->GetLastServer();
-
-	if (server.GetLogonType() == ASK) {
-		if (!CLoginManager::Get().GetPassword(server, false))
-			return;
 	}
 
+	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
+		return;
+	}
+
+	Site site = pState->GetLastSite();
 	CServerPath path = pState->GetLastServerPath();
-	ConnectToServer(server, path, true);
+	Bookmark bm;
+	bm.m_remoteDir = path;
+	ConnectToSite(site, bm);
 }
 
 void CMainFrame::OnRefresh(wxCommandEvent &)
@@ -1291,8 +1302,7 @@ void CMainFrame::OnRefresh(wxCommandEvent &)
 
 void CMainFrame::OnTimer(wxTimerEvent& event)
 {
-	if (event.GetId() == m_closeEventTimer.GetId())
-	{
+	if (event.GetId() == m_closeEventTimer.GetId()) {
 		if (m_closeEvent == 0)
 			return;
 
@@ -1309,82 +1319,76 @@ void CMainFrame::OnTimer(wxTimerEvent& event)
 #endif
 }
 
-void CMainFrame::OpenSiteManager(const CServer* pServer /*=0*/)
+void CMainFrame::OpenSiteManager(CServer const* pServer)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
+	if (!pState) {
 		return;
+	}
 
 	CSiteManagerDialog dlg;
 
-	std::set<wxString> handled_paths;
+	std::set<std::wstring> handled_paths;
 	std::vector<CSiteManagerDialog::_connected_site> connected_sites;
 
-	if (pServer)
-	{
+	if (pServer) {
 		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.server = *pServer;
 		connected_sites.push_back(connected_site);
 	}
 
-	for (int i = 0; i < m_pContextControl->GetTabCount(); i++)
-	{
+	for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
 		CContextControl::_context_controls *controls =  m_pContextControl->GetControlsFromTabIndex(i);
-		if (!controls)
+		if (!controls) {
 			continue;
+		}
 
-		const wxString& path = controls->site_bookmarks->path;
-		if (path.empty())
+		Site const& site = controls->pState->GetSite();
+		std::wstring const& path = site.m_path;
+		if (path.empty()) {
 			continue;
-		if (handled_paths.find(path) != handled_paths.end())
+		}
+		if (handled_paths.find(path) != handled_paths.end()) {
 			continue;
+		}
 
 		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.old_path = path;
-		connected_site.server = controls->pState->GetLastServer();
+		connected_site.server = site.m_server;
 		connected_sites.push_back(connected_site);
 		handled_paths.insert(path);
 	}
 
-	if (!dlg.Create(this, &connected_sites, pServer))
+	if (!dlg.Create(this, &connected_sites, pServer)) {
 		return;
+	}
 
 	int res = dlg.ShowModal();
-	if (res == wxID_YES || res == wxID_OK)
-	{
+	if (res == wxID_YES || res == wxID_OK) {
 		// Update bookmark paths
-		for (size_t j = 0; j < connected_sites.size(); j++)
-		{
-			for (int i = 0; i < m_pContextControl->GetTabCount(); i++)
-			{
-				CContextControl::_context_controls *controls =  m_pContextControl->GetControlsFromTabIndex(i);
-				if (!controls)
-					continue;
-
-				if (connected_sites[j].old_path != controls->site_bookmarks->path)
-					continue;
-
-				controls->site_bookmarks->path = connected_sites[j].new_path;
-
-				controls->site_bookmarks->bookmarks.clear();
-				CSiteManager::GetBookmarks(controls->site_bookmarks->path, controls->site_bookmarks->bookmarks);
-
-				break;
+		for (auto const& connected_site : connected_sites) {
+			std::unique_ptr<Site> site = CSiteManager::GetSiteByPath(connected_site.new_path, false).first;
+			if (site) {
+				for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
+					CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
+					if (!controls) {
+						continue;
+					}
+					controls->pState->UpdateSite(connected_site.old_path, *site);
+				}
 			}
 		}
 	}
 
-	if (res == wxID_YES)
-	{
-		CSiteManagerItemData_Site data;
-		if (!dlg.GetServer(data))
+	if (res == wxID_YES) {
+		Site data;
+		Bookmark bookmark;
+		if (!dlg.GetServer(data, bookmark)) {
 			return;
+		}
 
-		ConnectToSite(data);
+		ConnectToSite(data, bookmark);
 	}
-
-	if (m_pMenuBar)
-		m_pMenuBar->UpdateBookmarkMenu();
 }
 
 void CMainFrame::OnSiteManager(wxCommandEvent& e)
@@ -1404,14 +1408,16 @@ void CMainFrame::OnSiteManager(wxCommandEvent& e)
 
 void CMainFrame::UpdateActivityLed(int direction)
 {
-	if (m_pActivityLed[direction])
+	if (m_pActivityLed[direction]) {
 		m_pActivityLed[direction]->Ping();
+	}
 }
 
 void CMainFrame::OnProcessQueue(wxCommandEvent& event)
 {
-	if (m_pQueueView)
+	if (m_pQueueView) {
 		m_pQueueView->SetActive(event.IsChecked());
+	}
 }
 
 void CMainFrame::OnMenuEditSettings(wxCommandEvent&)
@@ -1659,12 +1665,12 @@ void CMainFrame::UpdaterStateChanged(UpdaterState s, build const& v)
 	wxString const name = wxString::Format(_("&Version %s"), v.version_);
 
 	wxMenuItem* pItem = m_pMenuBar->FindItem(GetAvailableUpdateMenuId());
-	if( !pItem ) {
+	if (!pItem) {
 		wxMenu* pMenu = new wxMenu();
 		pMenu->Append(GetAvailableUpdateMenuId(), name);
 		m_pMenuBar->Append(pMenu, _("&New version available!"));
 
-		if( !update_dialog_timer_.IsRunning() ) {
+		if (!update_dialog_timer_.IsRunning()) {
 			update_dialog_timer_.Start(1, true);
 		}
 	}
@@ -1676,12 +1682,16 @@ void CMainFrame::UpdaterStateChanged(UpdaterState s, build const& v)
 
 void CMainFrame::TriggerUpdateDialog()
 {
-	if( CUpdateDialog::IsRunning() ) {
+	if (m_bQuit) {
 		return;
 	}
 
-	if( !wxDialogEx::CanShowPopupDialog() ) {
-		update_dialog_timer_.Start( 1000, true );
+	if (CUpdateDialog::IsRunning()) {
+		return;
+	}
+
+	if (!wxDialogEx::CanShowPopupDialog()) {
+		update_dialog_timer_.Start(1000, true);
 		return;
 	}
 
@@ -1758,7 +1768,7 @@ void CMainFrame::UpdateLayout()
 	// Now the other panes
 	for (int i = 0; i < m_pContextControl->GetTabCount(); i++)
 	{
-		struct CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
+		CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
 		if (!controls)
 			continue;
 
@@ -1883,40 +1893,80 @@ void CMainFrame::OnSitemanagerDropdown(wxCommandEvent& event)
 	}
 }
 
-bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site & data, bool newTab)
+bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark)
 {
+	// First check if we need to ask user for a password
 	if (data.m_server.GetLogonType() == ASK ||
 		(data.m_server.GetLogonType() == INTERACTIVE && data.m_server.GetUser().empty()))
 	{
-		if (!CLoginManager::Get().GetPassword(data.m_server, false, data.m_server.GetName()))
+		if (!CLoginManager::Get().GetPassword(data.m_server, false, data.m_server.GetName())) {
 			return false;
+		}
 	}
 
-	if (newTab)
-		m_pContextControl->CreateTab();
-
-	if (!ConnectToServer(data.m_server, data.m_remoteDir))
+	// Check if current state is already connected and if needed ask whether to open in new tab
+	CState* pState = CContextManager::Get()->GetCurrentContext();
+	if (!pState) {
 		return false;
+	}
 
-	CState *pState = CContextManager::Get()->GetCurrentContext();
-	if (pState) {
-		if (!data.m_localDir.empty()) {
-			bool set = pState->SetLocalDir(data.m_localDir, 0, false);
-
-			if (set && data.m_sync) {
-				wxASSERT(!data.m_remoteDir.empty());
-				pState->SetSyncBrowse(true, data.m_remoteDir);
+	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
+		int action = COptions::Get()->GetOptionVal(OPTION_ALREADYCONNECTED_CHOICE);
+		if (action < 2) {
+			wxDialogEx dlg;
+			if (!dlg.Load(this, _T("ID_ALREADYCONNECTED"))) {
+				return false;
 			}
+
+			if (action != 0) {
+				XRCCTRL(dlg, "ID_OLDTAB", wxRadioButton)->SetValue(true);
+			}
+			else {
+				XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->SetValue(true);
+			}
+
+			if (dlg.ShowModal() != wxID_OK) {
+				return false;
+			}
+
+			if (XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->GetValue()) {
+				action = 0;
+			}
+			else {
+				action = 1;
+			}
+
+			if (XRCCTRL(dlg, "ID_REMEMBER", wxCheckBox)->IsChecked()) {
+				action |= 2;
+			}
+			COptions::Get()->SetOption(OPTION_ALREADYCONNECTED_CHOICE, action);
 		}
 
-		if (data.m_comparison && pState->GetComparisonManager()) {
-			pState->GetComparisonManager()->CompareListings();
+		if (!(action & 1)) {
+			m_pContextControl->CreateTab();
+			pState = CContextManager::Get()->GetCurrentContext();
 		}
 	}
 
-	SetBookmarksFromPath(data.m_path);
-	if (m_pMenuBar)
-		m_pMenuBar->UpdateBookmarkMenu();
+	// Next tell the state to connect
+	if (!pState->Connect(data, bookmark.m_remoteDir, bookmark.m_comparison)) {
+		return false;
+	}
+
+	// Apply comparison and sync browsing options
+	// FIXME: Move to state?
+	if (!bookmark.m_localDir.empty()) {
+		bool set = pState->SetLocalDir(bookmark.m_localDir.ToStdWstring(), 0, false);
+
+		if (set && bookmark.m_sync) {
+			wxASSERT(!bookmark.m_remoteDir.empty());
+			pState->SetSyncBrowse(true, bookmark.m_remoteDir);
+		}
+	}
+
+	if (bookmark.m_comparison && pState->GetComparisonManager()) {
+		pState->GetComparisonManager()->CompareListings();
+	}
 
 	return true;
 }
@@ -1928,7 +1978,9 @@ void CMainFrame::CheckChangedSettings()
 	CAutoAsciiFiles::SettingsChanged();
 
 #if FZ_MANUALUPDATECHECK
-	m_pUpdater->Init();
+	if (m_pUpdater) {
+		m_pUpdater->Init();
+	}
 #endif
 }
 
@@ -2048,7 +2100,7 @@ void CMainFrame::RememberSplitterPositions()
 	// m_lastQueueLogSplitterPos is a value between 0 and 1
 	posString += wxString::Format(_T("%d"), (int)(m_pQueueLogSplitter->GetRelativeSashPosition() * 1000000000));
 
-	COptions::Get()->SetOption(OPTION_MAINWINDOW_SPLITTER_POSITION, posString);
+	COptions::Get()->SetOption(OPTION_MAINWINDOW_SPLITTER_POSITION, posString.ToStdWstring());
 }
 
 bool CMainFrame::RestoreSplitterPositions()
@@ -2145,20 +2197,21 @@ void CMainFrame::OnActivate(wxActivateEvent& event)
 void CMainFrame::OnToolbarComparison(wxCommandEvent&)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
+	if (!pState) {
 		return;
+	}
 
 	CComparisonManager* pComparisonManager = pState->GetComparisonManager();
-	if (pComparisonManager->IsComparing())
-	{
+	if (pComparisonManager->IsComparing()) {
 		pComparisonManager->ExitComparisonMode();
 		return;
 	}
 
 	if (!COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT)) {
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
-		if (!controls)
+		if (!controls) {
 			return;
+		}
 
 		if ((controls->pLocalSplitter->IsSplit() && !controls->pRemoteSplitter->IsSplit()) ||
 			(!controls->pLocalSplitter->IsSplit() && controls->pRemoteSplitter->IsSplit()))
@@ -2239,44 +2292,47 @@ void CMainFrame::ShowDropdownMenu(wxMenu* pMenu, wxToolBar* pToolBar, wxCommandE
 void CMainFrame::OnDropdownComparisonMode(wxCommandEvent& event)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
+	if (!pState) {
 		return;
+	}
 
 	int old_mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
 	int new_mode = (event.GetId() == XRCID("ID_COMPARE_SIZE")) ? 0 : 1;
 	COptions::Get()->SetOption(OPTION_COMPARISONMODE, new_mode);
 
 	CComparisonManager* pComparisonManager = pState->GetComparisonManager();
-	if (old_mode != new_mode && pComparisonManager && pComparisonManager->IsComparing())
+	if (old_mode != new_mode && pComparisonManager && pComparisonManager->IsComparing()) {
 		pComparisonManager->CompareListings();
+	}
 }
 
 void CMainFrame::OnDropdownComparisonHide(wxCommandEvent&)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
+	if (!pState) {
 		return;
+	}
 
 	bool old_mode = COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0;
 	COptions::Get()->SetOption(OPTION_COMPARE_HIDEIDENTICAL, old_mode ? 0 : 1);
 
 	CComparisonManager* pComparisonManager = pState->GetComparisonManager();
-	if (pComparisonManager && pComparisonManager->IsComparing())
+	if (pComparisonManager && pComparisonManager->IsComparing()) {
 		pComparisonManager->CompareListings();
+	}
 }
 
 void CMainFrame::ProcessCommandLine()
 {
 	const CCommandLine* pCommandLine = wxGetApp().GetCommandLine();
-	if (!pCommandLine)
+	if (!pCommandLine) {
 		return;
+	}
 
-	wxString local;
-	if ((local = pCommandLine->GetOption(CCommandLine::local)) != _T(""))
-	{
+	std::wstring local;
+	if ((local = pCommandLine->GetOption(CCommandLine::local)) != _T("")) {
 
-		if (!wxDir::Exists(local))
-		{
+		if (!wxDir::Exists(local)) {
 			wxString str = _("Path not found:");
 			str += _T("\n") + local;
 			wxMessageBoxEx(str, _("Syntax error in command line"));
@@ -2284,13 +2340,14 @@ void CMainFrame::ProcessCommandLine()
 		}
 
 		CState *pState = CContextManager::Get()->GetCurrentContext();
-		if (!pState)
+		if (!pState) {
 			return;
+		}
 
 		pState->SetLocalDir(local);
 	}
 
-	wxString site;
+	std::wstring site;
 	if (pCommandLine->HasSwitch(CCommandLine::sitemanager)) {
 		if (COptions::Get()->GetOptionVal(OPTION_INTERFACE_SITEMANAGER_ON_STARTUP) == 0) {
 			Show();
@@ -2298,27 +2355,29 @@ void CMainFrame::ProcessCommandLine()
 		}
 	}
 	else if ((site = pCommandLine->GetOption(CCommandLine::site)) != _T("")) {
-		std::unique_ptr<CSiteManagerItemData_Site> pData = CSiteManager::GetSiteByPath(site);
+		auto const data = CSiteManager::GetSiteByPath(site);
 
-		if (pData) {
-			ConnectToSite(*pData);
+		if (data.first) {
+			ConnectToSite(*data.first, data.second);
 		}
 	}
 
-	wxString param = pCommandLine->GetParameter();
+	std::wstring param = pCommandLine->GetParameter().ToStdWstring();
 	if (!param.empty()) {
-		wxString error;
+		std::wstring error;
 
 		CServer server;
 
 		wxString logontype = pCommandLine->GetOption(CCommandLine::logontype);
-		if (logontype == _T("ask"))
+		if (logontype == _T("ask")) {
 			server.SetLogonType(ASK);
-		else if (logontype == _T("interactive"))
+		}
+		else if (logontype == _T("interactive")) {
 			server.SetLogonType(INTERACTIVE);
+		}
 
 		CServerPath path;
-		if (!server.ParseUrl(param, 0, _T(""), _T(""), error, path)) {
+		if (!server.ParseUrl(param, 0, std::wstring(), std::wstring(), error, path)) {
 			wxString str = _("Parameter not a valid URL");
 			str += _T("\n") + error;
 			wxMessageBoxEx(error, _("Syntax error in command line"));
@@ -2331,15 +2390,16 @@ void CMainFrame::ProcessCommandLine()
 		else if (server.GetLogonType() == ASK ||
 			(server.GetLogonType() == INTERACTIVE && server.GetUser().empty()))
 		{
-			if (!CLoginManager::Get().GetPassword(server, false))
+			if (!CLoginManager::Get().GetPassword(server, false)) {
 				return;
+			}
 		}
 
-		CState* pState = CContextManager::Get()->GetCurrentContext();
-		if (!pState)
-			return;
-
-		ConnectToServer(server, path);
+		Site site;
+		site.m_server = server;
+		Bookmark bm;
+		bm.m_remoteDir = path;
+		ConnectToSite(site, bm);
 	}
 }
 
@@ -2349,8 +2409,9 @@ void CMainFrame::OnFilterRightclicked(wxCommandEvent&)
 
 	CFilterManager::ToggleFilters();
 
-	if (active == CFilterManager::HasActiveFilters())
+	if (active == CFilterManager::HasActiveFilters()) {
 		return;
+	}
 
 	CContextManager::Get()->NotifyAllHandlers(STATECHANGE_APPLYFILTER);
 }
@@ -2486,17 +2547,14 @@ void CMainFrame::OnTaskBarClick_Delayed(wxCommandEvent&)
 void CMainFrame::OnSearch(wxCommandEvent&)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
-		return;
-
-	if (!pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
-		wxBell();
+	if (!pState) {
 		return;
 	}
 
-	CSearchDialog dlg(this, pState, m_pQueueView);
-	if (!dlg.Load())
+	CSearchDialog dlg(this, *pState, m_pQueueView);
+	if (!dlg.Load()) {
 		return;
+	}
 
 	dlg.Run();
 }
@@ -2538,84 +2596,6 @@ void CMainFrame::OnMenuCloseTab(wxCommandEvent&)
 	m_pContextControl->CloseTab(m_pContextControl->GetCurrentTab());
 }
 
-void CMainFrame::SetBookmarksFromPath(const wxString& path)
-{
-	if (!m_pContextControl)
-		return;
-
-	std::shared_ptr<CContextControl::_context_controls::_site_bookmarks> site_bookmarks;
-	for (int i = 0; i < m_pContextControl->GetTabCount(); i++) {
-		if (i == m_pContextControl->GetCurrentTab())
-			continue;
-
-		CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
-		if (!controls || !controls->site_bookmarks || controls->site_bookmarks->path != path)
-			continue;
-
-		site_bookmarks = controls->site_bookmarks;
-		site_bookmarks->bookmarks.clear();
-	}
-	if (!site_bookmarks) {
-		site_bookmarks = std::make_shared<CContextControl::_context_controls::_site_bookmarks>();
-		site_bookmarks->path = path;
-	}
-
-	CContextControl::_context_controls *controls = m_pContextControl->GetCurrentControls();
-	if (controls) {
-		controls->site_bookmarks = site_bookmarks;
-		CSiteManager::GetBookmarks(controls->site_bookmarks->path, controls->site_bookmarks->bookmarks);
-	}
-}
-
-bool CMainFrame::ConnectToServer(const CServer &server, const CServerPath &path /*=CServerPath()*/, bool isReconnect /*=true*/)
-{
-	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
-		return false;
-
-	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
-		int action = COptions::Get()->GetOptionVal(OPTION_ALREADYCONNECTED_CHOICE);
-		if( action < 2 ) {
-			wxDialogEx dlg;
-			if (!dlg.Load(this, _T("ID_ALREADYCONNECTED")))
-				return false;
-
-			if (action != 0)
-				XRCCTRL(dlg, "ID_OLDTAB", wxRadioButton)->SetValue(true);
-			else
-				XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->SetValue(true);
-
-			if (dlg.ShowModal() != wxID_OK)
-				return false;
-
-			if (XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->GetValue()) {
-				action = 0;
-			}
-			else {
-				action = 1;
-			}
-
-			if( XRCCTRL(dlg, "ID_REMEMBER", wxCheckBox)->IsChecked() ) {
-				action |= 2;
-			}
-			COptions::Get()->SetOption(OPTION_ALREADYCONNECTED_CHOICE, action);
-		}
-
-		if( !(action & 1) ) {
-			m_pContextControl->CreateTab();
-			pState = CContextManager::Get()->GetCurrentContext();
-		}
-	}
-
-	CContextControl::_context_controls* controls = m_pContextControl->GetControlsFromState(pState);
-	if (!isReconnect && controls) {
-		controls->site_bookmarks.reset();
-		m_pMenuBar->ClearBookmarks();
-	}
-
-	return pState->Connect(server, path);
-}
-
 void CMainFrame::OnToggleToolBar(wxCommandEvent& event)
 {
 	COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, event.IsChecked() ? 0 : 1);
@@ -2650,6 +2630,7 @@ void CMainFrame::SetupKeyboardAccelerators()
 	for (int i = 0; i < 10; i++) {
 		tab_hotkey_ids[i] = wxNewId();
 		entries.emplace_back(wxACCEL_CMD, (int)'0' + i, tab_hotkey_ids[i]);
+		entries.emplace_back(wxACCEL_ALT, (int)'0' + i, tab_hotkey_ids[i]);
 	}
 	entries.emplace_back(wxACCEL_CMD | wxACCEL_SHIFT, 'O', m_comparisonToggleAcceleratorId);
 	entries.emplace_back(wxACCEL_CMD | wxACCEL_SHIFT, 'I', XRCID("ID_MENU_VIEW_FILTERS"));
@@ -2664,6 +2645,14 @@ void CMainFrame::SetupKeyboardAccelerators()
 	for (auto const& command : keyboardCommands) {
 		entries.emplace_back(wxACCEL_CMD, command.second.second, command.first);
 	}
+
+	// Ctrl+(Shift+)Tab to switch between tabs
+	int id = wxNewId();
+	entries.emplace_back(wxACCEL_RAW_CTRL, '\t', id);
+	Bind(wxEVT_MENU, [this](wxEvent&) { if (m_pContextControl) { m_pContextControl->AdvanceTab(true); } }, id);
+	id = wxNewId();
+	entries.emplace_back(wxACCEL_RAW_CTRL | wxACCEL_SHIFT, '\t', id);
+	Bind(wxEVT_MENU, [this](wxEvent&) { if (m_pContextControl) { m_pContextControl->AdvanceTab(false); } }, id);
 #endif
 	wxAcceleratorTable accel(entries.size(), &entries[0]);
 	SetAcceleratorTable(accel);
@@ -2675,8 +2664,7 @@ void CMainFrame::OnOptionsChanged(changed_options_t const& options)
 		UpdateLayout();
 	}
 
-	bool const language_changed = options.test(OPTION_LANGUAGE);
-	if (options.test(OPTION_THEME) || options.test(OPTION_THEME_ICONSIZE) || language_changed) {
+	if (options.test(OPTION_ICONS_THEME) || options.test(OPTION_ICONS_SCALE)) {
 		CreateMainToolBar();
 		if (m_pToolBar) {
 			m_pToolBar->UpdateToolbarState();
