@@ -3,6 +3,7 @@
 
 #include "statusbar.h"
 #include "engine_context.h"
+#include "notification.h"
 
 #include <wx/timer.h>
 
@@ -22,7 +23,7 @@ class CMenuBar;
 class CQueue;
 class CQueueView;
 class CQuickconnectBar;
-class CSiteManagerItemData_Site;
+class Site;
 class CSplitterWindowEx;
 class CStatusView;
 class CState;
@@ -30,7 +31,7 @@ class CThemeProvider;
 class CToolBar;
 class CWindowStateManager;
 
-class CMainFrame final : public wxNavigationEnabled<wxFrame>, private COptionChangeEventHandler
+class CMainFrame final : public wxNavigationEnabled<wxFrame>, private COptionChangeEventHandler, public EngineNotificationHandler
 #if FZ_MANUALUPDATECHECK
 	, protected CUpdateHandler
 #endif
@@ -61,11 +62,9 @@ public:
 
 	void PostInitialize();
 
-	bool ConnectToServer(const CServer& server, const CServerPath& path = CServerPath(), bool isReconnect = false);
-
 	CContextControl* GetContextControl() { return m_pContextControl; }
 
-	bool ConnectToSite(CSiteManagerItemData_Site & data, bool newTab = false);
+	bool ConnectToSite(Site & data, Bookmark const& bookmark);
 
 	CFileZillaEngineContext& GetEngineContext() { return m_engineContext; }
 private:
@@ -80,8 +79,6 @@ private:
 
 	void FocusNextEnabled(std::list<wxWindow*>& windowOrder, std::list<wxWindow*>::iterator iter, bool skipFirst, bool forward);
 
-	void SetBookmarksFromPath(const wxString& path);
-
 	CStatusBar* m_pStatusBar{};
 	CMenuBar* m_pMenuBar{};
 	CToolBar* m_pToolBar{};
@@ -119,11 +116,13 @@ private:
 
 	void OnOptionsChanged(changed_options_t const& options);
 
+	virtual void OnEngineEvent(CFileZillaEngine* engine); // Before thread jump
+	void DoOnEngineEvent(CFileZillaEngine* engine); // After thread jump
+
 	// Event handlers
 	DECLARE_EVENT_TABLE()
 	void OnSize(wxSizeEvent& event);
 	void OnMenuHandler(wxCommandEvent& event);
-	void OnEngineEvent(wxFzEvent& event);
 	void OnUpdateLedTooltip(wxCommandEvent&);
 	void OnDisconnect(wxCommandEvent&);
 	void OnCancel(wxCommandEvent&);
