@@ -18,7 +18,6 @@ enum interfaceOptions
 	OPTION_ASCIIFILES,
 	OPTION_ASCIINOEXT,
 	OPTION_ASCIIDOTFILE,
-	OPTION_THEME,
 	OPTION_LANGUAGE,
 	OPTION_LASTSERVERPATH,
 	OPTION_CONCURRENTDOWNLOADLIMIT,
@@ -26,6 +25,7 @@ enum interfaceOptions
 	OPTION_UPDATECHECK,
 	OPTION_UPDATECHECK_INTERVAL,
 	OPTION_UPDATECHECK_LASTDATE,
+	OPTION_UPDATECHECK_LASTVERSION,
 	OPTION_UPDATECHECK_NEWVERSION,
 	OPTION_UPDATECHECK_CHECKBETA,
 	OPTION_DEBUG_MENU,
@@ -33,6 +33,7 @@ enum interfaceOptions
 	OPTION_FILEEXISTS_UPLOAD,
 	OPTION_ASCIIRESUME,
 	OPTION_GREETINGVERSION,
+	OPTION_GREETINGRESOURCES,
 	OPTION_ONETIME_DIALOGS,
 	OPTION_SHOW_TREE_LOCAL,
 	OPTION_SHOW_TREE_REMOTE,
@@ -60,7 +61,8 @@ enum interfaceOptions
 	OPTION_COMPARISONMODE,
 	OPTION_COMPARISON_THRESHOLD,
 	OPTION_SITEMANAGER_POSITION,
-	OPTION_THEME_ICONSIZE,
+	OPTION_ICONS_THEME,
+	OPTION_ICONS_SCALE,
 	OPTION_MESSAGELOG_TIMESTAMP,
 	OPTION_SITEMANAGER_LASTSELECTED,
 	OPTION_LOCALFILELIST_COLUMN_SHOWN,
@@ -94,12 +96,16 @@ enum interfaceOptions
 	OPTION_INTERFACE_SITEMANAGER_ON_STARTUP,
 	OPTION_PROMPTPASSWORDSAVE,
 	OPTION_PERSISTENT_CHOICES,
-	OPTION_SHOW_NOTIFICATION_BUBBLE,
+	OPTION_QUEUE_COMPLETION_ACTION,
+	OPTION_QUEUE_COMPLETION_COMMAND,
+	OPTION_DND_DISABLED,
+	OPTION_DISABLE_UPDATE_FOOTER,
 
 	// Default/internal options
 	OPTION_DEFAULT_SETTINGSDIR, // guaranteed to be (back)slash-terminated
 	OPTION_DEFAULT_KIOSKMODE,
 	OPTION_DEFAULT_DISABLEUPDATECHECK,
+	OPTION_DEFAULT_CACHE_DIR,
 
 	// Has to be last element
 	OPTIONS_NUM
@@ -107,25 +113,25 @@ enum interfaceOptions
 
 struct t_OptionsCache
 {
-	bool operator==(wxString const& v) const { return strValue == v; }
+	bool operator==(std::wstring const& v) const { return strValue == v; }
 	bool operator==(int v) const { return numValue == v; }
-	t_OptionsCache& operator=(wxString const& v);
+	t_OptionsCache& operator=(std::wstring const& v);
 	t_OptionsCache& operator=(int v);
 
 	bool from_default;
 	int numValue;
-	wxString strValue;
+	std::wstring strValue;
 };
 
 class CXmlFile;
-class COptions : public wxEvtHandler, public COptionsBase
+class COptions final : public wxEvtHandler, public COptionsBase
 {
 public:
 	virtual int GetOptionVal(unsigned int nID);
-	virtual wxString GetOption(unsigned int nID);
+	virtual std::wstring GetOption(unsigned int nID);
 
 	virtual bool SetOption(unsigned int nID, int value);
-	virtual bool SetOption(unsigned int nID, wxString const& value);
+	virtual bool SetOption(unsigned int nID, std::wstring const& value);
 
 	bool OptionFromFzDefaultsXml(unsigned int nID);
 
@@ -141,21 +147,22 @@ public:
 	void SaveIfNeeded();
 
 	static CLocalPath GetUnadjustedSettingsDir();
+	CLocalPath GetCacheDirectory();
 
 protected:
 	COptions();
 	virtual ~COptions();
 
 	int Validate(unsigned int nID, int value);
-	wxString Validate(unsigned int nID, wxString const& value);
+	std::wstring Validate(unsigned int nID, std::wstring const& value);
 
 	template<typename T> void ContinueSetOption(unsigned int nID, T const& value);
 	void SetXmlValue(unsigned int nID, int value);
-	void SetXmlValue(unsigned int nID, wxString const& value);
+	void SetXmlValue(unsigned int nID, std::wstring const& value);
 
 	// path is element path below document root, separated by slashes
-	void SetServer(wxString path, const CServer& server);
-	bool GetServer(wxString path, CServer& server);
+	void SetServer(std::wstring path, const CServer& server);
+	bool GetServer(std::wstring path, CServer& server);
 
 	pugi::xml_node CreateSettingsXmlElement();
 
@@ -183,7 +190,7 @@ protected:
 	DECLARE_EVENT_TABLE()
 	void OnTimer(wxTimerEvent& event);
 
-	fz::mutex m_sync_;
+	fz::mutex m_sync_{false};
 
 	changed_options_t changedOptions_;
 };
