@@ -19,17 +19,21 @@
 #include "sizeformatting.h"
 #include "timeformatting.h"
 
+#include "uri.h"
+
 #include <wx/clipbrd.h>
 #include <wx/dcclient.h>
 
 #include <algorithm>
 
+#include <libfilezilla/file.hpp>
+
 #ifdef __WXMSW__
 #include "shellapi.h"
 #include "commctrl.h"
 #endif
 
-class CRemoteListViewDropTarget : public CScrollableDropTarget<wxListCtrlEx>
+class CRemoteListViewDropTarget final : public CScrollableDropTarget<wxListCtrlEx>
 {
 public:
 	CRemoteListViewDropTarget(CRemoteListView* pRemoteListView)
@@ -113,30 +117,32 @@ public:
 		}
 
 		// Find drop directory (if it exists)
-		wxString subdir;
+		std::wstring subdir;
 		int flags = 0;
 		int hit = m_pRemoteListView->HitTest(wxPoint(x, y), flags, 0);
-		if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM))
-		{
+		if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM)) {
 			int index = m_pRemoteListView->GetItemIndex(hit);
 			if (index != -1 && m_pRemoteListView->m_fileData[index].comparison_flags != CComparableListing::fill)
 			{
-				if (index == (int)m_pRemoteListView->m_pDirectoryListing->GetCount())
+				if (index == (int)m_pRemoteListView->m_pDirectoryListing->GetCount()) {
 					subdir = _T("..");
-				else if ((*m_pRemoteListView->m_pDirectoryListing)[index].is_dir())
+				}
+				else if ((*m_pRemoteListView->m_pDirectoryListing)[index].is_dir()) {
 					subdir = (*m_pRemoteListView->m_pDirectoryListing)[index].name;
+				}
 			}
 		}
 
 		// Get target path
 		CServerPath target = m_pRemoteListView->m_pDirectoryListing->path;
-		if (subdir == _T(".."))
-		{
-			if (target.HasParent())
+		if (subdir == _T("..")) {
+			if (target.HasParent()) {
 				target = target.GetParent();
+			}
 		}
-		else if (!subdir.empty())
+		else if (!subdir.empty()) {
 			target.AddSegment(subdir);
+		}
 
 		// Make sure target path is valid
 		if (target == m_pRemoteDataObject->GetServerPath())
@@ -290,7 +296,7 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-class CInfoText : public wxWindow
+class CInfoText final : public wxWindow
 {
 public:
 	CInfoText(wxWindow* parent)
@@ -417,7 +423,7 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState& state, CQueueView* p
 CRemoteListView::~CRemoteListView()
 {
 	wxString str = wxString::Format(_T("%d %d"), m_sortDirection, m_sortColumn);
-	COptions::Get()->SetOption(OPTION_REMOTEFILELIST_SORTORDER, str);
+	COptions::Get()->SetOption(OPTION_REMOTEFILELIST_SORTORDER, str.ToStdWstring());
 }
 
 // See comment to OnGetItemText
@@ -829,7 +835,9 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 				resetDropTarget = true;
 
 		if (resetDropTarget) {
-			SetItemState(m_dropTarget, 0, wxLIST_STATE_DROPHILITED);
+			if (m_dropTarget < GetItemCount()) {
+				SetItemState(m_dropTarget, 0, wxLIST_STATE_DROPHILITED);
+			}
 			m_dropTarget = -1;
 		}
 	}
@@ -863,25 +871,26 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 // foo.bar;1
 // foo.bar;2
 // foo.bar;3
-wxString StripVMSRevision(const wxString& name)
+std::wstring StripVMSRevision(std::wstring const& name)
 {
-	int pos = name.Find(';', true);
-	if (pos < 1)
+	size_t pos = name.rfind(';');
+	if (pos == std::wstring::npos || !pos) {
 		return name;
-
-	const int len = name.Len();
-	if (pos == len - 1)
-		return name;
-
-	int p = pos;
-	while (++p < len)
-	{
-		const wxChar& c = name[p];
-		if (c < '0' || c > '9')
-			return name;
 	}
 
-	return name.Left(pos);
+	if (pos == name.size() - 1) {
+		return name;
+	}
+
+	size_t p = pos;
+	while (++p < name.size()) {
+		wchar_t const& c = name[p];
+		if (c < '0' || c > '9') {
+			return name;
+		}
+	}
+
+	return name.substr(0, pos);
 }
 
 
@@ -935,8 +944,8 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 		if (m_fileData[index].comparison_flags == fill)
 			return;
 
-		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		const wxString& name = entry.name;
+		CDirentry const& entry = (*m_pDirectoryListing)[index];
+		std::wstring const& name = entry.name;
 
 		const CServer* pServer = m_state.GetServer();
 		if (!pServer) {
@@ -989,11 +998,11 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 				return;
 			}
 
-			wxString localFile = CQueueView::ReplaceInvalidCharacters(name);
+			std::wstring localFile = CQueueView::ReplaceInvalidCharacters(name);
 			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
 				localFile = StripVMSRevision(localFile);
 			m_pQueue->QueueFile(queue_only, true, name,
-				(name == localFile) ? wxString() : localFile,
+				(name == localFile) ? std::wstring() : localFile,
 				local_path, m_pDirectoryListing->path, *pServer, entry.size);
 			m_pQueue->QueueFile_Finish(true);
 		}
@@ -1028,8 +1037,8 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 			return;
 		}
 
-		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		const wxString& name = entry.name;
+		CDirentry const& entry = (*m_pDirectoryListing)[index];
+		std::wstring const& name = entry.name;
 
 		const CServer* pServer = m_state.GetServer();
 		if (!pServer) {
@@ -1052,7 +1061,7 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 		m_state.ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
 	}
 	else {
-		m_state.ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, L"..");
 	}
 }
 
@@ -1064,8 +1073,9 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 	}
 
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_REMOTEFILELIST"));
-	if (!pMenu)
+	if (!pMenu) {
 		return;
+	}
 
 	bool const idle = m_state.IsRemoteIdle();
 	bool const userIdle = m_state.IsRemoteIdle(true);
@@ -1097,6 +1107,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 		pMenu->Enable(XRCID("ID_GETURL"), false);
+		pMenu->Enable(XRCID("ID_GETURL_PASSWORD"), false);
 		pMenu->Enable(XRCID("ID_CONTEXT_REFRESH"), false);
 		pMenu->Enable(XRCID("ID_NEW_FILE"), false);
 	}
@@ -1109,6 +1120,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 		pMenu->Enable(XRCID("ID_GETURL"), false);
+		pMenu->Enable(XRCID("ID_GETURL_PASSWORD"), false);
 	}
 	else {
 		if ((GetItemCount() && GetItemState(0, wxLIST_STATE_SELECTED))) {
@@ -1116,6 +1128,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 			pMenu->Enable(XRCID("ID_CHMOD"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
 			pMenu->Enable(XRCID("ID_GETURL"), false);
+			pMenu->Enable(XRCID("ID_GETURL_PASSWORD"), false);
 		}
 
 		int count = 0;
@@ -1130,15 +1143,17 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 			}
 
 			int index = GetItemIndex(item);
-			if (index == -1)
-				continue;
-			count++;
-			if (m_fileData[index].comparison_flags == fill) {
-				fillCount++;
+			if (index == -1) {
 				continue;
 			}
-			if ((*m_pDirectoryListing)[index].is_dir())
+			++count;
+			if (m_fileData[index].comparison_flags == fill) {
+				++fillCount;
+				continue;
+			}
+			if ((*m_pDirectoryListing)[index].is_dir()) {
 				selectedDir = true;
+			}
 		}
 		if (!count || fillCount == count) {
 			pMenu->Delete(XRCID("ID_ENTER"));
@@ -1149,15 +1164,19 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 			pMenu->Enable(XRCID("ID_CHMOD"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
 			pMenu->Enable(XRCID("ID_GETURL"), false);
+			pMenu->Enable(XRCID("ID_GETURL_PASSWORD"), false);
 		}
 		else {
-			if (selectedDir)
+			if (selectedDir) {
 				pMenu->Enable(XRCID("ID_EDIT"), false);
-			else
+			}
+			else {
 				pMenu->Delete(XRCID("ID_ENTER"));
+			}
 			if (count > 1) {
-				if (selectedDir)
+				if (selectedDir) {
 					pMenu->Delete(XRCID("ID_ENTER"));
+				}
 				pMenu->Enable(XRCID("ID_RENAME"), false);
 			}
 
@@ -1240,8 +1259,8 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 		if (m_fileData[index].comparison_flags == fill)
 			continue;
 
-		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		const wxString& name = entry.name;
+		CDirentry const& entry = (*m_pDirectoryListing)[index];
+		std::wstring const& name = entry.name;
 
 		if (entry.is_dir()) {
 			if (!idle)
@@ -1254,11 +1273,12 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 			}
 		}
 		else {
-			wxString localFile = CQueueView::ReplaceInvalidCharacters(name);
-			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
+			std::wstring localFile = CQueueView::ReplaceInvalidCharacters(name);
+			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION)) {
 				localFile = StripVMSRevision(localFile);
+			}
 			m_pQueue->QueueFile(queue_only, true,
-				name, (name == localFile) ? wxString() : localFile,
+				name, (name == localFile) ? std::wstring() : localFile,
 				local_parent, m_pDirectoryListing->path, *pServer, entry.size);
 			added = true;
 		}
@@ -1284,7 +1304,7 @@ void CRemoteListView::OnMenuMkdirChgDir(wxCommandEvent&)
 {
 	CServerPath newdir = MenuMkdir();
 	if (!newdir.empty()) {
-		m_state.ChangeRemoteDir(newdir, wxString(), 0, true);
+		m_state.ChangeRemoteDir(newdir, std::wstring(), 0, true);
 	}
 }
 
@@ -1306,7 +1326,7 @@ CServerPath CRemoteListView::MenuMkdir()
 	// Append a long segment which does (most likely) not exist in the path and
 	// replace it with "New directory" later. This way we get the exact position of
 	// "New directory" and can preselect it in the dialog.
-	wxString tmpName = _T("25CF809E56B343b5A12D1F0466E3B37A49A9087FDCF8412AA9AF8D1E849D01CF");
+	std::wstring tmpName = _T("25CF809E56B343b5A12D1F0466E3B37A49A9087FDCF8412AA9AF8D1E849D01CF");
 	if (path.AddSegment(tmpName)) {
 		wxString pathName = path.GetPath();
 		int pos = pathName.Find(tmpName);
@@ -1330,7 +1350,7 @@ CServerPath CRemoteListView::MenuMkdir()
 	}
 
 	path = m_pDirectoryListing->path;
-	if (!path.ChangePath(dlg.GetValue())) {
+	if (!path.ChangePath(dlg.GetValue().ToStdWstring())) {
 		wxBell();
 		return CServerPath();
 	}
@@ -1447,7 +1467,7 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 		if (entry.is_dir() && (follow_symlink || !entry.is_link())) {
 			CServerPath remotePath = m_pDirectoryListing->path;
 			if (remotePath.AddSegment(name)) {
-				root.add_dir_to_visit(m_pDirectoryListing->path, name, CLocalPath(), true);;
+				root.add_dir_to_visit(m_pDirectoryListing->path, name, CLocalPath(), true);
 			}
 		}
 		else {
@@ -1461,7 +1481,7 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 
 	if (!root.empty()) {
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
-		
+
 		CFilterManager filter;
 		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete,
 													 filter.GetActiveFilters(), m_pDirectoryListing->path);
@@ -1566,44 +1586,41 @@ bool CRemoteListView::OnBeginRename(const wxListEvent& event)
 
 bool CRemoteListView::OnAcceptRename(const wxListEvent& event)
 {
-	if (!m_state.IsRemoteIdle())
-	{
+	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return false;
 	}
 
-	if (!m_pDirectoryListing)
-	{
+	if (!m_pDirectoryListing) {
 		wxBell();
 		return false;
 	}
 
 	int item = event.GetIndex();
-	if (!item)
+	if (!item) {
 		return false;
+	}
 
 	int index = GetItemIndex(item);
-	if (index == -1 || m_fileData[index].comparison_flags == fill)
-	{
+	if (index == -1 || m_fileData[index].comparison_flags == fill) {
 		wxBell();
 		return false;
 	}
 
 	const CDirentry& entry = (*m_pDirectoryListing)[index];
 
-	wxString newFile = event.GetLabel();
+	std::wstring newFile = event.GetLabel().ToStdWstring();
 
 	CServerPath newPath = m_pDirectoryListing->path;
-	if (!newPath.ChangePath(newFile, true))
-	{
+	if (!newPath.ChangePath(newFile, true)) {
 		wxMessageBoxEx(_("Filename invalid"), _("Cannot rename file"), wxICON_EXCLAMATION);
 		return false;
 	}
 
-	if (newPath == m_pDirectoryListing->path)
-	{
-		if (entry.name == newFile)
+	if (newPath == m_pDirectoryListing->path) {
+		if (entry.name == newFile) {
 			return false;
+		}
 
 		// Check if target file already exists
 		for (unsigned int i = 0; i < m_pDirectoryListing->GetCount(); i++)
@@ -1731,7 +1748,7 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 		{
 			char newPermissions[9]{};
 			bool res = pChmodDlg->ConvertPermissions(*entry.permissions, newPermissions);
-			wxString newPerms = pChmodDlg->GetPermissions(res ? newPermissions : 0, entry.is_dir());
+			std::wstring const newPerms = pChmodDlg->GetPermissions(res ? newPermissions : 0, entry.is_dir()).ToStdWstring();
 
 			m_state.m_pCommandQueue->ProcessCommand(new CChmodCommand(m_pDirectoryListing->path, entry.name, newPerms));
 		}
@@ -2008,7 +2025,7 @@ void CRemoteListView::OnStateChange(t_statechange_notifications notification, co
 		SetDirectoryListing(m_state.GetRemoteDir());
 	else if (notification == STATECHANGE_REMOTE_LINKNOTDIR) {
 		wxASSERT(data2);
-		LinkIsNotDir(*(CServerPath*)data2, data);
+		LinkIsNotDir(*(CServerPath*)data2, data.ToStdWstring());
 	}
 	else if (notification == STATECHANGE_SERVER) {
 		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
@@ -2188,7 +2205,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 				}
 			}
 
-			CLocalPath target(ext->GetTarget());
+			CLocalPath target(ext->GetTarget().ToStdWstring());
 			if (target.empty()) {
 				ext.reset(); // Release extension before the modal message box
 				wxMessageBoxEx(_("Could not determine the target of the Drag&Drop operation.\nEither the shell extension is not installed properly or you didn't drop the files into an Explorer window."));
@@ -2476,14 +2493,14 @@ int64_t CRemoteListView::ItemGetSize(int index) const
 	return (*m_pDirectoryListing)[index].size;
 }
 
-void CRemoteListView::LinkIsNotDir(const CServerPath& path, const wxString& link)
+void CRemoteListView::LinkIsNotDir(CServerPath const& path, std::wstring const& link)
 {
 	if (m_pLinkResolveState && m_pLinkResolveState->remote_path == path && m_pLinkResolveState->link == link) {
-		wxString localFile = CQueueView::ReplaceInvalidCharacters(link);
+		std::wstring localFile = CQueueView::ReplaceInvalidCharacters(link);
 		if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
 			localFile = StripVMSRevision(localFile);
 		m_pQueue->QueueFile(false, true,
-			link, (link != localFile) ? localFile : wxString(),
+			link, (link != localFile) ? localFile : std::wstring(),
 			m_pLinkResolveState->local_path, m_pLinkResolveState->remote_path, m_pLinkResolveState->server, -1);
 		m_pQueue->QueueFile_Finish(true);
 	}
@@ -2533,10 +2550,10 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 
 	wxString const server = pServer->Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
-	auto getUrl = [](wxString const& serverPart, CServerPath const& path, wxString const& name) {
+	auto getUrl = [](wxString const& serverPart, CServerPath const& path, std::wstring const& name) {
 		wxString url = serverPart;
 
-		auto const pathPart = url_encode(path.FormatFilename(name, false).ToStdWstring(), true);
+		auto const pathPart = fz::percent_encode_w(path.FormatFilename(name, false), true);
 		if (!pathPart.empty() && pathPart[0] != '/') {
 			url += '/';
 		}
@@ -2622,7 +2639,7 @@ void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 		return;
 	}
 
-	wxString newFileName = dlg.GetValue();
+	std::wstring newFileName = dlg.GetValue().ToStdWstring();
 
 	// Check if target file already exists
 	for (unsigned int i = 0; i < m_pDirectoryListing->GetCount(); ++i) {
@@ -2634,17 +2651,16 @@ void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 
 	CEditHandler* edithandler = CEditHandler::Get(); // Used to get the temporary folder
 
-	wxString emptyfile_name = _T("empty_file_yq744zm");
-	wxString emptyfile = edithandler->GetLocalDirectory() + emptyfile_name;
+	std::wstring const emptyfile_name = L"empty_file_yq744zm";
+	std::wstring emptyfile = edithandler->GetLocalDirectory() + emptyfile_name;
 
 	// Create the empty temporary file
 	{
-		wxFile file;
-		wxLogNull log;
-		file.Create(emptyfile);
+		fz::file f(fz::to_native(emptyfile), fz::file::writing, fz::file::existing);
+		(void)f;
 	}
 
-	const CServer* pServer = m_state.GetServer();
+	CServer const* pServer = m_state.GetServer();
 	if (!pServer) {
 		wxBell();
 		return;
