@@ -5,16 +5,17 @@
 #include "listctrlex.h"
 #include "filelistctrl.h"
 
-class CQueueView;
 class CChmodDialog;
 class CInfoText;
+class CQueueView;
 class CRemoteListViewDropTarget;
+class CWindowTinter;
 
-class CRemoteListView : public CFileListCtrl<CGenericFileData>, CStateEventHandler
+class CRemoteListView final : public CFileListCtrl<CGenericFileData>, CStateEventHandler
 {
 	friend class CRemoteListViewDropTarget;
 public:
-	CRemoteListView(wxWindow* pParent, CState* pState, CQueueView* pQueue);
+	CRemoteListView(wxWindow* pParent, CState& state, CQueueView* pQueue);
 	virtual ~CRemoteListView();
 
 	virtual bool CanStartComparison();
@@ -23,7 +24,7 @@ public:
 	virtual void FinishComparison();
 	virtual void OnExitComparisonMode();
 
-	void LinkIsNotDir(const CServerPath& path, const wxString& link);
+	void LinkIsNotDir(CServerPath const& path, std::wstring const& link);
 protected:
 	virtual wxString GetItemText(int item, unsigned int column);
 
@@ -51,7 +52,7 @@ protected:
 
 	virtual CSortComparisonObject GetSortComparisonObject();
 
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
 	void ApplyCurrentFilter();
 	void SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
 	bool UpdateDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
@@ -67,7 +68,7 @@ protected:
 	std::shared_ptr<CDirectoryListing> m_pDirectoryListing;
 
 	// Caller is responsible to check selection is valid!
-	void TransferSelectedFiles(const CLocalPath& local_parent, bool queueOnly);
+	void TransferSelectedFiles(const CLocalPath& local_parent, bool queue_only);
 
 	// Cache icon for directories, no need to calculate it multiple times
 	int m_dirIcon;
@@ -99,6 +100,8 @@ protected:
 
 	CServerPath MenuMkdir();
 
+	std::unique_ptr<CWindowTinter> m_windowTinter;
+
 	DECLARE_EVENT_TABLE()
 	void OnItemActivated(wxListEvent &event);
 	void OnContextMenu(wxContextMenuEvent& event);
@@ -113,7 +116,7 @@ protected:
 	void OnBeginDrag(wxListEvent&);
 	void OnMenuEdit(wxCommandEvent&);
 	void OnMenuEnter(wxCommandEvent&);
-	void OnMenuGeturl(wxCommandEvent&);
+	void OnMenuGeturl(wxCommandEvent& event);
 	void OnMenuRefresh(wxCommandEvent&);
 	void OnMenuNewfile(wxCommandEvent& event);
 };
