@@ -8,20 +8,21 @@
 #include "treectrlex.h"
 
 class CQueueView;
-class CRemoteTreeView : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler, COptionChangeEventHandler
+class CWindowTinter;
+class CRemoteTreeView final : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler, COptionChangeEventHandler
 {
 	DECLARE_CLASS(CRemoteTreeView)
 
 	friend class CRemoteTreeViewDropTarget;
 
 public:
-	CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState, CQueueView* pQueue);
+	CRemoteTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView* pQueue);
 	virtual ~CRemoteTreeView();
 
 protected:
 	wxTreeItemId MakeParent(CServerPath path, bool select);
 	void SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool modified);
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*);
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString&, const void*);
 
 	void DisplayItem(wxTreeItemId parent, const CDirectoryListing& listing);
 	void RefreshItem(wxTreeItemId parent, const CDirectoryListing& listing, bool will_select_parent);
@@ -56,6 +57,10 @@ protected:
 
 	virtual void OnOptionsChanged(changed_options_t const& options);
 
+	std::unique_ptr<CWindowTinter> m_windowTinter;
+
+	wxTreeItemId m_contextMenuItem;
+
 	DECLARE_EVENT_TABLE()
 	void OnItemExpanding(wxTreeEvent& event);
 	void OnSelectionChanged(wxTreeEvent& event);
@@ -71,9 +76,7 @@ protected:
 	void OnMkdir(wxCommandEvent&);
 	void OnMenuMkdirChgDir(wxCommandEvent&);
 	void OnChar(wxKeyEvent& event);
-	void OnMenuGeturl(wxCommandEvent&);
-
-	wxTreeItemId m_contextMenuItem;
+	void OnMenuGeturl(wxCommandEvent& event);
 };
 
 #endif
