@@ -7,15 +7,17 @@
 
 #include <wx/timer.h>
 
+#include "option_change_event_handler.h"
+
 class CFastTextCtrl;
-class CStatusView final : public wxNavigationEnabled<wxWindow>
+class CStatusView final : public wxNavigationEnabled<wxWindow>, private COptionChangeEventHandler
 {
 public:
 	CStatusView(wxWindow* parent, wxWindowID id);
 	virtual ~CStatusView();
 
 	void AddToLog(CLogmsgNotification const& pNotification);
-	void AddToLog(MessageType messagetype, const wxString& message, const wxDateTime& time);
+	void AddToLog(MessageType messagetype, std::wstring const& message, fz::datetime const& time);
 
 	void InitDefAttr();
 
@@ -23,12 +25,13 @@ public:
 
 	virtual bool Show(bool show = true);
 
-protected:
+private:
 
 	int m_nLineCount{};
-	wxString m_Content;
 	CFastTextCtrl *m_pTextCtrl{};
 
+	void OnOptionsChanged(changed_options_t const& options);
+
 	DECLARE_EVENT_TABLE()
 	void OnSize(wxSizeEvent &);
 	void OnContextMenu(wxContextMenuEvent&);
@@ -41,8 +44,8 @@ protected:
 
 	struct t_attributeCache
 	{
-		wxString prefix;
-		int len;
+		std::wstring prefix;
+		size_t len;
 		wxTextAttr attr;
 #ifdef __WXMSW__
 		CHARFORMAT2 cf;
@@ -58,14 +61,17 @@ protected:
 	struct t_line
 	{
 		MessageType messagetype;
-		wxString message;
-		wxDateTime time;
+		std::wstring message;
+		fz::datetime time;
 	};
 	std::list<t_line> m_hiddenLines;
 
 	bool m_showTimestamps{};
-	wxDateTime m_lastTime;
-	wxString m_lastTimeString;
+	fz::datetime m_lastTime;
+	std::wstring m_lastTimeString;
+
+	// Re-using the same string to format the message in avoids uneeded allocations
+	std::wstring m_formattedMessage;
 };
 
 #endif
