@@ -1,17 +1,19 @@
 #ifndef __ASYNCREQUESTQUEUE_H__
 #define __ASYNCREQUESTQUEUE_H__
 
+#include <wx/timer.h>
+
 class CMainFrame;
 class CQueueView;
 class CVerifyCertDialog;
 
-class CAsyncRequestQueue : public wxEvtHandler
+class CAsyncRequestQueue final : public wxEvtHandler
 {
 public:
 	CAsyncRequestQueue(CMainFrame *pMainFrame);
 	~CAsyncRequestQueue();
 
-	bool AddRequest(CFileZillaEngine *pEngine, CAsyncRequestNotification *pNotification);
+	bool AddRequest(CFileZillaEngine *pEngine, std::unique_ptr<CAsyncRequestNotification> && pNotification);
 	void ClearPending(const CFileZillaEngine* pEngine);
 	void RecheckDefaults();
 
@@ -30,15 +32,23 @@ protected:
 	CVerifyCertDialog *m_pVerifyCertDlg;
 
 	bool ProcessNextRequest();
-	bool ProcessDefaults(CFileZillaEngine *pEngine, CAsyncRequestNotification *pNotification);
+	bool ProcessDefaults(CFileZillaEngine *pEngine, std::unique_ptr<CAsyncRequestNotification> & pNotification);
 
 	struct t_queueEntry
 	{
+		t_queueEntry(CFileZillaEngine *e, std::unique_ptr<CAsyncRequestNotification>&& n)
+			: pEngine(e)
+			, pNotification(std::move(n))
+		{
+		}
+
 		CFileZillaEngine *pEngine;
-		CAsyncRequestNotification *pNotification;
+		std::unique_ptr<CAsyncRequestNotification> pNotification;
 	};
 	std::list<t_queueEntry> m_requestList;
 
+	bool ProcessFileExistsNotification(t_queueEntry &entry);
+
 	DECLARE_EVENT_TABLE()
 	void OnProcessQueue(wxCommandEvent &event);
 	void OnTimer(wxTimerEvent& event);
