@@ -2,6 +2,7 @@
 #include <wx/aui/aui.h>
 #include "aui_notebook_ex.h"
 #include <wx/dcmirror.h>
+#include <wx/graphics.h>
 
 #include <memory>
 
@@ -59,6 +60,58 @@ public:
 		return size;
 	}
 
+	virtual void DrawTab(wxDC &dc, wxWindow *wnd, const wxAuiNotebookPage &page, const wxRect &rect, int close_button_state, wxRect *out_tab_rect, wxRect *out_button_rect, int *x_extent)
+	{
+		wxColour const tint = m_pNotebook->GetTabColour(page.window);
+
+		if (tint.IsOk()) {
+
+#if !defined(__WXGTK__) || !defined(wxHAS_NATIVE_TABART)
+			wxColour const baseOrig = m_baseColour;
+			wxColour const activeOrig = m_activeColour;
+
+
+			wxColour const base(
+				wxColour::AlphaBlend(tint.Red(),   baseOrig.Red(),   tint.Alpha() / 255.0f),
+				wxColour::AlphaBlend(tint.Green(), baseOrig.Green(), tint.Alpha() / 255.0f),
+				wxColour::AlphaBlend(tint.Blue(),  baseOrig.Blue(),  tint.Alpha() / 255.0f));
+
+			wxColour const active(
+				wxColour::AlphaBlend(tint.Red(),   activeOrig.Red(),   tint.Alpha() / 255.0f),
+				wxColour::AlphaBlend(tint.Green(), activeOrig.Green(), tint.Alpha() / 255.0f),
+				wxColour::AlphaBlend(tint.Blue(),  activeOrig.Blue(),  tint.Alpha() / 255.0f));
+
+			m_baseColour = base;
+			m_activeColour = active;
+
+			wxAuiDefaultTabArt::DrawTab(dc, wnd, page, rect, close_button_state, out_tab_rect, out_button_rect, x_extent);
+
+			m_baseColour = baseOrig;
+			m_activeColour = baseOrig;
+#else
+			wxRect tab_rect;
+			if (!out_tab_rect) {
+				out_tab_rect = &tab_rect;
+			}
+
+			wxAuiDefaultTabArt::DrawTab(dc, wnd, page, rect, close_button_state, out_tab_rect, out_button_rect, x_extent);
+
+			wxMemoryDC *mdc = dynamic_cast<wxMemoryDC*>(&dc);
+			if (mdc) {
+				wxGraphicsContext *gc = wxGraphicsContext::Create(*mdc);
+				if (gc) {
+					gc->SetBrush(wxBrush(tint));
+					gc->DrawRectangle(out_tab_rect->x, out_tab_rect->y, out_tab_rect->width, out_tab_rect->height);
+					delete gc;
+				}
+			}
+#endif
+		}
+		else {
+			wxAuiDefaultTabArt::DrawTab(dc, wnd, page, rect, close_button_state, out_tab_rect, out_button_rect, x_extent);
+		}
+	}
+
 protected:
 	wxAuiNotebookEx* m_pNotebook;
 
@@ -67,6 +120,7 @@ protected:
 
 BEGIN_EVENT_TABLE(wxAuiNotebookEx, wxAuiNotebook)
 EVT_AUINOTEBOOK_PAGE_CHANGED(wxID_ANY, wxAuiNotebookEx::OnPageChanged)
+EVT_AUINOTEBOOK_DRAG_MOTION(wxID_ANY, wxAuiNotebookEx::OnTabDragMotion)
 END_EVENT_TABLE()
 
 wxAuiNotebookEx::wxAuiNotebookEx()
@@ -77,6 +131,16 @@ wxAuiNotebookEx::~wxAuiNotebookEx()
 {
 }
 
+void wxAuiNotebookEx::OnTabDragMotion(wxAuiNotebookEvent& evt)
+{
+	wxAuiNotebook::OnTabDragMotion(evt);
+
+	int active = m_tabs.GetActivePage();
+	if (active != wxNOT_FOUND) {
+		m_curPage = active;
+	}
+}
+
 void wxAuiNotebookEx::RemoveExtraBorders()
 {
 	wxAuiPaneInfoArray& panes = m_mgr.GetAllPanes();
@@ -114,6 +178,14 @@ bool wxAuiNotebookEx::SetPageText(size_t page_idx, const wxString& text)
 	return true;
 }
 
+void wxAuiNotebookEx::SetTabColour(size_t page, wxColour const& c)
+{
+	wxWindow* w = GetPage(page);
+	if (w) {
+		m_colourMap[w] = c;
+	}
+}
+
 void wxAuiNotebookEx::Highlight(size_t page, bool highlight /*=true*/)
 {
 	if (GetSelection() == (int)page)
@@ -143,7 +215,7 @@ bool wxAuiNotebookEx::Highlighted(size_t page) const
 	return m_highlighted[page];
 }
 
-void wxAuiNotebookEx::OnPageChanged(wxAuiNotebookEvent& event)
+void wxAuiNotebookEx::OnPageChanged(wxAuiNotebookEvent&)
 {
 	size_t page = (size_t)GetSelection();
 	if (page >= m_highlighted.size())
@@ -185,3 +257,18 @@ bool wxAuiNotebookEx::AddPage(wxWindow *page, const wxString &text, bool select,
 
 	return res;
 }
+
+bool wxAuiNotebookEx::RemovePage(size_t page)
+{
+	m_colourMap.erase(GetPage(page));
+	return wxAuiNotebook::RemovePage(page);
+}
+
+wxColour wxAuiNotebookEx::GetTabColour(wxWindow* page)
+{
+	auto const it = m_colourMap.find(page);
+	if (it != m_colourMap.end()) {
+		return it->second;
+	}
+	return wxColour();
+}
