@@ -1,14 +1,15 @@
 #include <filezilla.h>
 #include "clearprivatedata.h"
-#include "Mainfrm.h"
-#include "recentserverlist.h"
-#include "quickconnectbar.h"
-#include "state.h"
 #include "commandqueue.h"
-#include "Options.h"
 #include "ipcmutex.h"
+#include "local_recursive_operation.h"
+#include "Mainfrm.h"
+#include "Options.h"
 #include "queue.h"
-#include "recursive_operation.h"
+#include "quickconnectbar.h"
+#include "recentserverlist.h"
+#include "remote_recursive_operation.h"
+#include "state.h"
 
 BEGIN_EVENT_TABLE(CClearPrivateDataDialog, wxDialogEx)
 EVT_TIMER(wxID_ANY, CClearPrivateDataDialog::OnTimer)
@@ -61,28 +62,24 @@ void CClearPrivateDataDialog::Run()
 
 	pCheck = XRCCTRL(*this, "ID_CLEARRECONNECT", wxCheckBox);
 
-	if (pCheck->GetValue())
-	{
+	if (pCheck->GetValue()) {
 		bool asked = false;
 
 		const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
 
-		for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter)
-		{
+		for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter) {
 			CState* pState = *iter;
-			if (pState->IsRemoteConnected() || !pState->IsRemoteIdle())
-			{
-				if (!asked)
-				{
+			if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
+				if (!asked) {
 					int res = wxMessageBoxEx(_("Reconnect information cannot be cleared while connected to a server.\nIf you continue, your connection will be disconnected."), _("Clear private data"), wxOK | wxCANCEL);
 					if (res != wxOK)
 						return;
 					asked = true;
 				}
 
-				pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
-				if (!pState->m_pCommandQueue->Cancel())
-				{
+				pState->GetLocalRecursiveOperation()->StopRecursiveOperation();
+				pState->GetRemoteRecursiveOperation()->StopRecursiveOperation();
+				if (!pState->m_pCommandQueue->Cancel()) {
 					m_timer.SetOwner(this);
 					m_timer.Start(250, true);
 				}
@@ -95,11 +92,9 @@ void CClearPrivateDataDialog::Run()
 		ClearReconnect();
 	}
 
-	if (pSitemanagerCheck->GetValue())
-	{
+	if (pSitemanagerCheck->GetValue()) {
 		CInterProcessMutex sitemanagerMutex(MUTEX_SITEMANAGERGLOBAL, false);
-		while (sitemanagerMutex.TryLock() == 0)
-		{
+		while (sitemanagerMutex.TryLock() == 0) {
 			int res = wxMessageBoxEx(_("The Site Manager is opened in another instance of FileZilla 3.\nPlease close it or the data cannot be deleted."), _("Clear private data"), wxOK | wxCANCEL);
 			if (res != wxYES)
 				return;
@@ -108,8 +103,7 @@ void CClearPrivateDataDialog::Run()
 		RemoveXmlFile(_T("sitemanager"));
 	}
 
-	if (pQueueCheck->GetValue())
-	{
+	if (pQueueCheck->GetValue()) {
 		m_pMainFrame->GetQueue()->SetActive(false);
 		m_pMainFrame->GetQueue()->RemoveAll();
 
@@ -158,11 +152,11 @@ bool CClearPrivateDataDialog::ClearReconnect()
 	COptions::Get()->SetOption(OPTION_LASTSERVERPATH, _T(""));
 
 	const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
-	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter)
-	{
+	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter) {
 		CState* pState = *iter;
-
-		pState->SetLastServer(CServer(), CServerPath());
+		if (pState) {
+			pState->SetLastSite(Site(), CServerPath());
+		}
 	}
 
 	return true;
