@@ -1,7 +1,7 @@
 #ifndef __CONDITIONALDIALOG_H__
 #define __CONDITIONALDIALOG_H__
 
-class CConditionalDialog : public wxDialog
+class CConditionalDialog final : public wxDialog
 {
 public:
 	enum Modes
@@ -23,18 +23,18 @@ public:
 		many_selected_for_edit
 	};
 
-	CConditionalDialog(wxWindow* parent, enum DialogType type, enum Modes mode, bool checked = false);
+	CConditionalDialog(wxWindow* parent, DialogType type, Modes mode, bool checked = false);
 
 	void AddText(const wxString &text);
 
 	bool Run();
 
 protected:
-	enum DialogType m_type;
+	DialogType m_type;
 
 	wxSizer* m_pTextSizer;
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnButton(wxCommandEvent& event);
 };
 
