@@ -46,13 +46,14 @@ public:
 	{
 		all,
 		any,
-		none
+		none,
+		not_all
 	};
 
 	wxString name;
 
-	bool filterFiles{};
-	bool filterDirs{};
+	bool filterFiles{true};
+	bool filterDirs{true};
 	t_matchType matchType{all};
 
 	// Filenames on Windows ignore case
@@ -100,12 +101,17 @@ public:
 	static bool CompileRegexes(std::vector<CFilter>& filters);
 	static bool CompileRegexes(CFilter& filter);
 
+	static void Import(pugi::xml_node& element);
 	static bool LoadFilter(pugi::xml_node& element, CFilter& filter);
+	static void SaveFilter(pugi::xml_node& element, const CFilter& filter);
 
 protected:
 	static bool CompileRegexes();
 
 	static void LoadFilters();
+	static void LoadFilters(pugi::xml_node& element);
+	static void SaveFilters();
+
 	static bool m_loaded;
 
 	static std::vector<CFilter> m_globalFilters;
@@ -117,7 +123,7 @@ protected:
 };
 
 class CMainFrame;
-class CFilterDialog : public wxDialogEx, public CFilterManager
+class CFilterDialog final : public wxDialogEx, public CFilterManager
 {
 public:
 	CFilterDialog();
@@ -125,12 +131,7 @@ public:
 
 	bool Create(CMainFrame* parent);
 
-	static void SaveFilter(pugi::xml_node& element, const CFilter& filter);
-
 protected:
-
-	void SaveFilters();
-
 	void DisplayFilters();
 
 	void SetCtrlState();
