@@ -130,7 +130,7 @@ void wxListCtrlEx::ScrollTopItem(int item)
 	else if (item >= GetItemCount()) {
 		item = GetItemCount() - 1;
 	}
-	
+
 	const int current = GetTopItem();
 
 	int delta = item - current;
@@ -152,7 +152,7 @@ void wxListCtrlEx::HandlePrefixSearch(wxChar character)
 
 	// Keyboard navigation within items
 	fz::datetime now = fz::datetime::now();
-	if (m_prefixSearch_lastKeyPress.empty()) {
+	if (!m_prefixSearch_lastKeyPress.empty()) {
 		fz::duration span = now - m_prefixSearch_lastKeyPress;
 		if (span.get_seconds() >= 1) {
 			m_prefixSearch_prefix = _T("");
@@ -495,32 +495,32 @@ void wxListCtrlEx::LoadColumnSettings(int widthsOptionId, int visibilityOptionId
 
 void wxListCtrlEx::SaveColumnSettings(int widthsOptionId, int visibilityOptionId, int sortOptionId)
 {
-	if (widthsOptionId != -1)
+	if (widthsOptionId != -1) {
 		SaveColumnWidths(widthsOptionId);
-
-	if (visibilityOptionId != -1)
-	{
-		wxString visibleColumns;
-		for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-		{
-			if (m_columnInfo[i].shown)
-				visibleColumns += _T("1");
-			else
-				visibleColumns += _T("0");
-		}
-		COptions::Get()->SetOption(visibilityOptionId, visibleColumns);
 	}
 
-	if (sortOptionId != -1)
-	{
+	if (visibilityOptionId != -1) {
+		wxString visibleColumns;
+		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
+			if (m_columnInfo[i].shown) {
+				visibleColumns += _T("1");
+			}
+			else {
+				visibleColumns += _T("0");
+			}
+		}
+		COptions::Get()->SetOption(visibilityOptionId, visibleColumns.ToStdWstring());
+	}
+
+	if (sortOptionId != -1) {
 		wxString order;
-		for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-		{
-			if (i)
+		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
+			if (i) {
 				order += _T(",");
+			}
 			order += wxString::Format(_T("%d"), m_columnInfo[i].order);
 		}
-		COptions::Get()->SetOption(sortOptionId, order);
+		COptions::Get()->SetOption(sortOptionId, order.ToStdWstring());
 	}
 }
 
@@ -577,7 +577,7 @@ void wxListCtrlEx::SaveColumnWidths(unsigned int optionId)
 	}
 	widths.RemoveLast();
 
-	COptions::Get()->SetOption(optionId, widths);
+	COptions::Get()->SetOption(optionId, widths.ToStdWstring());
 }
 
 
@@ -657,10 +657,8 @@ void wxListCtrlEx::MoveColumn(unsigned int col, unsigned int before)
 void wxListCtrlEx::CreateVisibleColumnMapping()
 {
 	int pos = 0;
-	for (unsigned int j = 0; j < m_columnInfo.size(); j++)
-	{
-		for (unsigned int i = 0; i < m_columnInfo.size(); i++)
-		{
+	for (unsigned int j = 0; j < m_columnInfo.size(); ++j) {
+		for (unsigned int i = 0; i < m_columnInfo.size(); ++i) {
 			const t_columnInfo &column = m_columnInfo[i];
 
 			if (!column.shown)
@@ -815,8 +813,7 @@ int wxListCtrlEx::GetColumnVisibleIndex(int col)
 	if (!m_pVisibleColumnMapping)
 		return -1;
 
-	for (int i = 0; i < GetColumnCount(); i++)
-	{
+	for (int i = 0; i < GetColumnCount(); ++i) {
 		if (m_pVisibleColumnMapping[i] == (unsigned int)col)
 			return i;
 	}
@@ -852,49 +849,7 @@ int wxListCtrlEx::GetHeaderSortIconIndex(int col)
 
 void wxListCtrlEx::InitHeaderSortImageList()
 {
-#ifdef __WXMSW__
-	if (wxPlatformInfo::Get().GetOSMajorVersion() >= 6)
-		return;
-
-	// Initialize imagelist for list header
-	m_pHeaderImageList = new wxImageListEx(8, 8, true, 3);
-
-	wxBitmap bmp;
-
-	bmp.LoadFile(wxGetApp().GetResourceDir().GetPath() + _T("up.png"), wxBITMAP_TYPE_PNG);
-	m_pHeaderImageList->Add(bmp);
-	bmp.LoadFile(wxGetApp().GetResourceDir().GetPath() + _T("down.png"), wxBITMAP_TYPE_PNG);
-	m_pHeaderImageList->Add(bmp);
-
-	HWND hWnd = (HWND)GetHandle();
-	if (!hWnd)
-	{
-		delete m_pHeaderImageList;
-		m_pHeaderImageList = 0;
-		return;
-	}
-
-	HWND header = (HWND)SendMessage(hWnd, LVM_GETHEADER, 0, 0);
-	if (!header)
-	{
-		delete m_pHeaderImageList;
-		m_pHeaderImageList = 0;
-		return;
-	}
-
-	TCHAR buffer[1000] = {0};
-	HDITEM item;
-	item.mask = HDI_TEXT;
-	item.pszText = buffer;
-	item.cchTextMax = 999;
-	SendMessage(header, HDM_GETITEM, 0, (LPARAM)&item);
-
-	SendMessage(header, HDM_SETIMAGELIST, 0, (LPARAM)m_pHeaderImageList->GetHandle());
-
-	m_header_icon_index.up = 0;
-	m_header_icon_index.down = 1;
-#else
-
+#ifndef __WXMSW__
 	wxColour colour = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
 
 	wxString lightness;
@@ -921,44 +876,31 @@ void wxListCtrlEx::SetHeaderSortIconIndex(int col, int icon)
 	HWND hWnd = (HWND)GetHandle();
 	HWND header = (HWND)SendMessage(hWnd, LVM_GETHEADER, 0, 0);
 
-	wxChar buffer[100];
-	HDITEM item = {0};
-	item.mask = HDI_TEXT | HDI_FORMAT;
-	item.pszText = buffer;
-	item.cchTextMax = 99;
+	HDITEM item = {};
+	item.mask = HDI_FORMAT;
 	SendMessage(header, HDM_GETITEM, col, (LPARAM)&item);
-	if (icon != -1)
-	{
-		if (wxPlatformInfo::Get().GetOSMajorVersion() >= 6)
-		{
-			item.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT | HDF_SORTUP | HDF_SORTDOWN);
-			item.iImage = -1;
-			if (icon)
-				item.fmt |= HDF_SORTDOWN;
-			else
-				item.fmt |= HDF_SORTUP;
-		}
+	if (icon != -1) {
+		item.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT | HDF_SORTUP | HDF_SORTDOWN);
+		item.iImage = -1;
+		if (icon)
+			item.fmt |= HDF_SORTDOWN;
 		else
-		{
-			item.fmt &= ~(HDF_SORTUP | HDF_SORTDOWN);
-			item.fmt |= HDF_IMAGE | HDF_BITMAP_ON_RIGHT;
-			item.iImage = icon ? m_header_icon_index.down : m_header_icon_index.up;
-			item.mask |= HDI_IMAGE;
-		}
+			item.fmt |= HDF_SORTUP;
 	}
-	else
-	{
+	else {
 		item.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT | HDF_SORTUP | HDF_SORTDOWN);
 		item.iImage = -1;
 	}
 	SendMessage(header, HDM_SETITEM, col, (LPARAM)&item);
 #else
 	wxListItem item;
-	if (!GetColumn(col, item))
+	if (!GetColumn(col, item)) {
 		return;
+	}
 
-	if( icon != -1 )
+	if (icon != -1) {
 		icon = icon ? m_header_icon_index.down : m_header_icon_index.up;
+	}
 
 	item.SetImage(icon);
 	SetColumn(col, item);
@@ -1129,9 +1071,9 @@ bool wxListCtrlEx::MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result)
 			return false;
 	case HDN_DIVIDERDBLCLICK:
 		{
-			wxListEvent event(wxEVT_LIST_COL_END_DRAG, GetId());
-			event.SetEventObject(this);
-			AddPendingEvent(event);
+			auto event = new wxListEvent(wxEVT_LIST_COL_END_DRAG, GetId());
+			event->SetEventObject(this);
+			QueueEvent(event);
 		}
 		break;
 	}
