@@ -16,8 +16,7 @@ EVT_MENU(wxID_ANY, CMenuBar::OnMenuEvent)
 END_EVENT_TABLE()
 
 CMenuBar::CMenuBar()
-	: CStateEventHandler(0)
-	, m_pMainFrame()
+	: m_pMainFrame()
 {
 }
 
@@ -34,7 +33,7 @@ CMenuBar::~CMenuBar()
 
 CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 {
-	CMenuBar* menubar = wxDynamicCast(wxXmlResource::Get()->LoadMenuBar(_T("ID_MENUBAR")), CMenuBar);
+	CMenuBar* menubar = dynamic_cast<CMenuBar*>(wxXmlResource::Get()->LoadMenuBar(_T("ID_MENUBAR")));
 	if (!menubar)
 		return 0;
 
@@ -48,14 +47,11 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 		wxMenu *helpMenu;
 
 		wxMenuItem* pUpdateItem = menubar->FindItem(XRCID("ID_CHECKFORUPDATES"), &helpMenu);
-		if (pUpdateItem)
-		{
+		if (pUpdateItem) {
 			// Get rid of separator
 			unsigned int count = helpMenu->GetMenuItemCount();
-			for (unsigned int i = 0; i < count - 1; i++)
-			{
-				if (helpMenu->FindItemByPosition(i) == pUpdateItem)
-				{
+			for (unsigned int i = 0; i < count - 1; ++i) {
+				if (helpMenu->FindItemByPosition(i) == pUpdateItem) {
 					helpMenu->Delete(helpMenu->FindItemByPosition(i + 1));
 					break;
 				}
@@ -65,8 +61,7 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 		}
 	}
 
-	if (COptions::Get()->GetOptionVal(OPTION_DEBUG_MENU))
-	{
+	if (COptions::Get()->GetOptionVal(OPTION_DEBUG_MENU)) {
 		wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_DEBUG"));
 		if (pMenu)
 			menubar->Append(pMenu, _("&Debug"));
@@ -122,6 +117,8 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_QUEUEPROCESSING, false);
 	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_CHANGEDCONTEXT, false);
 
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_GLOBALBOOKMARKS, false);
+
 	menubar->RegisterOption(OPTION_ASCIIBINARY);
 	menubar->RegisterOption(OPTION_PRESERVE_TIMESTAMPS);
 
@@ -143,48 +140,43 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 void CMenuBar::UpdateBookmarkMenu()
 {
 	wxMenu* pMenu;
-	if (!FindItem(XRCID("ID_BOOKMARK_ADD"), &pMenu))
+	if (!FindItem(XRCID("ID_BOOKMARK_ADD"), &pMenu)) {
 		return;
+	}
 
 	// Delete old bookmarks
-	for (std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_global.begin(); iter != m_bookmark_menu_id_map_global.end(); ++iter)
-	{
+	for (std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_global.begin(); iter != m_bookmark_menu_id_map_global.end(); ++iter) {
 		pMenu->Delete(iter->first);
 	}
 	m_bookmark_menu_id_map_global.clear();
 
-	for (std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_site.begin(); iter != m_bookmark_menu_id_map_site.end(); ++iter)
-	{
+	for (std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_site.begin(); iter != m_bookmark_menu_id_map_site.end(); ++iter) {
 		pMenu->Delete(iter->first);
 	}
 	m_bookmark_menu_id_map_site.clear();
 
 	// Delete the separators
-	while (pMenu->GetMenuItemCount() > 2)
-	{
+	while (pMenu->GetMenuItemCount() > 2) {
 		wxMenuItem* pSeparator = pMenu->FindItemByPosition(2);
-		if (pSeparator)
+		if (pSeparator) {
 			pMenu->Delete(pSeparator);
+		}
 	}
 
 	auto ids = m_bookmark_menu_ids.begin();
 
 	// Insert global bookmarks
 	std::list<wxString> global_bookmarks;
-	if (CBookmarksDialog::GetBookmarks(global_bookmarks) && !global_bookmarks.empty())
-	{
+	if (CBookmarksDialog::GetGlobalBookmarks(global_bookmarks) && !global_bookmarks.empty()) {
 		pMenu->AppendSeparator();
 
-		for (std::list<wxString>::const_iterator iter = global_bookmarks.begin(); iter != global_bookmarks.end(); ++iter)
-		{
+		for (std::list<wxString>::const_iterator iter = global_bookmarks.begin(); iter != global_bookmarks.end(); ++iter) {
 			int id;
-			if (ids == m_bookmark_menu_ids.end())
-			{
+			if (ids == m_bookmark_menu_ids.end()) {
 				id = wxNewId();
 				m_bookmark_menu_ids.push_back(id);
 			}
-			else
-			{
+			else {
 				id = *ids;
 				++ids;
 			}
@@ -199,86 +191,89 @@ void CMenuBar::UpdateBookmarkMenu()
 	// Insert site-specific bookmarks
 	CContextControl* pContextControl = m_pMainFrame ? m_pMainFrame->GetContextControl() : 0;
 	CContextControl::_context_controls* controls = pContextControl ? pContextControl->GetCurrentControls() : 0;
-	if (!controls)
+	CState* pState = controls ? controls->pState : 0;
+	if (!pState) {
 		return;
+	}
 
-	if (!controls->site_bookmarks || controls->site_bookmarks->bookmarks.empty())
+	Site site = pState->GetSite();
+	if (!site.m_server) {
+		site = pState->GetLastSite();
+	}
+	if (site.m_bookmarks.empty()) {
 		return;
+	}
 
 	pMenu->AppendSeparator();
 
-	for (std::list<wxString>::const_iterator iter = controls->site_bookmarks->bookmarks.begin(); iter != controls->site_bookmarks->bookmarks.end(); ++iter)
-	{
+	for (auto const& bookmark : site.m_bookmarks) {
 		int id;
-		if (ids == m_bookmark_menu_ids.end())
-		{
+		if (ids == m_bookmark_menu_ids.end()) {
 			id = wxNewId();
 			m_bookmark_menu_ids.push_back(id);
 		}
-		else
-		{
+		else {
 			id = *ids;
 			++ids;
 		}
-		wxString name(*iter);
+		wxString name(bookmark.m_name);
 		name.Replace(_T("&"), _T("&&"));
 		pMenu->Append(id, name);
 
-		m_bookmark_menu_id_map_site[id] = *iter;
+		m_bookmark_menu_id_map_site[id] = bookmark.m_name;
 	}
 }
 
-void CMenuBar::ClearBookmarks()
-{
-	CContextControl* pContextControl = m_pMainFrame ? m_pMainFrame->GetContextControl() : 0;
-	CContextControl::_context_controls* controls = pContextControl ? pContextControl->GetCurrentControls() : 0;
-
-	if (controls && !controls->site_bookmarks)
-		controls->site_bookmarks = std::make_shared<CContextControl::_context_controls::_site_bookmarks>();
-	UpdateBookmarkMenu();
-}
-
 void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
+	if (!pState) {
 		return;
+	}
 
 	std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_site.find(event.GetId());
 	if (iter != m_bookmark_menu_id_map_site.end()) {
 		// We hit a site-specific bookmark
 		CContextControl* pContextControl = m_pMainFrame ? m_pMainFrame->GetContextControl() : 0;
 		CContextControl::_context_controls* controls = pContextControl ? pContextControl->GetCurrentControls() : 0;
-		if (!controls)
-			return;
-		if (controls->site_bookmarks->path.empty())
+		CState* pState = controls ? controls->pState : 0;
+		if (!pState)
 			return;
 
-		wxString name = iter->second;
-		name.Replace(_T("\\"), _T("\\\\"));
-		name.Replace(_T("/"), _T("\\/"));
-		name = controls->site_bookmarks->path + _T("/") + name;
-
-		std::unique_ptr<CSiteManagerItemData_Site> pData = CSiteManager::GetSiteByPath(name);
-		if (!pData)
-			return;
-
-		pState->SetSyncBrowse(false);
-		if (!pData->m_remoteDir.empty() && pState->IsRemoteIdle(true)) {
-			const CServer* pServer = pState->GetServer();
-			if (!pServer || *pServer != pData->m_server) {
-				m_pMainFrame->ConnectToSite(*pData);
-				pData->m_localDir.clear(); // So not to set again below
-			}
-			else
-				pState->ChangeRemoteDir(pData->m_remoteDir);
+		Site site = pState->GetSite();
+		if (!site.m_server) {
+			site = pState->GetLastSite();
 		}
-		if (!pData->m_localDir.empty()) {
-			bool set = pState->SetLocalDir(pData->m_localDir);
 
-			if (set && pData->m_sync) {
-				wxASSERT(!pData->m_remoteDir.empty());
-				pState->SetSyncBrowse(true, pData->m_remoteDir);
+
+		for (auto const& bookmark : site.m_bookmarks) {
+			if (bookmark.m_name == iter->second) {
+
+				pState->SetSyncBrowse(false);
+				if (!bookmark.m_remoteDir.empty() && pState->IsRemoteIdle(true)) {
+					const CServer* pServer = pState->GetServer();
+					if (!pServer || *pServer != site.m_server) {
+						m_pMainFrame->ConnectToSite(site, bookmark);
+						break;
+					}
+					else {
+						pState->ChangeRemoteDir(bookmark.m_remoteDir, std::wstring(), 0, false, bookmark.m_comparison);
+					}
+				}
+				if (!bookmark.m_localDir.empty()) {
+					bool set = pState->SetLocalDir(bookmark.m_localDir.ToStdWstring());
+
+					if (set && bookmark.m_sync) {
+						wxASSERT(!bookmark.m_remoteDir.empty());
+						pState->SetSyncBrowse(true, bookmark.m_remoteDir);
+					}
+				}
+
+				if (bookmark.m_comparison && pState->GetComparisonManager()) {
+					pState->GetComparisonManager()->CompareListings();
+				}
+
+				break;
 			}
 		}
 
@@ -292,8 +287,9 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 		CServerPath remote_dir;
 		bool sync;
 		bool comparison;
-		if (!CBookmarksDialog::GetBookmark(iter2->second, local_dir, remote_dir, sync, comparison))
+		if (!CBookmarksDialog::GetBookmark(iter2->second, local_dir, remote_dir, sync, comparison)) {
 			return;
+		}
 
 		pState->SetSyncBrowse(false);
 		if (!remote_dir.empty() && pState->IsRemoteIdle(true)) {
@@ -304,11 +300,11 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 					wxMessageBoxEx(_("Selected global bookmark and current server use a different server type.\nUse site-specific bookmarks for this server."), _("Bookmark"), wxICON_EXCLAMATION, this);
 					return;
 				}
-				pState->ChangeRemoteDir(remote_dir);
+				pState->ChangeRemoteDir(remote_dir, _T(""), 0, false, comparison);
 			}
 		}
 		if (!local_dir.empty()) {
-			bool set = pState->SetLocalDir(local_dir);
+			bool set = pState->SetLocalDir(local_dir.ToStdWstring());
 
 			if (set && sync) {
 				wxASSERT(!remote_dir.empty());
@@ -316,7 +312,7 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 			}
 		}
 
-		if (comparison) {
+		if (comparison && pState->GetComparisonManager()) {
 			pState->GetComparisonManager()->CompareListings();
 		}
 
@@ -326,7 +322,7 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 	event.Skip();
 }
 
-void CMenuBar::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+void CMenuBar::OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*)
 {
 	switch (notification)
 	{
@@ -334,9 +330,13 @@ void CMenuBar::OnStateChange(CState* pState, enum t_statechange_notifications no
 		UpdateMenubarState();
 		UpdateBookmarkMenu();
 		break;
+	case STATECHANGE_GLOBALBOOKMARKS:
+		UpdateBookmarkMenu();
+		break;
 	case STATECHANGE_SERVER:
 	case STATECHANGE_REMOTE_IDLE:
 		UpdateMenubarState();
+		UpdateBookmarkMenu();
 		break;
 	case STATECHANGE_QUEUEPROCESSING:
 		{
@@ -436,8 +436,9 @@ void CMenuBar::UpdateSpeedLimitMenuItem()
 void CMenuBar::UpdateMenubarState()
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
+	if (!pState) {
 		return;
+	}
 
 	const CServer* const pServer = pState->GetServer();
 	const bool idle = pState->IsRemoteIdle();
@@ -447,7 +448,6 @@ void CMenuBar::UpdateMenubarState()
 	Enable(XRCID("ID_MENU_SERVER_CMD"), pServer && idle);
 	Enable(XRCID("ID_MENU_FILE_COPYSITEMANAGER"), pServer != 0);
 	Enable(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pServer != 0);
-	Enable(XRCID("ID_MENU_SERVER_SEARCH"), pServer && idle);
 
 	Check(XRCID("ID_TOOLBAR_COMPARISON"), pState->GetComparisonManager()->IsComparing());
 	Check(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pState->GetSyncBrowse());
@@ -456,8 +456,7 @@ void CMenuBar::UpdateMenubarState()
 	if (pServer || !idle)
 		canReconnect = false;
 	else {
-		CServer tmp;
-		canReconnect = !pState->GetLastServer().GetHost().empty();
+		canReconnect = static_cast<bool>(pState->GetLastSite().m_server);
 	}
 	Enable(XRCID("ID_MENU_SERVER_RECONNECT"), canReconnect);
 
@@ -471,14 +470,11 @@ void CMenuBar::UpdateMenubarState()
 
 bool CMenuBar::ShowItem(int id)
 {
-	for (auto menu_iter = m_hidden_items.begin(); menu_iter != m_hidden_items.end(); ++menu_iter)
-	{
+	for (auto menu_iter = m_hidden_items.begin(); menu_iter != m_hidden_items.end(); ++menu_iter) {
 		int offset = 0;
 
-		for (auto iter = menu_iter->second.begin(); iter != menu_iter->second.end(); ++iter)
-		{
-			if (iter->second->GetId() != id)
-			{
+		for (auto iter = menu_iter->second.begin(); iter != menu_iter->second.end(); ++iter) {
+			if (iter->second->GetId() != id) {
 				offset++;
 				continue;
 			}
@@ -510,8 +506,7 @@ bool CMenuBar::HideItem(int id)
 
 	auto menu_iter = m_hidden_items.insert(std::make_pair(pMenu, std::map<int, wxMenuItem*>())).first;
 
-	for (auto iter = menu_iter->second.begin(); iter != menu_iter->second.end(); ++iter)
-	{
+	for (auto iter = menu_iter->second.begin(); iter != menu_iter->second.end(); ++iter) {
 		if (iter->first > (int)pos)
 			break;
 
