@@ -24,17 +24,15 @@ END_EVENT_TABLE()
 
 // Mark some strings used by wx as translatable
 #if 0
-TRANSLATE_T("&Next >");
-TRANSLATE_T("< &Back");
+fztranslate_mark("&Next >");
+fztranslate_mark("< &Back");
 #endif
 
 CNetConfWizard::CNetConfWizard(wxWindow* parent, COptions* pOptions, CFileZillaEngineContext & engine_context)
 	: fz::event_handler(engine_context.GetEventLoop())
+	, engine_context_(engine_context)
 	, m_parent(parent), m_pOptions(pOptions), m_pSocketServer(0)
 {
-	m_pIPResolver = 0;
-	m_pSendBuffer = 0;
-
 	m_timer.SetOwner(this);
 
 	ResetTest();
@@ -48,6 +46,7 @@ CNetConfWizard::~CNetConfWizard()
 	delete m_pIPResolver;
 	delete [] m_pSendBuffer;
 	delete m_pSocketServer;
+	delete m_pDataSocket;
 }
 
 bool CNetConfWizard::Load()
@@ -57,8 +56,7 @@ bool CNetConfWizard::Load()
 
 	wxSize minPageSize = GetPageAreaSizer()->GetMinSize();
 
-	for (int i = 1; i <= 7; i++)
-	{
+	for (int i = 1; i <= 7; ++i) {
 		wxWizardPageSimple* page = new wxWizardPageSimple();
 		bool res = wxXmlResource::Get()->LoadPanel(page, this, wxString::Format(_T("NETCONF_PANEL%d"), i));
 		if (!res)
@@ -192,8 +190,10 @@ void CNetConfWizard::OnPageChanging(wxWizardEvent& event)
 		}
 	}
 	else if (event.GetPage() == m_pages[5] && !event.GetDirection()) {
-		wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-		pNext->SetLabel(m_nextLabelText);
+		auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+		if (pNext) {
+			pNext->SetLabel(m_nextLabelText);
+		}
 	}
 	else if (event.GetPage() == m_pages[5] && event.GetDirection()) {
 		if (m_testDidRun)
@@ -201,17 +201,21 @@ void CNetConfWizard::OnPageChanging(wxWizardEvent& event)
 
 		m_testDidRun = true;
 
-		wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-		pNext->Disable();
-		wxButton* pPrev = wxDynamicCast(FindWindow(wxID_BACKWARD), wxButton);
-		pPrev->Disable();
+		auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+		if (pNext) {
+			pNext->Disable();
+		}
+		auto pPrev = dynamic_cast<wxButton*>(FindWindow(wxID_BACKWARD));
+		if (pPrev) {
+			pPrev->Disable();
+		}
 		event.Veto();
 
 		PrintMessage(wxString::Format(_("Connecting to %s"), _T("probe.filezilla-project.org")), 0);
-		m_socket = new CSocket(this);
+		m_socket = new CSocket(engine_context_.GetThreadPool(), this);
 		m_recvBufferPos = 0;
 
-		int res = m_socket->Connect(_T("probe.filezilla-project.org"), 21);
+		int res = m_socket->Connect(fzT("probe.filezilla-project.org"), 21);
 		if (res && res != EINPROGRESS) {
 			PrintMessage(wxString::Format(_("Connect failed: %s"), CSocket::GetErrorDescription(res)), 1);
 			CloseSocket();
@@ -222,15 +226,21 @@ void CNetConfWizard::OnPageChanging(wxWizardEvent& event)
 void CNetConfWizard::OnPageChanged(wxWizardEvent& event)
 {
 	if (event.GetPage() == m_pages[5]) {
-		wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-		m_nextLabelText = pNext->GetLabel();
-		pNext->SetLabel(_("&Test"));
+		auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+		if (pNext) {
+			m_nextLabelText = pNext->GetLabel();
+			pNext->SetLabel(_("&Test"));
+		}
 	}
 	else if (event.GetPage() == m_pages[6]) {
-		wxButton* pPrev = wxDynamicCast(FindWindow(wxID_BACKWARD), wxButton);
-		pPrev->Disable();
-		wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-		pNext->SetFocus();
+		auto pPrev = dynamic_cast<wxButton*>(FindWindow(wxID_BACKWARD));
+		if (pPrev) {
+			pPrev->Disable();
+		}
+		auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+		if (pNext) {
+			pNext->SetFocus();
+		}
 	}
 }
 
@@ -392,7 +402,7 @@ void CNetConfWizard::OnReceive()
 			i = -1;
 		}
 
-		if (m_recvBufferPos == 100) {
+		if (m_recvBufferPos == 200) {
 			m_testResult = servererror;
 			PrintMessage(_("Invalid data received"), 1);
 			CloseSocket();
@@ -551,9 +561,11 @@ void CNetConfWizard::CloseSocket()
 
 	PrintMessage(_("Connection closed"), 0);
 
-	wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-	pNext->Enable();
-	pNext->SetLabel(m_nextLabelText);
+	auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+	if (pNext) {
+		pNext->Enable();
+		pNext->SetLabel(m_nextLabelText);
+	}
 
 	wxString text[5];
 	if (!m_connectSuccessful)
@@ -573,6 +585,7 @@ void CNetConfWizard::CloseSocket()
 			text[3] = _("See also: https://wiki.filezilla-project.org/Network_Configuration");
 			break;
 		case successful:
+			PrintMessage(_("Test finished successfully"), 0);
 			text[0] = _("Congratulations, your configuration seems to be working.");
 			text[1] = _("You should have no problems connecting to other servers, file transfers should work properly.");
 			text[2] = _("If you keep having problems with a specific server, the server itself or a remote router or firewall might be misconfigured. In this case try to toggle passive mode and contact the server administrator for help.");
@@ -634,11 +647,13 @@ void CNetConfWizard::CloseSocket()
 			break;
 		}
 	}
-	for (unsigned int i = 0; i < 5; i++)
-	{
+	for (unsigned int i = 0; i < 5; ++i) {
 		wxString name = wxString::Format(_T("ID_SUMMARY%d"), i + 1);
 		int id = wxXmlResource::GetXRCID(name);
-		wxDynamicCast(FindWindowById(id, this), wxStaticText)->SetLabel(text[i]);
+		auto ctrl = dynamic_cast<wxStaticText*>(FindWindowById(id, this));
+		if (ctrl) {
+			ctrl->SetLabel(text[i]);
+		}
 	}
 	m_pages[6]->GetSizer()->Layout();
 	m_pages[6]->GetSizer()->Fit(m_pages[6]);
@@ -694,7 +709,7 @@ wxString CNetConfWizard::GetExternalIPAddress()
 	if (!mode) {
 		ret = m_socket->GetLocalIP();
 		if (ret.empty()) {
-			PrintMessage(_("Failed to retrieve local ip address. Aborting"), 1);
+			PrintMessage(_("Failed to retrieve local ip address, aborting."), 1);
 			CloseSocket();
 		}
 	}
@@ -705,20 +720,21 @@ wxString CNetConfWizard::GetExternalIPAddress()
 	else if (mode == 2) {
 		if (!m_pIPResolver) {
 			wxTextCtrl* pResolver = XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl);
-			wxString address = pResolver->GetValue();
+			std::wstring address = pResolver->GetValue().ToStdWstring();
 
 			PrintMessage(wxString::Format(_("Retrieving external IP address from %s"), address), 0);
 
-			m_pIPResolver = new CExternalIPResolver(*this);
+			m_pIPResolver = new CExternalIPResolver(engine_context_.GetThreadPool(), *this);
 			m_pIPResolver->GetExternalIP(address, CSocket::ipv4, true);
-			if (!m_pIPResolver->Done())
+			if (!m_pIPResolver->Done()) {
 				return wxString();
+			}
 		}
 		if (m_pIPResolver->Successful()) {
 			ret = m_pIPResolver->GetIP();
 		}
 		else {
-			PrintMessage(_("Failed to retrieve external ip address, aborting"), 1);
+			PrintMessage(_("Failed to retrieve external ip address, aborting."), 1);
 
 			m_testResult = externalfailed;
 			CloseSocket();
@@ -789,7 +805,7 @@ void CNetConfWizard::SendNextCommand()
 			int port = CreateListenSocket();
 			if (!port)
 			{
-				PrintMessage(_("Failed to create listen socket, aborting"), 1);
+				PrintMessage(_("Failed to create listen socket, aborting."), 1);
 				CloseSocket();
 				return;
 			}
@@ -863,8 +879,8 @@ void CNetConfWizard::OnFinish(wxWizardEvent&)
 	XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->GetValue().ToLong(&tmp); m_pOptions->SetOption(OPTION_LIMITPORTS_LOW, tmp);
 	XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->GetValue().ToLong(&tmp); m_pOptions->SetOption(OPTION_LIMITPORTS_HIGH, tmp);
 
-	m_pOptions->SetOption(OPTION_EXTERNALIP, XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl)->GetValue());
-	m_pOptions->SetOption(OPTION_EXTERNALIPRESOLVER, XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->GetValue());
+	m_pOptions->SetOption(OPTION_EXTERNALIP, XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl)->GetValue().ToStdWstring());
+	m_pOptions->SetOption(OPTION_EXTERNALIPRESOLVER, XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->GetValue().ToStdWstring());
 	m_pOptions->SetOption(OPTION_NOEXTERNALONLOCAL, XRCCTRL(*this, "ID_NOEXTERNALONLOCAL", wxCheckBox)->GetValue());
 }
 
@@ -900,8 +916,8 @@ int CNetConfWizard::CreateListenSocket()
 
 int CNetConfWizard::CreateListenSocket(unsigned int port)
 {
-	m_pSocketServer = new CSocket(this);
-	int res = m_pSocketServer->Listen(CSocket::unspec, port);
+	m_pSocketServer = new CSocket(engine_context_.GetThreadPool(), this);
+	int res = m_pSocketServer->Listen(m_socket ? m_socket->GetAddressFamily() : CSocket::unspec, port);
 
 	if (res < 0) {
 		delete m_pSocketServer;
@@ -935,9 +951,10 @@ void CNetConfWizard::OnAccept()
 	m_pDataSocket = m_pSocketServer->Accept(error);
 	if (!m_pDataSocket)
 		return;
+	m_pDataSocket->SetEventHandler(this);
 
-	wxString peerAddr = m_socket->GetPeerHost();
-	wxString dataPeerAddr = m_pDataSocket->GetPeerHost();
+	std::string peerAddr = m_socket->GetPeerIP();
+	std::string dataPeerAddr = m_pDataSocket->GetPeerIP();
 	if (peerAddr.empty()) {
 		delete m_pDataSocket;
 		m_pDataSocket = 0;
