@@ -4,7 +4,7 @@
 #include "aui_notebook_ex.h"
 #include "listctrlex.h"
 #include "edithandler.h"
-#include "optional.h"
+#include <libfilezilla/optional.hpp>
 
 enum class QueuePriority : char {
 	lowest,
@@ -20,7 +20,6 @@ enum class QueueItemType {
 	Server,
 	File,
 	Folder,
-	FolderScan,
 	Status
 };
 
@@ -79,7 +78,7 @@ private:
 };
 
 class CFileItem;
-class CServerItem : public CQueueItem
+class CServerItem final : public CQueueItem
 {
 public:
 	CServerItem(const CServer& server);
@@ -98,7 +97,7 @@ public:
 	virtual bool RemoveChild(CQueueItem* pItem, bool destroy = true, bool forward = true); // Removes a child item with is somewhere in the tree of children
 	virtual bool TryRemoveAll();
 
-	int64_t GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const;
+	int64_t GetTotalSize(int& filesWithUnknownSize, int& queuedFiles) const;
 
 	void QueueImmediateFiles();
 	void QueueImmediateFile(CFileItem* pItem);
@@ -147,8 +146,8 @@ class CFileItem : public CQueueItem
 {
 public:
 	CFileItem(CServerItem* parent, bool queued, bool download,
-		const wxString& sourceFile, const wxString& targetFile,
-		const CLocalPath& localPath, const CServerPath& remotePath, int64_t size);
+		std::wstring const& sourceFile, std::wstring const& targetFile,
+		CLocalPath const& localPath, CServerPath const& remotePath, int64_t size);
 
 	virtual ~CFileItem();
 
@@ -159,7 +158,7 @@ public:
 	std::wstring const& GetLocalFile() const { return !Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
 	std::wstring const& GetRemoteFile() const { return Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
 	std::wstring const& GetSourceFile() const { return m_sourceFile; }
-	CSparseOptional<std::wstring> const& GetTargetFile() const { return m_targetFile; }
+	fz::sparse_optional<std::wstring> const& GetTargetFile() const { return m_targetFile; }
 	CLocalPath const& GetLocalPath() const { return m_localPath; }
 	CServerPath const& GetRemotePath() const { return m_remotePath; }
 	int64_t GetSize() const { return m_size; }
@@ -262,17 +261,17 @@ public:
 
 protected:
 	std::wstring const m_sourceFile;
-	CSparseOptional<std::wstring> m_targetFile;
+	fz::sparse_optional<std::wstring> m_targetFile;
 	CLocalPath const m_localPath;
 	CServerPath const m_remotePath;
 	int64_t m_size{};
 };
 
-class CFolderItem : public CFileItem
+class CFolderItem final : public CFileItem
 {
 public:
-	CFolderItem(CServerItem* parent, bool queued, const CLocalPath& localPath);
-	CFolderItem(CServerItem* parent, bool queued, const CServerPath& remotePath, const wxString& remoteFile);
+	CFolderItem(CServerItem* parent, bool queued, CLocalPath const& localPath);
+	CFolderItem(CServerItem* parent, bool queued, CServerPath const& remotePath, std::wstring const& remoteFile);
 
 	virtual QueueItemType GetType() const { return QueueItemType::Folder; }
 
@@ -283,43 +282,7 @@ public:
 	virtual bool TryRemoveAll() { return true; }
 };
 
-class CFolderScanItem : public CQueueItem
-{
-public:
-	CFolderScanItem(CServerItem* parent, bool queued, bool download, const CLocalPath& localPath, const CServerPath& remotePath);
-	virtual ~CFolderScanItem() {}
-
-	virtual QueueItemType GetType() const { return QueueItemType::FolderScan; }
-	CLocalPath GetLocalPath() const { return m_localPath; }
-	CServerPath GetRemotePath() const { return m_remotePath; }
-	bool Download() const { return m_download; }
-	int GetCount() const { return m_count; }
-	virtual bool TryRemoveAll();
-
-	bool queued() const { return m_queued; }
-	void set_queued(bool q) { m_queued = q; }
-
-	wxString m_statusMessage;
-
-	volatile bool m_remove{};
-	bool m_active{};
-
-	int m_count{};
-
-	CFileExistsNotification::OverwriteAction m_defaultFileExistsAction{CFileExistsNotification::unknown};
-
-	bool m_dir_is_empty{};
-	CLocalPath m_current_local_path;
-	CServerPath m_current_remote_path;
-
-protected:
-	bool m_queued;
-	CLocalPath m_localPath;
-	CServerPath m_remotePath;
-	bool m_download;
-};
-
-class CStatusItem : public CQueueItem
+class CStatusItem final : public CQueueItem
 {
 public:
 	CStatusItem() {}
@@ -389,13 +352,11 @@ protected:
 	void DisplayNumberQueuedFiles();
 
 	// Position at which insertions start and number of insertions
-	int m_insertionStart;
-	unsigned int m_insertionCount;
+	int m_insertionStart{-1};
+	unsigned int m_insertionCount{};
 
-	int m_fileCount;
-	int m_folderScanCount;
-	bool m_fileCountChanged;
-	bool m_folderScanCountChanged;
+	int m_fileCount{};
+	bool m_fileCountChanged{};
 
 	// Selection management.
 	void UpdateSelections_ItemAdded(int added);
@@ -403,8 +364,8 @@ protected:
 	void UpdateSelections_ItemRemoved(int removed);
 	void UpdateSelections_ItemRangeRemoved(int removed, int count);
 
-	int m_itemCount;
-	bool m_allowBackgroundErase;
+	int m_itemCount{};
+	bool m_allowBackgroundErase{true};
 
 	std::vector<CServerItem*> m_serverList;
 
@@ -433,7 +394,7 @@ class CQueueViewSuccessful;
 
 class CMainFrame;
 class CAsyncRequestQueue;
-class CQueue : public wxAuiNotebookEx
+class CQueue final : public wxAuiNotebookEx
 {
 public:
 	CQueue(wxWindow* parent, CMainFrame* pMainFrame, CAsyncRequestQueue* pAsyncRequestQueue);
