@@ -14,6 +14,10 @@
 #include "window_state_manager.h"
 #include "xrc_helper.h"
 
+#include "uri.h"
+
+#include <wx/clipbrd.h>
+
 class CRemoteSearchFileData final : public CDirentry
 {
 public:
@@ -84,12 +88,12 @@ inline int DoCmpName(CLocalSearchFileData const& data1, CLocalSearchFileData con
 	return res;
 }
 
-class CSearchDialogFileList : public CFileListCtrl<CGenericFileData>
+class CSearchDialogFileList final : public CFileListCtrl<CGenericFileData>
 {
 	friend class CSearchDialog;
 	friend class CSearchSortType;
 public:
-	CSearchDialogFileList(CSearchDialog* pParent, CState &state, CQueueView* pQueue);
+	CSearchDialogFileList(CSearchDialog* pParent, CQueueView* pQueue);
 
 	void clear();
 	void set_mode(CSearchDialog::search_mode mode);
@@ -131,9 +135,9 @@ private:
 // -----------------------
 
 // Defined in RemoteListView.cpp
-extern wxString StripVMSRevision(const wxString& name);
+std::wstring StripVMSRevision(std::wstring const& name);
 
-CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CState& state, CQueueView* pQueue)
+CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CQueueView* pQueue)
 	: CFileListCtrl<CGenericFileData>(pParent, pQueue, true),
 	m_searchDialog(pParent)
 {
@@ -194,7 +198,7 @@ CFileListCtrl<CGenericFileData>::CSortComparisonObject CSearchDialogFileList::Ge
 				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
 			else if (m_sortColumn == 4)
 				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-		
+
 			else
 				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
 		}
@@ -207,7 +211,7 @@ CFileListCtrl<CGenericFileData>::CSortComparisonObject CSearchDialogFileList::Ge
 				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
 			else if (m_sortColumn == 4)
 				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
-			
+
 			else
 				return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>(localFileData_, m_fileData, dirSortMode, nameSortMode, this));
 		}
@@ -361,6 +365,8 @@ EVT_MENU(XRCID("ID_MENU_SEARCH_DOWNLOAD"), CSearchDialog::OnDownload)
 EVT_MENU(XRCID("ID_MENU_SEARCH_UPLOAD"), CSearchDialog::OnUpload)
 EVT_MENU(XRCID("ID_MENU_SEARCH_EDIT"), CSearchDialog::OnEdit)
 EVT_MENU(XRCID("ID_MENU_SEARCH_DELETE"), CSearchDialog::OnDelete)
+EVT_MENU(XRCID("ID_MENU_SEARCH_GETURL"), CSearchDialog::OnGetUrl)
+EVT_MENU(XRCID("ID_MENU_SEARCH_GETURL_PASSWORD"), CSearchDialog::OnGetUrl)
 EVT_CHAR_HOOK(CSearchDialog::OnCharHook)
 EVT_RADIOBUTTON(XRCID("ID_LOCAL_SEARCH"), CSearchDialog::OnChangeSearchMode)
 EVT_RADIOBUTTON(XRCID("ID_REMOTE_SEARCH"), CSearchDialog::OnChangeSearchMode)
@@ -396,7 +402,7 @@ bool CSearchDialog::Load()
 	if (!CreateListControl(filter_name | filter_size | filter_path | filter_date))
 		return false;
 
-	m_results = new CSearchDialogFileList(this, m_state, 0);
+	m_results = new CSearchDialogFileList(this, 0);
 	ReplaceControl(XRCCTRL(*this, "ID_RESULTS", wxWindow), m_results);
 
 	m_results->SetFilelistStatusBar(pStatusBar);
@@ -645,7 +651,7 @@ void CSearchDialog::OnSearch(wxCommandEvent&)
 			return;
 		}
 
-		wxString error;
+		std::wstring error;
 		if (!path.Exists(&error)) {
 			wxMessageBoxEx(error, _("Local file search"), wxICON_EXCLAMATION);
 			return;
@@ -667,7 +673,7 @@ void CSearchDialog::OnSearch(wxCommandEvent&)
 			return;
 		}
 		path.SetType(pServer->GetType());
-		if (!path.SetPath(xrc_call(*this, "ID_PATH", &wxTextCtrl::GetValue)) || path.empty()) {
+		if (!path.SetPath(xrc_call(*this, "ID_PATH", &wxTextCtrl::GetValue).ToStdWstring()) || path.empty()) {
 			wxMessageBoxEx(_("Need to enter valid remote path"), _("Remote file search"), wxICON_EXCLAMATION);
 			return;
 		}
@@ -698,7 +704,7 @@ void CSearchDialog::OnSearch(wxCommandEvent&)
 	m_visited.clear();
 
 	// Start
-	
+
 	if (localSearch) {
 		local_recursion_root root;
 		root.add_dir_to_visit(m_local_search_root);
@@ -708,7 +714,7 @@ void CSearchDialog::OnSearch(wxCommandEvent&)
 	}
 	else {
 		recursion_root root(m_remote_search_root, true);
-		root.add_dir_to_visit_restricted(m_remote_search_root, _T(""), true);
+		root.add_dir_to_visit_restricted(m_remote_search_root, std::wstring(), true);
 		m_state.GetRemoteRecursiveOperation()->AddRecursionRoot(std::move(root));
 		ActiveFilters const filters; // Empty, recurse into everything
 		m_state.GetRemoteRecursiveOperation()->StartRecursiveOperation(CRecursiveOperation::recursive_list, filters, m_remote_search_root);
@@ -778,9 +784,12 @@ void CSearchDialog::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Delete(XRCID("ID_MENU_SEARCH_DOWNLOAD"));
 		pMenu->Delete(XRCID("ID_MENU_SEARCH_DELETE"));
 		pMenu->Delete(XRCID("ID_MENU_SEARCH_EDIT"));
+		pMenu->Delete(XRCID("ID_MENU_SEARCH_GETURL"));
+		pMenu->Delete(XRCID("ID_MENU_SEARCH_GETURL_PASSWORD"));
 	}
 	else {
 		pMenu->Delete(XRCID("ID_MENU_SEARCH_UPLOAD"));
+		pMenu->Delete(XRCID(wxGetKeyState(WXK_SHIFT) ? "ID_MENU_SEARCH_GETURL" : "ID_MENU_SEARCH_GETURL_PASSWORD"));
 	}
 
 	PopupMenu(pMenu);
@@ -789,7 +798,7 @@ void CSearchDialog::OnContextMenu(wxContextMenuEvent& event)
 
 
 
-class CSearchTransferDialog : public wxDialogEx
+class CSearchTransferDialog final : public wxDialogEx
 {
 public:
 	bool Run(wxWindow* parent, bool download, const wxString& path, int count_files, int count_dirs)
@@ -854,7 +863,7 @@ void CSearchTransferDialog::OnOK(wxCommandEvent&)
 	if (download_) {
 		wxTextCtrl *pText = XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl);
 
-		CLocalPath path(pText->GetValue());
+		CLocalPath path(pText->GetValue().ToStdWstring());
 		if (path.empty()) {
 			wxMessageBoxEx(_("You have to enter a local directory."), _("Download search results"), wxICON_EXCLAMATION);
 			return;
@@ -868,7 +877,7 @@ void CSearchTransferDialog::OnOK(wxCommandEvent&)
 	else {
 		wxTextCtrl *pText = XRCCTRL(*this, "ID_REMOTEPATH", wxTextCtrl);
 
-		CServerPath path(pText->GetValue());
+		CServerPath path(pText->GetValue().ToStdWstring());
 		if (path.empty()) {
 			wxMessageBoxEx(_("You have to enter a remote directory."), _("Upload search results"), wxICON_EXCLAMATION);
 			return;
@@ -895,8 +904,9 @@ void ProcessSelection(std::list<int> &selected_files, std::deque<Path> &selected
 
 	int sel = -1;
 	while ((sel = results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
-		if (sel > (int)results->indexMapping().size())
+		if (static_cast<size_t>(sel) >= results->indexMapping().size()) {
 			continue;
+		}
 		int index = results->indexMapping()[sel];
 
 		if (fileData[index].is_dir()) {
@@ -965,7 +975,7 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 
 	wxTextCtrl *pText = XRCCTRL(dlg, "ID_LOCALPATH", wxTextCtrl);
 
-	CLocalPath path(pText->GetValue());
+	CLocalPath path(pText->GetValue().ToStdWstring());
 	if (path.empty() || !path.IsWriteable()) {
 		wxBell();
 		return;
@@ -987,7 +997,7 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 		if (!flatten) {
 			// Append relative path to search root to local target path
 			CServerPath remote_path = entry.path;
-			std::list<wxString> segments;
+			std::list<std::wstring> segments;
 			while (m_remote_search_root.IsParentOf(remote_path, false) && remote_path.HasParent()) {
 				segments.push_front(remote_path.GetLastSegment());
 				remote_path = remote_path.GetParent();
@@ -998,12 +1008,12 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 		}
 
 		CServerPath remote_path = entry.path;
-		wxString localName = CQueueView::ReplaceInvalidCharacters(entry.name);
+		std::wstring localName = CQueueView::ReplaceInvalidCharacters(entry.name);
 		if (!entry.is_dir() && remote_path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
 			localName = StripVMSRevision(localName);
 
 		m_pQueue->QueueFile(!start, true,
-			entry.name, (localName != entry.name) ? localName : wxString(),
+			entry.name, (localName != entry.name) ? localName : std::wstring(),
 			target_path, remote_path, *pServer, entry.size);
 	}
 	m_pQueue->QueueFile_Finish(start);
@@ -1017,7 +1027,7 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 		}
 
 		recursion_root root(dir, true);
-		root.add_dir_to_visit(dir, _T(""), target_path, false);
+		root.add_dir_to_visit(dir, std::wstring(), target_path, false);
 		m_state.GetRemoteRecursiveOperation()->AddRecursionRoot(std::move(root));
 	}
 	ActiveFilters const filters; // Empty, recurse into everything
@@ -1045,7 +1055,7 @@ void CSearchDialog::OnUpload(wxCommandEvent&)
 
 	wxTextCtrl *pText = XRCCTRL(dlg, "ID_REMOTEPATH", wxTextCtrl);
 
-	CServerPath path(pText->GetValue());
+	CServerPath path(pText->GetValue().ToStdWstring());
 	if (path.empty()) {
 		wxBell();
 		return;
@@ -1067,7 +1077,7 @@ void CSearchDialog::OnUpload(wxCommandEvent&)
 		if (!flatten) {
 			// Append relative path to search root to local target path
 			CLocalPath local_path = entry.path;
-			std::list<wxString> segments;
+			std::list<std::wstring> segments;
 			while (m_local_search_root.IsParentOf(local_path) && local_path.HasParent()) {
 				segments.push_front(local_path.GetLastSegment());
 				local_path = local_path.GetParent();
@@ -1078,16 +1088,16 @@ void CSearchDialog::OnUpload(wxCommandEvent&)
 		}
 
 		CLocalPath local_path = entry.path;
-		wxString localName = CQueueView::ReplaceInvalidCharacters(entry.name);
+		std::wstring localName = CQueueView::ReplaceInvalidCharacters(entry.name);
 
 		m_pQueue->QueueFile(!start, false,
-			entry.name, (localName != entry.name) ? localName : wxString(),
+			entry.name, (localName != entry.name) ? localName : std::wstring(),
 			local_path, target_path, *pServer, entry.size);
 	}
 	m_pQueue->QueueFile_Finish(start);
 
 	auto const mode = flatten ? CRecursiveOperation::recursive_transfer_flatten : CRecursiveOperation::recursive_transfer;
-	
+
 	for (auto const& dir : selected_dirs) {
 		CServerPath target_path = path;
 		if (!flatten && dir.HasParent()) {
@@ -1200,7 +1210,7 @@ void CSearchDialog::OnDelete(wxCommandEvent&)
 	}
 
 	for (auto path : selected_dirs) {
-		wxString segment;
+		std::wstring segment;
 		if (path.HasParent()) {
 			segment = path.GetLastSegment();
 			path = path.GetParent();
@@ -1247,7 +1257,7 @@ void CSearchDialog::LoadConditions()
 	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("search")));
-	auto document = file.Load();
+	auto document = file.Load(true);
 	if (!document) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return;
@@ -1266,7 +1276,7 @@ void CSearchDialog::SaveConditions()
 	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("search")));
-	auto document = file.Load();
+	auto document = file.Load(true);
 	if (!document) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return;
@@ -1277,7 +1287,7 @@ void CSearchDialog::SaveConditions()
 		document.remove_child(filter);
 	filter = document.append_child("Filter");
 
-	CFilterDialog::SaveFilter(filter, m_search_filter);
+	CFilterManager::SaveFilter(filter, m_search_filter);
 
 	file.Save(true);
 }
@@ -1301,4 +1311,59 @@ void CSearchDialog::OnChangeSearchMode(wxCommandEvent&)
 		}
 	}
 
-}
\ No newline at end of file
+}
+
+void CSearchDialog::OnGetUrl(wxCommandEvent& event)
+{
+	if (m_results->mode_ != search_mode::remote) {
+		return;
+	}
+
+	CServer const* pServer = m_state.GetServer();
+	if (!pServer) {
+		wxBell();
+		return;
+	}
+
+	if (!wxTheClipboard->Open()) {
+		wxMessageBoxEx(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
+		return;
+	}
+
+	wxString const server = pServer->Format((event.GetId() == XRCID("ID_MENU_SEARCH_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
+
+	auto getUrl = [](wxString const& serverPart, CServerPath const& path, std::wstring const& name) {
+		wxString url = serverPart;
+
+		auto const pathPart = fz::percent_encode_w(path.FormatFilename(name, false), true);
+		if (!pathPart.empty() && pathPart[0] != '/') {
+			url += '/';
+		}
+		url += pathPart;
+
+		return url;
+	};
+
+	wxString urls;
+
+	int sel = -1;
+	while ((sel = m_results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) >= 0) {
+		if (static_cast<size_t>(sel) >= m_results->m_indexMapping.size()) {
+			continue;
+		}
+		int index = m_results->m_indexMapping[sel];
+
+		auto const& entry = m_results->remoteFileData_[index];
+		urls += getUrl(server, entry.path, entry.name);
+#ifdef __WXMSW__
+		urls += _T("\r\n");
+#else
+		urls += _T("\n");
+#endif
+	}
+
+	wxTheClipboard->SetData(new wxURLDataObject(urls));
+
+	wxTheClipboard->Flush();
+	wxTheClipboard->Close();
+}
