@@ -9,32 +9,39 @@
 #include <wx/dcclient.h>
 #include <wx/scrolwin.h>
 
+#include "xrc_helper.h"
+
 BEGIN_EVENT_TABLE(COptionsPageThemes, COptionsPage)
 EVT_CHOICE(XRCID("ID_THEME"), COptionsPageThemes::OnThemeChange)
 END_EVENT_TABLE()
 
 const int BORDER = 5;
 
-class CIconPreview : public wxScrolledWindow
+class CIconPreview final : public wxScrolledWindow
 {
 public:
+	CIconPreview() = default;
+
 	CIconPreview(wxWindow* pParent)
 		: wxScrolledWindow(pParent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSTATIC_BORDER | wxVSCROLL)
 	{
-		m_sizeInitialized = false;
-		m_extra_padding = 0;
 	}
 
-	void LoadIcons(const wxString& theme, const wxSize& iconSize)
+	void LoadIcons(std::wstring const& theme, wxSize const& size)
 	{
-		m_iconSize = iconSize;
-		m_icons = CThemeProvider::GetAllImages(theme, m_iconSize);
+		m_iconSize = size;
+
+		m_icons = CThemeProvider::Get()->GetAllImages(theme, size);
+
+		m_sizeInitialized = false;
+		Refresh();
 	}
 
 	void CalcSize()
 	{
-		if (m_sizeInitialized)
+		if (m_sizeInitialized) {
 			return;
+		}
 		m_sizeInitialized = true;
 
 		wxSize size = GetClientSize();
@@ -94,8 +101,8 @@ protected:
 		int x = BORDER;
 		int y = BORDER;
 
-		for( auto const& bmp : m_icons ) {
-			dc.DrawBitmap(*bmp, x, y, true);
+		for (auto const& bmp : m_icons) {
+			dc.DrawBitmap(bmp, x, y, true);
 			x += m_iconSize.GetWidth() + BORDER + m_extra_padding;
 			if ((x + m_iconSize.GetWidth() + BORDER) > size.GetWidth()) {
 				x = BORDER;
@@ -104,12 +111,16 @@ protected:
 		}
 	}
 
-	std::vector<std::unique_ptr<wxBitmap>> m_icons;
+	std::vector<wxBitmap> m_icons;
 	wxSize m_iconSize;
-	bool m_sizeInitialized;
-	int m_extra_padding;
+	bool m_sizeInitialized{};
+	int m_extra_padding{};
+
+	DECLARE_DYNAMIC_CLASS(CIconPreview)
 };
 
+IMPLEMENT_DYNAMIC_CLASS(CIconPreview, wxScrolledWindow)
+
 BEGIN_EVENT_TABLE(CIconPreview, wxScrolledWindow)
 EVT_PAINT(CIconPreview::OnPaint)
 END_EVENT_TABLE()
@@ -118,6 +129,36 @@ COptionsPageThemes::~COptionsPageThemes()
 {
 }
 
+bool COptionsPageThemes::CreatePage(COptions* pOptions, CSettingsDialog* pOwner, wxWindow* parent, wxSize& maxSize)
+{
+	bool success = COptionsPage::CreatePage(pOptions, pOwner, parent, maxSize);
+	if (success) {
+		auto dummy = XRCCTRL(*this, "ID_SCALE", wxWindow);
+		auto sizer = dummy ? dummy->GetContainingSizer() : 0;
+		if (!sizer) {
+			return false;
+		}
+
+		wxWindow* sizerParent = dummy->GetParent();
+		sizer->Detach(dummy);
+		delete dummy;
+
+		auto scale = new wxSpinCtrlDouble(sizerParent, XRCID("ID_SCALE"));
+		scale->SetRange(0.5, 4);
+		scale->SetIncrement(0.25);
+		scale->SetValue(1.25);
+		scale->SetDigits(2);
+		sizer->Add(scale, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
+		sizer->Layout();
+		GetSizer()->Layout();
+		GetSizer()->Fit(this);
+
+		scale->Connect(wxEVT_SPINCTRLDOUBLE, wxCommandEventHandler(COptionsPageThemes::OnThemeChange), 0, this);
+	}
+
+	return success;
+}
+
 bool COptionsPageThemes::LoadPage()
 {
 	return true;
@@ -125,19 +166,18 @@ bool COptionsPageThemes::LoadPage()
 
 bool COptionsPageThemes::SavePage()
 {
-	if (!m_was_selected)
+	if (!m_was_selected) {
 		return true;
+	}
 
 	wxChoice* pTheme = XRCCTRL(*this, "ID_THEME", wxChoice);
 
 	const int sel = pTheme->GetSelection();
 	const wxString theme = ((wxStringClientData*)pTheme->GetClientObject(sel))->GetData();
 
-	m_pOptions->SetOption(OPTION_THEME, theme);
+	m_pOptions->SetOption(OPTION_ICONS_THEME, theme.ToStdWstring());
 
-	wxNotebook *pPreview = XRCCTRL(*this, "ID_PREVIEW", wxNotebook);
-	wxString size = pPreview->GetPageText(pPreview->GetSelection());
-	m_pOptions->SetOption(OPTION_THEME_ICONSIZE, size);
+	m_pOptions->SetOption(OPTION_ICONS_SCALE, static_cast<int>(100 * xrc_call(*this, "ID_SCALE", &wxSpinCtrlDouble::GetValue)));
 
 	return true;
 }
@@ -147,10 +187,10 @@ bool COptionsPageThemes::Validate()
 	return true;
 }
 
-bool COptionsPageThemes::DisplayTheme(const wxString& theme)
+bool COptionsPageThemes::DisplayTheme(std::wstring const& theme)
 {
-	wxString name, author, mail;
-	if (!CThemeProvider::GetThemeData(theme, name, author, mail)) {
+	std::wstring name, author, mail;
+	if (!CThemeProvider::Get()->GetThemeData(theme, name, author, mail)) {
 		return false;
 	}
 	if (name.empty()) {
@@ -158,60 +198,21 @@ bool COptionsPageThemes::DisplayTheme(const wxString& theme)
 	}
 
 	if (author.empty()) {
-		author = _("N/a");
+		author = _("N/a").ToStdWstring();
 	}
 	if (mail.empty()) {
-		mail = _("N/a");
+		mail = _("N/a").ToStdWstring();
 	}
 
 	bool failure = false;
 	SetStaticText(XRCID("ID_AUTHOR"), author, failure);
 	SetStaticText(XRCID("ID_EMAIL"), mail, failure);
 
-	wxNotebook *pPreview = XRCCTRL(*this, "ID_PREVIEW", wxNotebook);
+	auto scale_factor = xrc_call(*this, "ID_SCALE", &wxSpinCtrlDouble::GetValue);
+	wxSize size = CThemeProvider::Get()->GetIconSize(iconSizeSmall);
+	size.Scale(scale_factor, scale_factor);
 
-	wxString selected_size;
-	int selected_page = pPreview->GetSelection();
-	if (selected_page != -1)
-		selected_size = pPreview->GetPageText(selected_page);
-	else
-		selected_size = COptions::Get()->GetOption(OPTION_THEME_ICONSIZE);
-
-	pPreview->DeleteAllPages();
-
-	for( auto const& size : CThemeProvider::GetThemeSizes(theme) ) {
-		int pos = size.Find('x');
-		if (pos <= 0 || pos >= (int)size.Len() - 1)
-			continue;
-
-		long width = 0;
-		long height = 0;
-		if (!size.Left(pos).ToLong(&width) ||
-			!size.Mid(pos + 1).ToLong(&height) ||
-			width <= 0 || height <= 0)
-		{
-			continue;
-		}
-
-		wxSize iconSize(width, height);
-
-		wxPanel* pPanel = new wxPanel();
-#ifdef __WXMSW__
-		// Drawing glitch on MSW
-		pPanel->Hide();
-#endif
-		pPanel->Create(pPreview);
-		wxSizer* pSizer = new wxBoxSizer(wxVERTICAL);
-		pPanel->SetSizer(pSizer);
-		pSizer->Add(new wxStaticText(pPanel, wxID_ANY, _("Preview:")), 0, wxLEFT|wxRIGHT|wxTOP, 5);
-
-		CIconPreview *pIconPreview = new CIconPreview(pPanel);
-		pIconPreview->LoadIcons(theme, iconSize);
-
-		pSizer->Add(pIconPreview, 1, wxGROW | wxALL, 5);
-
-		pPreview->AddPage(pPanel, size, selected_size == size);
-	}
+	xrc_call(*this, "ID_PREVIEW", &CIconPreview::LoadIcons, theme, size);
 
 	return !failure;
 }
@@ -221,8 +222,7 @@ void COptionsPageThemes::OnThemeChange(wxCommandEvent&)
 	wxChoice* pTheme = XRCCTRL(*this, "ID_THEME", wxChoice);
 
 	const int sel = pTheme->GetSelection();
-	const wxString theme = ((wxStringClientData*)pTheme->GetClientObject(sel))->GetData();
-
+	std::wstring const theme = ((wxStringClientData*)pTheme->GetClientObject(sel))->GetData().ToStdWstring();
 	DisplayTheme(theme);
 }
 
@@ -231,36 +231,47 @@ bool COptionsPageThemes::OnDisplayedFirstTime()
 	bool failure = false;
 
 	wxChoice* pTheme = XRCCTRL(*this, "ID_THEME", wxChoice);
-	if (!pTheme)
+	if (!pTheme) {
 		return false;
+	}
 
-	if (!pTheme || !XRCCTRL(*this, "ID_PREVIEW", wxNotebook) ||
+	if (!pTheme || !XRCCTRL(*this, "ID_PREVIEW", CIconPreview) ||
 		!XRCCTRL(*this, "ID_AUTHOR", wxStaticText) ||
 		!XRCCTRL(*this, "ID_EMAIL", wxStaticText))
+	{
 		return false;
+	}
 
 	auto const themes = CThemeProvider::GetThemes();
-	if (themes.empty())
+	if (themes.empty()) {
 		return false;
-
-	wxString activeTheme = m_pOptions->GetOption(OPTION_THEME);
-	wxString firstName;
-	for (auto const& theme : themes) {
-		wxString name, author, mail;
-		if (!CThemeProvider::GetThemeData(theme, name, author, mail))
-			continue;
-		if (firstName.empty())
-			firstName = name;
-		int n = pTheme->Append(name, new wxStringClientData(theme));
-		if (theme == activeTheme)
-			pTheme->SetSelection(n);
 	}
-	if (pTheme->GetSelection() == wxNOT_FOUND)
+
+	xrc_call<wxSpinCtrlDouble, double>(*this, "ID_SCALE", &wxSpinCtrlDouble::SetValue, static_cast<double>(m_pOptions->GetOptionVal(OPTION_ICONS_SCALE)) / 100.f);
+
+	std::wstring activeTheme = m_pOptions->GetOption(OPTION_ICONS_THEME);
+	std::wstring firstName;
+	for (auto const& theme : themes) {
+		std::wstring name, author, mail;
+		if (!CThemeProvider::Get()->GetThemeData(theme, name, author, mail)) {
+			continue;
+		}
+		if (firstName.empty()) {
+			firstName = name;
+		}
+		int n = pTheme->Append(name, new wxStringClientData(theme));
+		if (theme == activeTheme) {
+			pTheme->SetSelection(n);
+		}
+	}
+	if (pTheme->GetSelection() == wxNOT_FOUND) {
 		pTheme->SetSelection(pTheme->FindString(firstName));
+	}
 	activeTheme = ((wxStringClientData*)pTheme->GetClientObject(pTheme->GetSelection()))->GetData();
 
-	if (!DisplayTheme(activeTheme))
+	if (!DisplayTheme(activeTheme)) {
 		failure = true;
+	}
 
 	pTheme->GetContainingSizer()->Layout();
 
