@@ -10,8 +10,10 @@ public:
 	virtual bool SavePage();
 	virtual bool Validate();
 
+	virtual bool CreatePage(COptions* pOptions, CSettingsDialog* pOwner, wxWindow* parent, wxSize& maxSize);
+
 protected:
-	bool DisplayTheme(const wxString& theme);
+	bool DisplayTheme(std::wstring const& theme);
 
 	virtual bool OnDisplayedFirstTime();
 
