@@ -33,11 +33,10 @@ EVT_BUTTON(XRCID("wxID_OK"), CSettingsDialog::OnOK)
 EVT_BUTTON(XRCID("wxID_CANCEL"), CSettingsDialog::OnCancel)
 END_EVENT_TABLE()
 
-CSettingsDialog::CSettingsDialog()
+CSettingsDialog::CSettingsDialog(CFileZillaEngineContext & engine_context)
+	: m_engine_context(engine_context)
 {
-	m_pMainFrame = 0;
 	m_pOptions = COptions::Get();
-	m_activePanel = 0;
 }
 
 CSettingsDialog::~CSettingsDialog()
@@ -123,8 +122,8 @@ bool CSettingsDialog::LoadPages()
 	// Compensate for scrollbar
 	wxSize size = treeCtrl->GetBestSize();
 	int scrollWidth = wxSystemSettings::GetMetric(wxSYS_VSCROLL_X, treeCtrl);
-	size.x += scrollWidth;
-	size.y = 0;
+	size.x += scrollWidth + 2;
+	size.y = 400;
 	treeCtrl->SetInitialSize(size);
 	Layout();
 
@@ -229,7 +228,7 @@ void CSettingsDialog::OnPageChanged(wxTreeEvent& event)
 	}
 }
 
-void CSettingsDialog::OnOK(wxCommandEvent& event)
+void CSettingsDialog::OnOK(wxCommandEvent&)
 {
 	for( auto const& page : m_pages ) {
 		if (!page.page->Validate()) {
@@ -248,9 +247,13 @@ void CSettingsDialog::OnOK(wxCommandEvent& event)
 	EndModal(wxID_OK);
 }
 
-void CSettingsDialog::OnCancel(wxCommandEvent& event)
+void CSettingsDialog::OnCancel(wxCommandEvent&)
 {
 	EndModal(wxID_CANCEL);
+
+	for (auto const& saved : m_oldValues) {
+		m_pOptions->SetOption(saved.first, saved.second);
+	}
 }
 
 void CSettingsDialog::OnPageChanging(wxTreeEvent& event)
@@ -261,3 +264,8 @@ void CSettingsDialog::OnPageChanging(wxTreeEvent& event)
 	if (!m_activePanel->Validate())
 		event.Veto();
 }
+
+void CSettingsDialog::RememberOldValue(int option)
+{
+	m_oldValues[option] = m_pOptions->GetOption(option);
+}
