@@ -6,38 +6,46 @@
 class COptions;
 class COptionsPage;
 class CMainFrame;
-class CSettingsDialog : public wxDialogEx
+class CSettingsDialog final : public wxDialogEx
 {
 public:
-	CSettingsDialog();
+	CSettingsDialog(CFileZillaEngineContext & engine_context);
 	virtual ~CSettingsDialog();
 
 	bool Create(CMainFrame* pMainFrame);
 	bool LoadSettings();
 
-	CMainFrame* m_pMainFrame;
+	CMainFrame* m_pMainFrame{};
+
+	CFileZillaEngineContext& GetEngineContext() { return m_engine_context; }
+
+	void RememberOldValue(int option);
 
 protected:
 	bool LoadPages();
 
 	COptions* m_pOptions;
 
-	COptionsPage* m_activePanel;
+	COptionsPage* m_activePanel{};
 
 	void AddPage( wxString const& name, COptionsPage* page, int nest );
 
 	struct t_page
 	{
 		wxTreeItemId id;
-		COptionsPage* page;
+		COptionsPage* page{};
 	};
 	std::vector<t_page> m_pages;
 
+	std::map<int, std::wstring> m_oldValues;
+
 	DECLARE_EVENT_TABLE()
 	void OnPageChanging(wxTreeEvent& event);
 	void OnPageChanged(wxTreeEvent& event);
 	void OnOK(wxCommandEvent& event);
 	void OnCancel(wxCommandEvent& event);
+
+	CFileZillaEngineContext& m_engine_context;
 };
 
 #endif //__SETTINGSDIALOG_H__
