@@ -9,18 +9,18 @@
 namespace {
 struct background_color {
 	wxColour const color;
-	wchar_t const*const name;
+	char const*const name;
 };
 
 background_color const background_colors[] = {
-	{ wxColour(), TRANSLATE_T("None") },
-	{ wxColour(255, 0, 0, 32), TRANSLATE_T("Red") },
-	{ wxColour(0, 255, 0, 32), TRANSLATE_T("Green") },
-	{ wxColour(0, 0, 255, 32), TRANSLATE_T("Blue") },
-	{ wxColour(255, 255, 0, 32), TRANSLATE_T("Yellow") },
-	{ wxColour(0, 255, 255, 32), TRANSLATE_T("Cyan") },
-	{ wxColour(255, 0, 255, 32), TRANSLATE_T("Magenta") },
-	{ wxColour(255, 128, 0, 32), TRANSLATE_T("Orange") },
+	{ wxColour(), fztranslate_mark("None") },
+	{ wxColour(255, 0, 0, 32), fztranslate_mark("Red") },
+	{ wxColour(0, 255, 0, 32), fztranslate_mark("Green") },
+	{ wxColour(0, 0, 255, 32), fztranslate_mark("Blue") },
+	{ wxColour(255, 255, 0, 32), fztranslate_mark("Yellow") },
+	{ wxColour(0, 255, 255, 32), fztranslate_mark("Cyan") },
+	{ wxColour(255, 0, 255, 32), fztranslate_mark("Magenta") },
+	{ wxColour(255, 128, 0, 32), fztranslate_mark("Orange") },
 	{ wxColour(), 0 }
 };
 }
@@ -30,7 +30,7 @@ bool Bookmark::operator==(Bookmark const& b) const
 	if (m_localDir != b.m_localDir) {
 		return false;
 	}
-	
+
 	if (m_remoteDir != b.m_remoteDir) {
 		return false;
 	}
@@ -105,7 +105,7 @@ bool CSiteManager::Load(pugi::xml_node element, CSiteManagerXmlHandler& handler)
 
 	for (auto child = element.first_child(); child; child = child.next_sibling()) {
 		if (!strcmp(child.name(), "Folder")) {
-			std::wstring name = GetTextElement_Trimmed(child).ToStdWstring();
+			std::wstring name = GetTextElement_Trimmed(child);
 			if (name.empty()) {
 				continue;
 			}
@@ -162,7 +162,7 @@ std::unique_ptr<Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 
 	data->m_comments = GetTextElement(element, "Comments");
 	data->m_colour = GetColourFromIndex(GetTextElementInt(element, "Colour"));
-	
+
 	ReadBookmarkElement(data->m_default_bookmark, element);
 
 	// Bookmarks
@@ -330,8 +330,9 @@ std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu()
 	}
 
 	if (pMenu) {
-		if (!predefinedSites)
+		if (!predefinedSites) {
 			return pMenu;
+		}
 
 		auto pRootMenu = std::make_unique<wxMenu>();
 		pRootMenu->AppendSubMenu(predefinedSites.release(), _("Predefined Sites"));
@@ -340,8 +341,9 @@ std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu()
 		return pRootMenu;
 	}
 
-	if (predefinedSites)
+	if (predefinedSites) {
 		return predefinedSites;
+	}
 
 	pMenu = std::make_unique<wxMenu>();
 	wxMenuItem* pItem = pMenu->Append(wxID_ANY, _("No sites available"));
@@ -357,19 +359,22 @@ void CSiteManager::ClearIdMap()
 bool CSiteManager::LoadPredefined(CSiteManagerXmlHandler& handler)
 {
 	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
-	if (defaultsDir.empty())
+	if (defaultsDir.empty()) {
 		return false;
+	}
 
-	wxString const name(defaultsDir.GetPath() + _T("fzdefaults.xml"));
+	std::wstring const name(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	CXmlFile file(name);
 
 	auto document = file.Load();
-	if (!document)
+	if (!document) {
 		return false;
+	}
 
 	auto element = document.child("Servers");
-	if (!element)
+	if (!element) {
 		return false;
+	}
 
 	if (!Load(element, handler)) {
 		return false;
@@ -580,14 +585,16 @@ wxString CSiteManager::AddServer(CServer server)
 	}
 
 	auto element = document.child("Servers");
-	if (!element)
-		return wxString();
+	if (!element) {
+		element = document.append_child("Servers");
+	}
 
 	std::list<wxString> names;
 	for (auto child = element.child("Server"); child; child = child.next_sibling("Server")) {
 		wxString name = GetTextElement(child, "Name");
-		if (name.empty())
+		if (name.empty()) {
 			continue;
+		}
 
 		names.push_back(name);
 	}
@@ -598,11 +605,13 @@ wxString CSiteManager::AddServer(CServer server)
 	for (;;) {
 		std::list<wxString>::const_iterator iter;
 		for (iter = names.begin(); iter != names.end(); ++iter) {
-			if (*iter == name)
+			if (*iter == name) {
 				break;
+			}
 		}
-		if (iter == names.end())
+		if (iter == names.end()) {
 			break;
+		}
 
 		name = _("New site") + wxString::Format(_T(" %d"), ++i);
 	}
@@ -614,8 +623,9 @@ wxString CSiteManager::AddServer(CServer server)
 	AddTextElement(xServer, name);
 
 	if (!file.Save(false)) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return wxString();
+		}
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName(), file.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
@@ -654,12 +664,14 @@ pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<s
 
 bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison)
 {
-	if (local_dir.empty() && remote_dir.empty())
+	if (local_dir.empty() && remote_dir.empty()) {
 		return false;
+	}
 
-	wxChar c = sitePath.empty() ? 0 : sitePath[0];
-	if (c != '0')
+	auto const c = sitePath[0];
+	if (c != '0') {
 		return false;
+	}
 
 	sitePath = sitePath.substr(1);
 
@@ -677,8 +689,9 @@ bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, cons
 	}
 
 	auto element = document.child("Servers");
-	if (!element)
+	if (!element) {
 		return false;
+	}
 
 	std::vector<std::wstring> segments;
 	if (!UnescapeSitePath(sitePath, segments)) {
@@ -695,35 +708,44 @@ bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, cons
 	// Bookmarks
 	pugi::xml_node insertBefore, bookmark;
 	for (bookmark = child.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
-		wxString old_name = GetTextElement_Trimmed(bookmark, "Name");
-		if (old_name.empty())
+		std::wstring old_name = GetTextElement_Trimmed(bookmark, "Name");
+		if (old_name.empty()) {
 			continue;
+		}
 
 		if (name == old_name) {
 			wxMessageBoxEx(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
 			return false;
 		}
-		if (name < old_name && !insertBefore)
+		if (name < old_name && !insertBefore) {
 			insertBefore = bookmark;
+		}
 	}
 
-	if (insertBefore)
+	if (insertBefore) {
 		bookmark = child.insert_child_before("Bookmark", insertBefore);
-	else
+	}
+	else {
 		bookmark = child.append_child("Bookmark");
-	AddTextElement(bookmark, "Name", name);
-	if (!local_dir.empty())
-		AddTextElement(bookmark, "LocalDir", local_dir);
-	if (!remote_dir.empty())
+	}
+	AddTextElement(bookmark, "Name", name.ToStdWstring());
+	if (!local_dir.empty()) {
+		AddTextElement(bookmark, "LocalDir", local_dir.ToStdWstring());
+	}
+	if (!remote_dir.empty()) {
 		AddTextElement(bookmark, "RemoteDir", remote_dir.GetSafePath());
-	if (sync)
-		AddTextElementRaw(bookmark, "SyncBrowsing", "1");
-	if (comparison)
-		AddTextElementRaw(bookmark, "DirectoryComparison", "1");
+	}
+	if (sync) {
+		AddTextElementUtf8(bookmark, "SyncBrowsing", "1");
+	}
+	if (comparison) {
+		AddTextElementUtf8(bookmark, "DirectoryComparison", "1");
+	}
 
 	if (!file.Save(false)) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return true;
+		}
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName(), file.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
@@ -735,8 +757,9 @@ bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, cons
 bool CSiteManager::ClearBookmarks(std::wstring sitePath)
 {
 	wxChar const c = sitePath.empty() ? 0 : sitePath[0];
-	if (c != '0')
+	if (c != '0') {
 		return false;
+	}
 
 	sitePath = sitePath.substr(1);
 
@@ -754,8 +777,9 @@ bool CSiteManager::ClearBookmarks(std::wstring sitePath)
 	}
 
 	auto element = document.child("Servers");
-	if (!element)
+	if (!element) {
 		return false;
+	}
 
 	std::vector<std::wstring> segments;
 	if (!UnescapeSitePath(sitePath, segments)) {
@@ -776,8 +800,9 @@ bool CSiteManager::ClearBookmarks(std::wstring sitePath)
 	}
 
 	if (!file.Save(false)) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return true;
+		}
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName(), file.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
