@@ -22,7 +22,6 @@ bool CStatusLineCtrl::m_initialized = false;
 CStatusLineCtrl::CStatusLineCtrl(CQueueView* pParent, const t_EngineData* const pEngineData, const wxRect& initialPosition)
 	: m_pParent(pParent)
 	, m_pEngineData(pEngineData)
-	, m_gcLastTimeStamp(wxDateTime::Now())
 {
 	wxASSERT(pEngineData);
 
@@ -83,11 +82,15 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 	int refresh = 0;
 	if (!m_data.IsOk() || rect.GetWidth() != m_data.GetWidth() || rect.GetHeight() != m_data.GetHeight()) {
 		m_mdc.reset();
-		m_data = wxBitmap(rect.GetWidth(), rect.GetHeight());
+
+		double sf = dc.GetContentScaleFactor();
+		m_data.CreateScaled(rect.width, rect.height, -1, sf);
+
 		m_mdc = std::make_unique<wxMemoryDC>(m_data);
 		// Use same layout direction as the DC which bitmap is drawn on.
 		// This avoids problem with mirrored characters on RTL locales.
 		m_mdc->SetLayoutDirection(dc.GetLayoutDirection());
+
 		refresh = 31;
 	}
 
@@ -119,7 +122,7 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 		}
 
 		int elapsed_milli_seconds = 0;
-		if (status_.started.empty()) {
+		if (!status_.started.empty()) {
 			elapsed = fz::datetime::now() - status_.started;
 			elapsed_milli_seconds = static_cast<int>(elapsed.get_milliseconds()); // Assume it doesn't overflow
 		}
@@ -130,9 +133,9 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 		}
 
 		if (COptions::Get()->GetOptionVal(OPTION_SPEED_DISPLAY))
-			rate = GetCurrentSpeed();
+			rate = GetMomentarySpeed();
 		else
-			rate = GetSpeed(elapsed_milli_seconds);
+			rate = GetAverageSpeed(elapsed_milli_seconds);
 
 		if (status_.totalSize > 0 && elapsed_milli_seconds >= 1000 && rate > 0) {
 			wxFileOffset r = status_.totalSize - status_.currentOffset;
@@ -196,11 +199,11 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 		wxCoord h = (rect.GetHeight() - m_textHeight) / 2;
 
 		if (refresh & 1) {
-			m_mdc->DrawRectangle(0, 0, m_fieldOffsets[0], rect.GetHeight());
+			m_mdc->DrawRectangle(0, 0, m_fieldOffsets[0], rect.GetHeight() + 1);
 			DrawRightAlignedText(*m_mdc, wxTimeSpan::Milliseconds(elapsed.get_milliseconds()).Format(_("%H:%M:%S elapsed")), m_fieldOffsets[0], h);
 		}
 		if (refresh & 2) {
-			m_mdc->DrawRectangle(m_fieldOffsets[0], 0, m_fieldOffsets[1] - m_fieldOffsets[0], rect.GetHeight());
+			m_mdc->DrawRectangle(m_fieldOffsets[0], 0, m_fieldOffsets[1] - m_fieldOffsets[0], rect.GetHeight() + 1);
 			if (left != -1) {
 				wxTimeSpan timeLeft(0, 0, left);
 				DrawRightAlignedText(*m_mdc, timeLeft.Format(_("%H:%M:%S left")), m_fieldOffsets[1], h);
@@ -209,14 +212,14 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 				DrawRightAlignedText(*m_mdc, _("--:--:-- left"), m_fieldOffsets[1], h);
 		}
 		if (refresh & 8) {
-			m_mdc->DrawRectangle(m_fieldOffsets[3], 0, rect.GetWidth() - m_fieldOffsets[3], rect.GetHeight());
+			m_mdc->DrawRectangle(m_fieldOffsets[3], 0, rect.GetWidth() - m_fieldOffsets[3], rect.GetHeight() + 1);
 			m_mdc->DrawText(bytes_and_rate, m_fieldOffsets[3], h);
 		}
 		if (refresh & 16) {
-			m_mdc->DrawRectangle(m_fieldOffsets[1], 0, m_fieldOffsets[2] - m_fieldOffsets[1], rect.GetHeight());
+			m_mdc->DrawRectangle(m_fieldOffsets[1], 0, m_fieldOffsets[2] - m_fieldOffsets[1], rect.GetHeight() + 1);
 		}
 		if (refresh & 4) {
-			m_mdc->DrawRectangle(m_fieldOffsets[2], 0, m_fieldOffsets[3] - m_fieldOffsets[2], rect.GetHeight());
+			m_mdc->DrawRectangle(m_fieldOffsets[2], 0, m_fieldOffsets[3] - m_fieldOffsets[2], rect.GetHeight() + 1);
 			if (bar_split != -1)
 				DrawProgressBar(*m_mdc, m_fieldOffsets[2], 1, rect.GetHeight() - 2, bar_split, permill);
 		}
@@ -240,7 +243,7 @@ void CStatusLineCtrl::ClearTransferStatus()
 		m_statusText = _("Waiting for transfer to be cancelled");
 		break;
 	case t_EngineData::connect:
-		m_statusText = wxString::Format(_("Connecting to %s"), m_pEngineData->lastServer.FormatServer());
+		m_statusText = wxString::Format(_("Connecting to %s"), m_pEngineData->lastServer.Format(ServerFormat::with_user_and_optional_port));
 		break;
 	default:
 		m_statusText = _("Transferring");
@@ -251,8 +254,8 @@ void CStatusLineCtrl::ClearTransferStatus()
 		m_transferStatusTimer.Stop();
 
 	m_past_data_count = 0;
-	m_gcLastOffset = -1;
-	m_gcLastSpeed = -1;
+
+	m_monentary_speed_data = monentary_speed_data();
 	Refresh(false);
 }
 
@@ -297,7 +300,7 @@ void CStatusLineCtrl::OnTimer(wxTimerEvent&)
 		m_transferStatusTimer.Stop();
 }
 
-void CStatusLineCtrl::DrawRightAlignedText(wxDC& dc, wxString text, int x, int y)
+void CStatusLineCtrl::DrawRightAlignedText(wxDC& dc, wxString const& text, int x, int y)
 {
 	wxCoord w, h;
 	dc.GetTextExtent(text, &w, &h);
@@ -324,8 +327,8 @@ void CStatusLineCtrl::DrawProgressBar(wxDC& dc, int x, int y, int height, int ba
 	if (bar_split && height > 2) {
 		// Draw pretty gradient
 
-		int greenmin = 128;
-		int greenmax = 255;
+		int greenmin = 160;
+		int greenmax = 223;
 		int colourCount = ((height + 1) / 2);
 
 		for (int i = 0; i < colourCount; ++i) {
@@ -336,7 +339,7 @@ void CStatusLineCtrl::DrawProgressBar(wxDC& dc, int x, int y, int height, int ba
 		}
 	}
 
-	dc.SetPen(*wxBLACK_PEN);
+	dc.SetPen(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNSHADOW));
 	dc.SetBrush(*wxTRANSPARENT_BRUSH);
 	dc.DrawRectangle(x, y, PROGRESSBAR_WIDTH, height);
 
@@ -354,7 +357,7 @@ void CStatusLineCtrl::DrawProgressBar(wxDC& dc, int x, int y, int height, int ba
 	dc.DrawText(text, x + PROGRESSBAR_WIDTH / 2 - w / 2, y + height / 2 - h / 2);
 }
 
-wxFileOffset CStatusLineCtrl::GetSpeed(int elapsed_milli_seconds)
+wxFileOffset CStatusLineCtrl::GetAverageSpeed(int elapsed_milli_seconds)
 {
 	if (status_.empty())
 		return -1;
@@ -384,35 +387,44 @@ wxFileOffset CStatusLineCtrl::GetSpeed(int elapsed_milli_seconds)
 	return ((status_.currentOffset - status_.startOffset - forget.offset) * 1000) / (elapsed_milli_seconds - forget.elapsed);
 }
 
-wxFileOffset CStatusLineCtrl::GetCurrentSpeed()
+wxFileOffset CStatusLineCtrl::GetMomentarySpeed()
 {
-	if (status_.empty())
+	if (status_.empty()) {
 		return -1;
-
-	const wxTimeSpan timeDiff( wxDateTime::UNow().Subtract(m_gcLastTimeStamp) );
-
-	if (timeDiff.GetMilliseconds().GetLo() <= 2000)
-		return m_gcLastSpeed;
-
-	m_gcLastTimeStamp = wxDateTime::UNow();
-
-	if (m_gcLastOffset < 0)
-		m_gcLastOffset = status_.startOffset;
-
-	const wxFileOffset fileOffsetDiff = status_.currentOffset - m_gcLastOffset;
-	m_gcLastOffset = status_.currentOffset;
-	if( fileOffsetDiff >= 0 ) {
-		m_gcLastSpeed = fileOffsetDiff * 1000 / timeDiff.GetMilliseconds().GetLo();
 	}
 
-	return m_gcLastSpeed;
+	if (!m_monentary_speed_data.last_update) {
+		m_monentary_speed_data.last_update = fz::monotonic_clock::now();
+		m_monentary_speed_data.last_offset = status_.startOffset;
+		return -1;
+	}
+
+	fz::duration const time_diff = fz::monotonic_clock::now() - m_monentary_speed_data.last_update;
+	if (time_diff.get_seconds() < 2) {
+		return m_monentary_speed_data.last_speed;
+	}
+
+	m_monentary_speed_data.last_update = fz::monotonic_clock::now();
+
+	if (m_monentary_speed_data.last_offset < 0) {
+		m_monentary_speed_data.last_offset = status_.startOffset;
+	}
+
+	wxFileOffset const fileOffsetDiff = status_.currentOffset - m_monentary_speed_data.last_offset;
+	m_monentary_speed_data.last_offset = status_.currentOffset;
+	if (fileOffsetDiff >= 0) {
+		m_monentary_speed_data.last_speed = fileOffsetDiff * 1000 / time_diff.get_milliseconds();
+	}
+
+	return m_monentary_speed_data.last_speed;
 }
 
-bool CStatusLineCtrl::Show(bool show /*=true*/)
+bool CStatusLineCtrl::Show(bool show)
 {
 	if (show) {
-		if (!m_transferStatusTimer.IsRunning())
+		if (!m_transferStatusTimer.IsRunning()) {
 			m_transferStatusTimer.Start(100);
+		}
 	}
 	else
 		m_transferStatusTimer.Stop();
