@@ -7,6 +7,8 @@
 #include "file_utils.h"
 #include "update_dialog.h"
 #include "themeprovider.h"
+#include "xrc_helper.h"
+#include "Options.h"
 
 #include <wx/animate.h>
 #include <wx/hyperlink.h>
@@ -15,6 +17,7 @@ BEGIN_EVENT_TABLE(CUpdateDialog, wxDialogEx)
 EVT_BUTTON(XRCID("ID_INSTALL"), CUpdateDialog::OnInstall)
 EVT_TIMER(wxID_ANY, CUpdateDialog::OnTimer)
 EVT_HYPERLINK(XRCID("ID_SHOW_DETAILS"), CUpdateDialog::ShowDetails)
+EVT_HYPERLINK(XRCID("ID_SHOW_DETAILS_DL"), CUpdateDialog::ShowDetailsDl)
 EVT_HYPERLINK(XRCID("ID_RETRY"), CUpdateDialog::Retry)
 EVT_HYPERLINK(XRCID("ID_DOWNLOAD_RETRY"), CUpdateDialog::Retry)
 EVT_BUTTON(XRCID("ID_DEBUGLOG"), CUpdateDialog::OnDebugLog)
@@ -77,9 +80,12 @@ int CUpdateDialog::ShowModal()
 	XRCCTRL(*this, "ID_WAIT_DOWNLOAD", wxAnimationCtrl)->SetAnimation(a);
 	XRCCTRL(*this, "ID_WAIT_DOWNLOAD", wxAnimationCtrl)->Play();
 
+	InitFooter();
+
 	Wrap();
 
-	XRCCTRL(*this, "ID_DETAILS", wxTextCtrl)->Hide();
+	xrc_call(*this, "ID_DETAILS", &wxTextCtrl::Hide);
+	xrc_call(*this, "ID_DETAILS_DL", &wxTextCtrl::Hide);
 
 	UpdaterState s = updater_.GetState();
 	UpdaterStateChanged( s, updater_.AvailableBuild() );
@@ -94,6 +100,55 @@ int CUpdateDialog::ShowModal()
 	return ret;
 }
 
+void MakeLinksFromTooltips(wxWindow& parent)
+{
+	// Iterates over all children.
+	// If tooltip is a URL, make the child have the hand cursor
+	// and launch URL in browser upon click.
+	wxWindowList children = parent.GetChildren();
+	for (auto child : children) {
+		wxString const tooltip = child->GetToolTipText();
+		if (tooltip.find(_T("http://")) == 0 || tooltip.find(_T("https://")) == 0) {
+			child->SetCursor(wxCURSOR_HAND);
+			child->Bind(wxEVT_LEFT_UP, [tooltip](wxEvent const&) { wxLaunchDefaultBrowser(tooltip); });
+		}
+		MakeLinksFromTooltips(*child);
+	}
+}
+
+void CUpdateDialog::InitFooter()
+{
+#if FZ_WINDOWS
+	if (CBuildInfo::GetBuildType() == _T("official") && !COptions::Get()->GetOptionVal(OPTION_DISABLE_UPDATE_FOOTER)) {
+		wxString const resources = updater_.GetResources();
+		if (!resources.empty()) {
+			wxLogNull null;
+
+			wxXmlResource res(wxXRC_NO_RELOADING);
+			InitHandlers(res);
+			if (res.Load(_T("blob:") + resources)) {
+				auto sizer = xrc_call(*this, "ID_NEWVERSION_PANEL", &wxPanel::GetSizer);
+				if (sizer) {
+					wxPanel* p{};
+					bool top{};
+					if ((p = res.LoadPanel(sizer->GetContainingWindow(), _T("ID_UPDATE_FOOTER")))) {
+						top = false;
+					}
+					else if ((p = res.LoadPanel(sizer->GetContainingWindow(), _T("ID_UPDATE_HEADER")))) {
+						top = true;
+					}
+
+					if (p) {
+						MakeLinksFromTooltips(*p);
+						sizer->Insert(top ? 0 : 1, p, wxSizerFlags().Align(wxALIGN_CENTER_HORIZONTAL).Border(top ? wxBOTTOM : wxTOP, 5));
+					}
+				}
+			}
+		}
+	}
+#endif
+}
+
 void CUpdateDialog::Wrap()
 {
 	wxPanel* parentPanel = XRCCTRL(*this, "ID_CONTENT", wxPanel);
@@ -205,6 +260,9 @@ void CUpdateDialog::UpdaterStateChanged( UpdaterState s, build const& v )
 		bool dlfail = s == UpdaterState::newversion && !v.url_.empty();
 		XRCCTRL(*this, "ID_DOWNLOAD_FAIL", wxStaticText)->Show(dlfail);
 		XRCCTRL(*this, "ID_DOWNLOAD_RETRY", wxHyperlinkCtrl)->Show(dlfail);
+		XRCCTRL(*this, "ID_SHOW_DETAILS_DL", wxHyperlinkCtrl)->Show(dlfail);
+
+		XRCCTRL(*this, "ID_DETAILS_DL", wxTextCtrl)->ChangeValue(updater_.GetLog());
 
 		XRCCTRL(*this, "ID_NEWVERSION_WEBSITE_TEXT", wxStaticText)->Show(manual && !dlfail);
 		XRCCTRL(*this, "ID_NEWVERSION_WEBSITE_TEXT_DLFAIL", wxStaticText)->Show(manual && dlfail);
@@ -221,8 +279,9 @@ void CUpdateDialog::OnInstall(wxCommandEvent&)
 	if( f.empty() ) {
 		return;
 	}
+	COptions::Get()->SetOption(OPTION_GREETINGRESOURCES, updater_.GetResources().ToStdWstring());
 #ifdef __WXMSW__
-	wxExecute(_T("\"") + f +  _T("\" /update"));
+	wxExecute(_T("\"") + f +  _T("\" /update /NCRC"));
 	wxWindow* p = parent_;
 	while( p->GetParent() ) {
 		p = p->GetParent();
@@ -262,6 +321,15 @@ void CUpdateDialog::ShowDetails(wxHyperlinkEvent&)
 	panels_[pagenames::failed]->Layout();
 }
 
+void CUpdateDialog::ShowDetailsDl(wxHyperlinkEvent&)
+{
+	XRCCTRL(*this, "ID_SHOW_DETAILS_DL", wxHyperlinkCtrl)->Hide();
+	XRCCTRL(*this, "ID_DETAILS_DL", wxTextCtrl)->Show();
+	XRCCTRL(*this, "ID_DETAILS_DL", wxTextCtrl)->SetMinSize(wxSize(-1, 200));
+
+	panels_[pagenames::newversion]->Layout();
+}
+
 void CUpdateDialog::Retry(wxHyperlinkEvent&)
 {
 	updater_.RunIfNeeded();
