@@ -6,24 +6,21 @@
 #include "updater.h"
 #include "Options.h"
 #include "file_utils.h"
-#include <wx/base64.h>
 #include <wx/tokenzr.h>
 #include <string>
 
+#ifdef __WXMSW__
+#include <wx/msw/registry.h>
+#endif
+
+#include <libfilezilla/file.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
 #include <string>
 
-// This is ugly but does the job
-#define SHA512_STANDALONE
-typedef unsigned int uint32;
-namespace {
-#include "../putty/int64.h"
-#include "../putty/sshsh512.c"
-}
+#include <nettle/sha2.h>
 
 BEGIN_EVENT_TABLE(CUpdater, wxEvtHandler)
-EVT_FZ_NOTIFICATION(wxID_ANY, CUpdater::OnEngineEvent)
 EVT_TIMER(wxID_ANY, CUpdater::OnTimer)
 END_EVENT_TABLE()
 
@@ -82,15 +79,14 @@ static CUpdater* instance = 0;
 
 CUpdater::CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context)
 	: state_(UpdaterState::idle)
-	, engine_(new CFileZillaEngine(engine_context))
+	, engine_(new CFileZillaEngine(engine_context, *this))
 {
 	AddHandler(parent);
-	engine_->Init(this);
 }
 
 void CUpdater::Init()
 {
-	if( state_ == UpdaterState::checking || state_ == UpdaterState::newversion_downloading ) {
+	if (state_ == UpdaterState::checking || state_ == UpdaterState::newversion_downloading) {
 		return;
 	}
 
@@ -152,7 +148,7 @@ bool CUpdater::LongTimeSinceLastCheck() const
 		return true;
 
 	fz::datetime lastCheck(lastCheckStr.ToStdWstring(), fz::datetime::utc);
-	if (!lastCheck.empty())
+	if (lastCheck.empty())
 		return true;
 
 	auto const span = fz::datetime::now() - lastCheck;
@@ -173,7 +169,7 @@ wxString CUpdater::GetUrl()
 	if (host.empty())
 		host = _T("unknown");
 
-	wxString version(PACKAGE_VERSION, wxConvLocal);
+	wxString version = CBuildInfo::GetVersion();
 	version.Replace(_T(" "), _T("%20"));
 
 	wxString url = wxString::Format(_T("https://update.filezilla-project.org/update.php?platform=%s&version=%s"), host, version);
@@ -188,6 +184,28 @@ wxString CUpdater::GetUrl()
 		url += _T("&osarch=64");
 	else
 		url += _T("&osarch=32");
+
+	// Add information about package
+	{
+		wxLogNull log;
+
+		// Installer always writes to 32bit section
+		auto key = std::make_unique<wxRegKey>(_T("HKEY_CURRENT_USER\\Software\\FileZilla Client"), wxRegKey::WOW64ViewMode_32);
+		if (!key->Exists()) {
+			// wxRegKey is sad, it doesn't even have a copy constructor.
+			key = std::make_unique<wxRegKey>(_T("HKEY_LOCAL_MACHINE\\Software\\FileZilla Client"), wxRegKey::WOW64ViewMode_32);
+		}
+
+		long updated{};
+		if (key->GetValueType(_T("Updated")) == wxRegKey::Type_Dword && key->QueryValue(_T("Updated"), &updated)) {
+			url += wxString::Format(_T("&updated=%d"), updated);
+		}
+
+		long package{};
+		if (key->GetValueType(_T("Package")) == wxRegKey::Type_Dword && key->QueryValue(_T("Package"), &package)) {
+			url += wxString::Format(_T("&package=%d"), package);
+		}
+	}
 #endif
 
 	wxString const cpuCaps = CBuildInfo::GetCPUCaps(',');
@@ -195,6 +213,14 @@ wxString CUpdater::GetUrl()
 		url += _T("&cpuid=") + cpuCaps;
 	}
 
+	wxString const lastVersion = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTVERSION);
+	if (lastVersion != CBuildInfo::GetVersion()) {
+		url += _T("&initial=1");
+	}
+	else {
+		url += _T("&initial=0");
+	}
+
 	return url;
 }
 
@@ -221,7 +247,7 @@ bool CUpdater::Run()
 	SetState(UpdaterState::checking);
 
 	m_use_internal_rootcert = true;
-	int res = Download(GetUrl(), wxString());
+	int res = Download(GetUrl(), std::wstring());
 
 	if (res != FZ_REPLY_WOULDBLOCK) {
 		SetState(UpdaterState::failed);
@@ -231,7 +257,7 @@ bool CUpdater::Run()
 	return state_ == UpdaterState::checking;
 }
 
-int CUpdater::Download(wxString const& url, wxString const& local_file)
+int CUpdater::Download(wxString const& url, std::wstring const& local_file)
 {
 	wxASSERT(pending_commands_.empty());
 	pending_commands_.clear();
@@ -262,8 +288,8 @@ bool CUpdater::CreateConnectCommand(wxString const& url)
 {
 	CServer s;
 	CServerPath path;
-	wxString error;
-	if( !s.ParseUrl( url, 0, wxString(), wxString(), error, path ) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS) ) {
+	std::wstring error;
+	if (!s.ParseUrl(url.ToStdWstring(), 0, std::wstring(), std::wstring(), error, path) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS)) {
 		return false;
 	}
 
@@ -271,26 +297,31 @@ bool CUpdater::CreateConnectCommand(wxString const& url)
 	return true;
 }
 
-bool CUpdater::CreateTransferCommand(wxString const& url, wxString const& local_file)
+bool CUpdater::CreateTransferCommand(wxString const& url, std::wstring const& local_file)
 {
 	CFileTransferCommand::t_transferSettings transferSettings;
 
 	CServer s;
 	CServerPath path;
-	wxString error;
-	if( !s.ParseUrl( url, 0, wxString(), wxString(), error, path ) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS) ) {
+	std::wstring error;
+	if (!s.ParseUrl(url.ToStdWstring(), 0, std::wstring(), std::wstring(), error, path) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS)) {
 		return false;
 	}
-	wxString file = path.GetLastSegment();
+	std::wstring file = path.GetLastSegment();
 	path = path.GetParent();
 
 	pending_commands_.emplace_back(new CFileTransferCommand(local_file, path, file, true, transferSettings));
 	return true;
 }
 
-void CUpdater::OnEngineEvent(wxFzEvent& event)
+void CUpdater::OnEngineEvent(CFileZillaEngine* engine)
 {
-	if (!engine_ || engine_ != event.engine_)
+	CallAfter(&CUpdater::DoOnEngineEvent, engine);
+}
+
+void CUpdater::DoOnEngineEvent(CFileZillaEngine* engine)
+{
+	if (!engine_ || engine_ != engine)
 		return;
 
 	std::unique_ptr<CNotification> notification;
@@ -317,14 +348,12 @@ void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notificatio
 				auto & certNotification = static_cast<CCertificateNotification &>(*pData.get());
 				if (m_use_internal_rootcert) {
 					auto certs = certNotification.GetCertificates();
-					if( certs.size() > 1 ) {
-						auto ca = certs.back();
+					if (certs.size() > 1) {
+						auto const& ca = certs.back();
+						std::vector<uint8_t> ca_data = ca.GetRawData();
 
-						unsigned int ca_data_length{};
-						unsigned char const* ca_data = ca.GetRawData(ca_data_length);
-
-						wxMemoryBuffer updater_root = wxBase64Decode(s_update_cert, wxNO_LEN, wxBase64DecodeMode_SkipWS);
-						if( ca_data_length == updater_root.GetDataLen() && !memcmp(ca_data, updater_root.GetData(), ca_data_length) ) {
+						std::string updater_root = fz::base64_decode(s_update_cert);
+						if (ca_data.size() == updater_root.size() && !memcmp(&ca_data[0], updater_root.c_str(), ca_data.size()) ) {
 							certNotification.m_trusted = true;
 						}
 					}
@@ -364,7 +393,7 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 	}
 	else if (!version_information_.available_.url_.empty()) {
 
-		wxString const temp = GetTempFile();
+		std::wstring const temp = GetTempFile();
 		wxString const local_file = GetLocalFile(version_information_.available_, true);
 		if (!local_file.empty() && fz::local_filesys::get_file_type(fz::to_native(local_file)) != fz::local_filesys::unknown) {
 			local_file_ = local_file;
@@ -384,7 +413,7 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 				if (size >= 0 && size >= version_information_.available_.size_) {
 					s = ProcessFinishedDownload();
 				}
-				else if( !can_download || Download( version_information_.available_.url_, GetTempFile() ) != FZ_REPLY_WOULDBLOCK ) {
+				else if (!can_download || Download(version_information_.available_.url_, temp) != FZ_REPLY_WOULDBLOCK ) {
 					s = UpdaterState::newversion;
 				}
 			}
@@ -399,7 +428,7 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 
 void CUpdater::ProcessOperation(COperationNotification const& operation)
 {
-	if( state_ != UpdaterState::checking && state_ != UpdaterState::newversion_downloading ) {
+	if (state_ != UpdaterState::checking && state_ != UpdaterState::newversion_downloading) {
 		return;
 	}
 
@@ -425,7 +454,8 @@ void CUpdater::ProcessOperation(COperationNotification const& operation)
 			s = UpdaterState::newversion;
 		}
 	}
-	else if( state_ == UpdaterState::checking ) {
+	else if (state_ == UpdaterState::checking) {
+		COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTVERSION, CBuildInfo::GetVersion());
 		s = ProcessFinishedData(true);
 	}
 	else {
@@ -438,11 +468,11 @@ UpdaterState CUpdater::ProcessFinishedDownload()
 {
 	UpdaterState s = UpdaterState::newversion;
 
-	wxString const temp = GetTempFile();
-	if( temp.empty() ) {
+	std::wstring const temp = GetTempFile();
+	if (temp.empty()) {
 		s = UpdaterState::newversion;
 	}
-	else if( !VerifyChecksum( temp, version_information_.available_.size_, version_information_.available_.hash_ ) ) {
+	else if (!VerifyChecksum(temp, version_information_.available_.size_, version_information_.available_.hash_)) {
 		wxLogNull log;
 		wxRemoveFile(temp);
 		s = UpdaterState::newversion;
@@ -450,7 +480,7 @@ UpdaterState CUpdater::ProcessFinishedDownload()
 	else {
 		s = UpdaterState::newversion_ready;
 
-		wxString local_file = GetLocalFile( version_information_.available_, false );
+		wxString local_file = GetLocalFile(version_information_.available_, false);
 
 		wxLogNull log;
 		if (local_file.empty() || !wxRenameFile( temp, local_file, false ) ) {
@@ -511,7 +541,7 @@ void CUpdater::ProcessData(CDataNotification& dataNotification)
 		log_ += wxString::Format(_T("ProcessData %d\n"), len);
 	}
 
-	if( raw_version_information_.size() + len > 131072 ) {
+	if( raw_version_information_.size() + len > 0x40000) {
 		log_ += _("Received version information is too large");
 		engine_->Cancel();
 		SetState(UpdaterState::failed);
@@ -555,31 +585,40 @@ void CUpdater::ParseData()
 		}
 
 		wxStringTokenizer tokens(line, _T(" \t\n"),  wxTOKEN_STRTOK);
-		if( !tokens.CountTokens() ) {
+		if (!tokens.CountTokens()) {
 			// After empty line, changelog follows
-			version_information_.changelog = raw_version_information;
-			version_information_.changelog.Trim(true);
-			version_information_.changelog.Trim(false);
+			version_information_.changelog_ = raw_version_information;
+			version_information_.changelog_.Trim(true);
+			version_information_.changelog_.Trim(false);
 
-			if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
-				log_ += wxString::Format(_T("Changelog: %s\n"), version_information_.changelog);
+			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				log_ += wxString::Format(_T("Changelog: %s\n"), version_information_.changelog_);
 			}
 			break;
 		}
 
-		if( tokens.CountTokens() != 2 && tokens.CountTokens() != 6 ) {
-			if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
-				log_ += wxString::Format(_T("Skipping line with %d tokens\n"), static_cast<int>(tokens.CountTokens()));
+		wxString const type = tokens.GetNextToken();
+		if (type == _T("resources")) {
+			if (tokens.HasMoreTokens()) {
+				if (UpdatableBuild()) {
+					version_information_.resources_ = tokens.GetNextToken();
+				}
+			}
+			continue;
+		}
+
+		if (tokens.CountTokens() != 1 && tokens.CountTokens() != 5) {
+			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				log_ += wxString::Format(_T("Skipping line with %d tokens\n"), static_cast<int>(tokens.CountTokens() + 1));
 			}
 			continue;
 		}
 
-		wxString type = tokens.GetNextToken();
 		wxString versionOrDate = tokens.GetNextToken();
 
 		if (type == _T("nightly")) {
 			fz::datetime nightlyDate(versionOrDate.ToStdWstring(), fz::datetime::utc);
-			if (!nightlyDate.empty()) {
+			if (nightlyDate.empty()) {
 				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 					log_ += _T("Could not parse nightly date\n");
 				}
@@ -587,7 +626,7 @@ void CUpdater::ParseData()
 			}
 
 			fz::datetime buildDate = CBuildInfo::GetBuildDate();
-			if (!buildDate.empty() || !nightlyDate.empty() || nightlyDate <= buildDate) {
+			if (buildDate.empty() || nightlyDate.empty() || nightlyDate <= buildDate) {
 				if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
 					log_ += _T("Nightly isn't newer\n");
 				}
@@ -651,7 +690,7 @@ void CUpdater::ParseData()
 
 	version_information_.update_available();
 
-	COptions::Get()->SetOption( OPTION_UPDATECHECK_NEWVERSION, raw_version_information_ );
+	COptions::Get()->SetOption(OPTION_UPDATECHECK_NEWVERSION, raw_version_information_.ToStdWstring());
 }
 
 void CUpdater::OnTimer(wxTimerEvent&)
@@ -670,19 +709,18 @@ bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const
 		return false;
 	}
 
-	SHA512_State state;
-	SHA512_Init(&state);
+	sha512_ctx state;
+	sha512_init(&state);
 
 	{
-		wxLogNull null;
-		wxFile f;
-		if (!f.Open(file)) {
+		fz::file f(fz::to_native(file), fz::file::reading);
+		if (!f.opened()) {
 			return false;
 		}
-		char buffer[65536];
-		ssize_t read;
-		while ((read = f.Read(buffer, sizeof(buffer))) > 0) {
-			SHA512_Bytes(&state, buffer, read);
+		unsigned char buffer[65536];
+		int64_t read;
+		while ((read = f.read(buffer, sizeof(buffer))) > 0) {
+			sha512_update(&state, static_cast<size_t>(read), buffer);
 		}
 		if (read < 0) {
 			return false;
@@ -690,7 +728,7 @@ bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const
 	}
 
 	unsigned char raw_digest[64];
-	SHA512_Final(&state, raw_digest);
+	sha512_digest(&state, 64, raw_digest);
 
 	wxString digest;
 	for (unsigned int i = 0; i < sizeof(raw_digest); ++i) {
@@ -717,7 +755,7 @@ bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const
 	return true;
 }
 
-wxString CUpdater::GetTempFile() const
+std::wstring CUpdater::GetTempFile() const
 {
 	wxASSERT( !version_information_.available_.hash_.empty() );
 	wxString ret = wxFileName::GetTempDir();
@@ -729,7 +767,7 @@ wxString CUpdater::GetTempFile() const
 		ret += _T("fzupdate_") + version_information_.available_.hash_.Left(16) + _T(".tmp");
 	}
 
-	return ret;
+	return ret.ToStdWstring();
 }
 
 wxString CUpdater::GetFilename( wxString const& url) const
@@ -811,9 +849,9 @@ int64_t CUpdater::BytesDownloaded() const
 			ret = fz::local_filesys::get_size(fz::to_native(local_file_));
 		}
 	}
-	else if( state_ == UpdaterState::newversion_downloading ) {
-		wxString const temp = GetTempFile();
-		if( !temp.empty() ) {
+	else if (state_ == UpdaterState::newversion_downloading) {
+		std::wstring const temp = GetTempFile();
+		if (!temp.empty()) {
 			ret = fz::local_filesys::get_size(fz::to_native(temp));
 		}
 	}
