@@ -5,7 +5,7 @@
 
 #include <wx/timer.h>
 
-class CWelcomeDialog : public wxDialogEx
+class CWelcomeDialog final : public wxDialogEx
 {
 public:
 	CWelcomeDialog() {}
@@ -14,6 +14,9 @@ public:
 	bool Run(wxWindow* parent, bool force = false, bool delay = false);
 
 protected:
+
+	void InitFooter(wxString const& resources);
+
 	wxTimer m_delayedShowTimer;
 
 	DECLARE_EVENT_TABLE()
