@@ -16,24 +16,22 @@ class CXmlFile final
 {
 public:
 	CXmlFile() = default;
-	explicit CXmlFile(const wxString& fileName, wxString const& root = wxString());
-
-	~CXmlFile();
+	explicit CXmlFile(std::wstring const& fileName, std::string const& root = std::string());
 
 	CXmlFile(CXmlFile const&) = delete;
 	CXmlFile& operator=(CXmlFile const&) = delete;
 
 	pugi::xml_node CreateEmpty();
 
-	wxString GetFileName() const { return m_fileName; }
-	void SetFileName(const wxString& name);
+	std::wstring GetFileName() const { return m_fileName; }
+	void SetFileName(std::wstring const& name);
 
 	bool HasFileName() const { return !m_fileName.empty(); }
 
 	// Sets error description on failure
-	pugi::xml_node Load();
+	pugi::xml_node Load(bool overwriteInvalid = false);
 
-	wxString GetError() const { return m_error; }
+	std::wstring GetError() const { return m_error; }
 	size_t GetRawDataLength();
 	void GetRawDataHere(char* p, size_t size); // p has to big enough to hold at least GetRawDataLength() bytes
 
@@ -50,11 +48,11 @@ public:
 
 	bool IsFromFutureVersion() const;
 protected:
-	wxString GetRedirectedName() const;
+	std::wstring GetRedirectedName() const;
 
 	// Opens the specified XML file if it exists or creates a new one otherwise.
 	// Returns 0 on error.
-	bool GetXmlFile(wxString const& file);
+	bool GetXmlFile(std::wstring const& file);
 
 	// Sets version and platform in root element
 	void UpdateMetadata();
@@ -63,43 +61,44 @@ protected:
 	bool SaveXmlFile();
 
 	fz::datetime m_modificationTime;
-	wxString m_fileName;
+	std::wstring m_fileName;
 	pugi::xml_document m_document;
 	pugi::xml_node m_element;
 
-	wxString m_error;
+	std::wstring m_error;
 
-	wxString m_rootName{_T("FileZilla3")};
+	std::string m_rootName{"FileZilla3"};
 };
 
-// Convert the given utf-8 string into wxString
-wxString ConvLocal(const char *value);
+void SetTextAttribute(pugi::xml_node node, char const* name, std::string const& value);
+void SetTextAttribute(pugi::xml_node node, char const* name, std::wstring const& value);
+void SetTextAttributeUtf8(pugi::xml_node node, char const* name, std::string const& utf8);
+std::wstring GetTextAttribute(pugi::xml_node node, char const* name);
 
-void SetTextAttribute(pugi::xml_node node, const char* name, const wxString& value);
-wxString GetTextAttribute(pugi::xml_node node, const char* name);
+int GetAttributeInt(pugi::xml_node node, char const* name);
+void SetAttributeInt(pugi::xml_node node, char const* name, int value);
 
-int GetAttributeInt(pugi::xml_node node, const char* name);
-void SetAttributeInt(pugi::xml_node node, const char* name, int value);
-
-pugi::xml_node FindElementWithAttribute(pugi::xml_node node, const char* element, const char* attribute, const char* value);
+pugi::xml_node FindElementWithAttribute(pugi::xml_node node, char const* element, char const* attribute, char const* value);
 
 // Add a new child element with the specified name and value to the xml document
-pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, const wxString& value, bool overwrite = false);
-void AddTextElement(pugi::xml_node node, const char* name, int64_t value, bool overwrite = false);
-pugi::xml_node AddTextElementRaw(pugi::xml_node node, const char* name, const char* value, bool overwrite = false);
+pugi::xml_node AddTextElement(pugi::xml_node node, char const* name, std::string const& value, bool overwrite = false);
+pugi::xml_node AddTextElement(pugi::xml_node node, char const* name, std::wstring const& value, bool overwrite = false);
+void AddTextElement(pugi::xml_node node, char const* name, int64_t value, bool overwrite = false);
+pugi::xml_node AddTextElementUtf8(pugi::xml_node node, char const* name, std::string const& value, bool overwrite = false);
 
 // Set the current element's text value
-void AddTextElement(pugi::xml_node node, const wxString& value);
+void AddTextElement(pugi::xml_node node, std::string const& value);
+void AddTextElement(pugi::xml_node node, std::wstring const& value);
 void AddTextElement(pugi::xml_node node, int64_t value);
-void AddTextElementRaw(pugi::xml_node node, const char* value);
+void AddTextElementUtf8(pugi::xml_node node, std::string const& value);
 
 // Get string from named child element
-wxString GetTextElement(pugi::xml_node node, const char* name);
-wxString GetTextElement_Trimmed(pugi::xml_node node, const char* name);
+std::wstring GetTextElement(pugi::xml_node node, const char* name);
+std::wstring GetTextElement_Trimmed(pugi::xml_node node, const char* name);
 
 // Get string from current element
-wxString GetTextElement(pugi::xml_node node);
-wxString GetTextElement_Trimmed(pugi::xml_node node);
+std::wstring GetTextElement(pugi::xml_node node);
+std::wstring GetTextElement_Trimmed(pugi::xml_node node);
 
 // Get (64-bit) integer from named element
 int64_t GetTextElementInt(pugi::xml_node node, const char* name, int defValue = 0);
