@@ -10,6 +10,8 @@
 #include <limits.h>
 #include <assert.h>
 #include <time.h>
+#include <errno.h>
+#include <string.h>
 
 #include "putty.h"
 #include "ssh.h"
@@ -220,8 +222,6 @@ int main(int argc, char **argv)
 	}
 	else if (!strcmp(cmd, "fingerprint")) {
 	    const char* error = 0;
-	    void* blob = NULL;
-	    int bloblen = 0;
 
 	    if (!fingerprint) {
 		if (ssh2key) {
