@@ -72,8 +72,7 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
 	"adding the key to the cache, enter \"n\".\n"
 	"If you do not trust this host, press Return to abandon the\n"
 	"connection.\n"
-	"Store key in cache? (y/n) ";*/
-
+	"Store key in cache? (y/n) ";
     static const char wrongmsg_batch[] =
 	"WARNING - POTENTIAL SECURITY BREACH!\n"
 	"The server's host key does not match the one PuTTY has\n"
@@ -84,7 +83,7 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
 	"The new %s key fingerprint is:\n"
 	"%s\n"
 	"Connection abandoned.\n";
-    /*static const char wrongmsg[] =
+    static const char wrongmsg[] =
 	"WARNING - POTENTIAL SECURITY BREACH!\n"
 	"The server's host key does not match the one PuTTY has\n"
 	"cached in the registry. This means that either the\n"
@@ -114,16 +113,7 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
     if (ret == 0)		       /* success - key matched OK */
 	return 1;
 
-    if (ret == 2) {		       /* key was different */
-	if (console_batch_mode) {
-	    fprintf(stderr, wrongmsg_batch, keytype, fingerprint);
-            return 0;
-	}
-	fzprintf_raw(sftpRequest, "%d%s\n%d\n%s\n", (int)sftpReqHostkeyChanged, host, port, fingerprint);
-    }
-    if (ret == 1) {		       /* key was absent */
-	fzprintf_raw(sftpRequest, "%d%s\n%d\n%s\n", (int)sftpReqHostkey, host, port, fingerprint);
-    }
+    fzprintf_raw((ret == 1) ? sftpAskHostkey : sftpAskHostkeyChanged, "%s\n%d\n", host, port);
 
     hin = GetStdHandle(STD_INPUT_HANDLE);
     GetConsoleMode(hin, &savemode);
@@ -157,6 +147,52 @@ int askalg(void *frontend, const char *algtype, const char *algname,
     return 0;
 }
 
+int askhk(void *frontend, const char *algname, const char *betteralgs,
+          void (*callback)(void *ctx, int result), void *ctx)
+{
+    HANDLE hin;
+    DWORD savemode, i;
+
+    /*static const char msg[] =
+	"The first host key type we have stored for this server\n"
+	"is %s, which is below the configured warning threshold.\n"
+	"The server also provides the following types of host key\n"
+        "above the threshold, which we do not have stored:\n"
+        "%s\n"
+	"Continue with connection? (y/n) ";*/
+    static const char msg_batch[] =
+	"The first host key type we have stored for this server\n"
+	"is %s, which is below the configured warning threshold.\n"
+	"The server also provides the following types of host key\n"
+        "above the threshold, which we do not have stored:\n"
+        "%s\n"
+	"Connection abandoned.\n";
+    static const char abandoned[] = "Connection abandoned.";
+
+    char line[32];
+
+    if (console_batch_mode) {
+	fprintf(stderr, msg_batch, algname, betteralgs);
+	return 0;
+    }
+
+    fzprintf_raw(sftpAskHostkeyBetteralg, "%s\n%s\n", algname, betteralgs);
+
+    hin = GetStdHandle(STD_INPUT_HANDLE);
+    GetConsoleMode(hin, &savemode);
+    SetConsoleMode(hin, (savemode |
+			 ENABLE_PROCESSED_INPUT | ENABLE_LINE_INPUT));
+    ReadFile(hin, line, sizeof(line) - 1, &i, NULL);
+    SetConsoleMode(hin, savemode);
+
+    if (line[0] == 'y' || line[0] == 'Y') {
+	return 1;
+    } else {
+	fzprintf(sftpError, abandoned);
+	return 0;
+    }
+}
+
 /*
  * Ask whether to wipe a session log file before writing to it.
  * Returns 2 for wipe, 1 for append, 0 for cancel (don't log).
@@ -313,13 +349,13 @@ int console_get_userpass_input(prompts_t *p,
      */
     /* We only print the `name' caption if we have to... */
     if (p->name)
-	fzprintf_raw_untrusted(sftpRequestPreamble, p->name);
+	fzprintf_raw_untrusted(sftpRequestPreamble, "%s", p->name);
     else
 	fzprintf_raw_untrusted(sftpRequestPreamble, "");
 
     /* ...but we always print any `instruction'. */
     if (p->instruction)
-	fzprintf_raw_untrusted(sftpRequestInstruction, p->instruction);
+	fzprintf_raw_untrusted(sftpRequestInstruction, "%s", p->instruction);
     else
 	fzprintf_raw_untrusted(sftpRequestInstruction, "");
 
@@ -337,7 +373,7 @@ int console_get_userpass_input(prompts_t *p,
 	//    newmode |= ENABLE_ECHO_INPUT;
 	SetConsoleMode(hin, newmode);
 
-	fzprintf_raw_untrusted(sftpRequest, "%d%s\n", (int)sftpReqPassword, pr->prompt);
+	fzprintf_raw_untrusted(sftpAskPassword, "%s", pr->prompt);
 	
         len = 0;
         while (1) {
